/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import examples.invoice.persistclass.ProdUnits;
import examples.invoice.persistclass.ProdUnitsHelper;
import examples.invoice.persistclass.MapHelper;
import examples.invoice.persistclass.ProductHelper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.io.InputStream;
import javax.resource.ResourceException;

/**
 * @author	P. Dechamboux
 */
public class ProdUnitsAppli implements ShellExtent {
	private static ProdUnitsAppli singleton = null;
	private static ProductAppli productAppli;
	private ProductHelper productHelper;
	private ProdUnitsHelper mapHelper;
	private BasicShell basicShell;

	public static ProdUnitsAppli getInstance() {
		if (singleton == null) {
			productAppli = ProductAppli.getInstance();
			singleton = new ProdUnitsAppli();
		}
		return singleton;
	}

	private ProdUnitsAppli() {
		productHelper = (ProductHelper) productAppli.getHelper();
		mapHelper = new ProdUnitsHelper(productHelper);
		basicShell = BasicShell.getInstance();
	}

	public boolean interpCmd(String cmd) {
		if (cmd.equals("cu") || cmd.equals("createprodunits"))
			createProdUnits();
		else if (cmd.equals("lu") || cmd.equals("listprodunits"))
			listProdUnits();
		else if (cmd.equals("du") || cmd.equals("deleteprodunits"))
			deleteProdUnits();
		else if (productAppli.interpCmd(cmd))
			return true;
		else
			return false;
		return true;
	}

	public void help() {
		productAppli.help();
		System.out.println("\t\t- createprodunits | cu : creates a new ProdUnits using the active mapper.");
		System.out.println("\t\t- listprodunits | lu : lists existing ProdUnitses using the active mapper.");
		System.out.println("\t\t- deleteprodunits | du : deletes an existing ProdUnits using the active mapper.");
	}

	public MapHelper getHelper() {
		return mapHelper;
	}

	public static void main(String[] args) {
		InputStream in = System.in;
		try {
			switch (args.length) {
			case 2:
				in = ObsInputStream.observes(args[1]);
			case 1:
				BasicShell.getInstance().shellLoop(args[0], getInstance(), in);
				break;
			default:
				System.err.println("Usage: AddressAppli prop_file_name [jormexsh_cmdfile]");
				System.exit(-1);
			}
		} catch (Exception e) {
			System.err.println("Exit with exception: " + e.getMessage());
			do {
				e.printStackTrace();
				if (e instanceof PException)
					e = ((PException) e).getNestedException();
				if (e instanceof ResourceException)
					e = ((ResourceException) e).getLinkedException();
				else
					e = null;
			} while (e != null);
		}
	}

	private void createProdUnits() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		if (!JormInit.getInstance().isMapped(basicShell.activeMapperName)) {
			basicShell.errorMsg("requires a mapped mapper (run <map> command before)");
			return;
		}
		try {
			String p, q;
			p = basicShell.promptString("\tProduct: ", 200);
			q = basicShell.promptString("\tQuantity: ", 10);
			if ((p == null) || (q == null) || (p.length() == 0) || (q.length() == 0)) {
				basicShell.errorMsg("wrong product or quantity (mandatory)");
				return;
			}
			ProdUnits ad = mapHelper.createProdUnits(basicShell.activeMapper,
												 p, Integer.parseInt(q));
			System.out.println("\tCreated ProdUnits:\n\t\t" + ad.toPrintable());
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void listProdUnits() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		if (!JormInit.getInstance().isMapped(basicShell.activeMapperName)) {
			basicShell.errorMsg("requires a mapped mapper (run <map> command before)");
			return;
		}
		try {
			Iterator adit = mapHelper.listProdUnits(basicShell.activeMapper);
			System.out.println("\tExisting ProdUnits:");
			while (adit.hasNext()) {
				PName pn = (PName) adit.next();
				ProdUnits ad = mapHelper.getProdUnits(basicShell.activeMapper, pn);
				System.out.println("\t\t" + ad.toPrintable());
			}
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void deleteProdUnits() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		if (!JormInit.getInstance().isMapped(basicShell.activeMapperName)) {
			basicShell.errorMsg("requires a mapped mapper (run <map> command before)");
			return;
		}
		String sid;
		sid = basicShell.promptString("\tID: ", 20);
		try {
			if (mapHelper.deleteProdUnits(basicShell.activeMapper, Long.parseLong(sid)))
				System.out.println("ProdUnits deleted.");
			else
				basicShell.errorMsg("cannot delete ProdUnits");
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}
}
