/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.api.ClassRef;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.rdb.api.RdbMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbClassMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbTable;
import org.objectweb.jorm.metainfo.rdb.api.RdbClassMultiMapping;
import org.objectweb.jorm.metainfo.rdb.api.RdbExternalTable;
import org.objectweb.jorm.metainfo.rdb.api.RdbJoin;
import org.objectweb.jorm.metainfo.rdb.api.RdbGenClassMapping;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * This example builds the jorm meta information for a multivalued relation
 * between two persistent object. The A objects has a multivalued reference
 * 'myB' to a B object, whereas the B has a reference 'myA' to the A object.
 * This is a one-many relationship.
 * In a mapping point of view the objects are stored on sql table with the
 * following schema
 * <!--
 * |---------|---------|  *   |---------|---------|---------|
 * | pka_col |  f1_col |  *   | pkb_col |  f2_col | fka_col |
 * |---------|---------|  *   |---------|---------|---------|
 * |   ...   |   ...   |  *   |   ...   |   ...   |   ...   |
 * |---------|---------|  *   |---------|---------|---------|
 *          table_A       *          table_B
 * -->
 * <table border=0>
 * <TR><TD>table_A</TD><TD>table_B</TD></TR>
 * <TR>
 * <TD><TABLE>
 * <TR><TD>pka_col</TD><TD>f1_col</TD></TR>
 * <TR><TD>...</TD><TD>...</TD></TR>
 * </TABLE></TD>
 * <TD><TABLE>
 * <TR><TD>pkb_col</TD><TD>f2_col</TD><TD>fka_col</TD></TR>
 * <TR><TD>...</TD><TD>...</TD><TD>...</TD></TR>
 * </TABLE></TD>
 * </TR>
 * </table>
 *
 * S.Chassande-Barrioz
 */

public class OneManyBi extends JormcRunner {

    public final static String PROJECT_NAME = "jormExamples";
    public final static String A_CLASS_NAME = "org.objectweb.jorm.examples.omb.A";
    public final static String B_CLASS_NAME = "org.objectweb.jorm.examples.omb.B";


    public static void main(String[] args) {
        try {
            OneManyBi omb = new OneManyBi(args.length == 0 ? "./" : args[0]);
            logger.log(BasicLevel.INFO, "Example OneManyBi");
            omb.createMIofA();
            omb.createMIofB();
            omb.generate();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "", getInner(e));
        }
    }
    public OneManyBi(String output) throws PException {
        super(PROJECT_NAME, output);
    }

    public void createMIofA() throws PException {
        logger.log(BasicLevel.INFO, "Create the meta information of the A class");
        Class aclass = compiler.getMIManager().createClass(A_CLASS_NAME);

        PrimitiveElement ida =
                aclass.createPrimitiveElement("ida", PTypeSpace.LONG);
        PrimitiveElement f1 =
                        aclass.createPrimitiveElement("f1", PTypeSpace.STRING);

        // class identifier
        NameDef aNameDef = aclass.createNameDef();
        aNameDef.setFieldName(ida.getName());

        // mulitvalued reference to B
        GenClassRef bgcr = aclass.createGenClassRef("myB", "Set");
        //How the gen class is referenced
        NameDef myBNameDef = bgcr.createRefNameDef();
        myBNameDef.setFieldName(ida.getName());

        //how the gen clas is identified
        ScalarField fka = bgcr.createHiddenField("fka", PTypeSpace.LONG);
        NameDef  gcrIdNd = bgcr.createIdNameDef();
        gcrIdNd.setFieldName("fka");

        //element of the gen class
        Class bclass = compiler.getMIManager().createClass(B_CLASS_NAME);
        ClassRef elemCr = bgcr.createClassRef(bclass);
        ScalarField idb = bgcr.createHiddenField("idb", PTypeSpace.LONG);
        NameDef gcrElemNd = elemCr.createRefNameDef();
        gcrElemNd.setFieldName("idb");


        // ----------------- Mapping for A -----------------
        ClassProject acp = aclass.createClassProject(PROJECT_NAME);
        RdbMapping am = (RdbMapping) acp.createMapping("rdb");

        RdbClassMapping acm = am.createClassMapping("to-table");
        RdbTable tableA = acm.createRdbTable("table_A");
        //Identifier of the A class
        acm.createIdentifierMapping(aNameDef);

        // Mapping rule of the reference myB
        acm.createReferenceMapping("embedded-target-reference", myBNameDef);

        // Mapping of the ida field over the pka_col
        tableA.createPrimitiveElementMapping(ida, "pka_col", null, true);
        // Mapping of the f1 field over the f1_col
        tableA.createPrimitiveElementMapping(f1, "f1_col", null, true);


        //Mapping of the reference to B (multivalued)
        RdbGenClassMapping gccm =
                am.createGenClassMapping("embedded-target-objects", bgcr);
        //The table of the generic class is also the table of B
        RdbTable tableGC = gccm.createRdbTable("table_B");
        tableGC.setReadOnly(true); //optimisaton to avoid writings
        tableGC.createPrimitiveElementMapping(idb, "pkb_col", null, false);
        tableGC.createPrimitiveElementMapping(fka, "fka_col", null, false);
        gccm.createIdentifierMapping(gcrIdNd);
        gccm.createReferenceMapping("embedded-target-object", gcrElemNd);
    }

    public void createMIofB() throws PException {
        logger.log(BasicLevel.INFO, "Create the meta information of the B class");
        Class bclass = compiler.getMIManager().createClass(B_CLASS_NAME);
        PrimitiveElement idb =
                bclass.createPrimitiveElement("idb", PTypeSpace.LONG);
        PrimitiveElement f2 =
                bclass.createPrimitiveElement("f2", PTypeSpace.STRING);

        // pk simple
        NameDef bNameDef = bclass.createNameDef();
        bNameDef.setFieldName("idb");

        // reference sur A
        ScalarField ida = bclass.createHiddenField("ida", PTypeSpace.LONG);
        Class aclass = compiler.getMIManager().createClass(A_CLASS_NAME);
        ClassRef acr = bclass.createClassRef("myA", aclass);
        NameDef myANameDef = acr.createRefNameDef();
        myANameDef.setFieldName("ida");

        // ----------------- Mapping for B -----------------

        ClassProject bcp = bclass.createClassProject(PROJECT_NAME);
        RdbMapping bm = (RdbMapping) bcp.createMapping("rdb");

        RdbClassMultiMapping bcm = bm.createClassMultiMapping("to-table");
        RdbTable tableB = bcm.createRdbTable("table_B");
        bcm.createIdentifierMapping(bNameDef);

        bcm.createReferenceMapping("embedded-target-reference", myANameDef);

        tableB.createPrimitiveElementMapping(idb, "pkb_col", null, true);
        tableB.createPrimitiveElementMapping(f2, "f2_col", null, true);
        //mapping of the reference to A
        tableB.createPrimitiveElementMapping(ida, "fka_col");
    }
}
