/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.facility.naming.polymorphid;

import org.objectweb.jorm.api.*;
import org.objectweb.jorm.facility.naming.generator.LongGen;
import org.objectweb.jorm.facility.naming.generator.LongGenMgr;
import org.objectweb.jorm.facility.naming.generator.LongGenMgrRegistry;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PBinder;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author	P. Dechamboux
 */
public class PolymorphIdMgrImpl implements PolymorphIdMgr {
	private static final String POLYMORPHIDBINDERCLASS
			= "org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdBinderInfo";
	// Static binding to LongGenIncrMgr: TO BE REMOVED
	private final static String LONGGENMGRCLASS
			= "org.objectweb.jorm.facility.naming.generator.LongGenIncrMapping";
	private HashMap cn2binders = new HashMap();
	private HashMap id2binders = new HashMap();
	private HashMap refNC = new HashMap();
	private LongGenMgr lgManager = null;
	private LongGen binderLG;

	protected Logger logger = null;

	public void setLogger(Logger l) {
		logger = l;
	}

	// IMPLEMENTATION OF METHODS FROM THE PolymorphIdMgr INTERFACE

	/**
	 * Notifies the manager that a PType has been associated with this binder.
	 */
	public void binderTypeDef(PolymorphIdBinderInfo b) {
		if (Debug.ON && logger != null) {
			if (b.getPType() == null)
				logger.log(BasicLevel.DEBUG, b.getPType().getJormName());
			else
				logger.log(BasicLevel.DEBUG, "Null type of the binder: " + b);
		}
		Iterator it = refNC.values().iterator();
		while (it.hasNext()) {
			PolymorphRefNC p = (PolymorphRefNC) it.next();
			if (p.getPType() == null)
				continue;
			if (Debug.ON && logger != null)
				logger.log(BasicLevel.DEBUG, "compare to " + p.getPType().getJormName());
			if (b.getPType().isa(p.getPType())) {
				p.addValidBinder(b);
				if (Debug.ON && logger != null) logger.log(BasicLevel.DEBUG, "added");
			}
		}
	}

	/**
	 * Retrieves the LongGenMgr associated with this PolymorphIdMgr.
	 */
	public LongGenMgr getLongGenMgr() {
		return lgManager;
	}

	/**
	 * Retrieves a PBinder that manages polymorphic identifiers for the class
	 * with the given name. If it does not exist, creates it.
	 * @param cn	The class name.
	 */
	public PBinder getPBinder(String cn) throws PException {
		return getPBinder(cn, null);
	}

	/**
	 * Retrieves a PBinder that manages polymorphic identifiers for the class
	 * with the given name. If it does not exist, creates it.
	 * @param cn	The class name.
	 */
	public PBinder getPBinder(String cn, Object conn) throws PException {
		PolymorphIdBinderInfo res = (PolymorphIdBinderInfo) cn2binders.get(cn);
		if (res == null) {
			try {
				res = (PolymorphIdBinderInfo) Class.forName(POLYMORPHIDBINDERCLASS)
						.newInstance();
			} catch (Exception e) {
				throw new PException(e, "Cannot allocate the Binder (probably a ClassLoader problem).");
			}
			if (conn == null)
				res.init(lgManager.getPMapper(), cn, this);
			else
				res.init(lgManager.getPMapper(), cn, this, conn);
			cn2binders.put(res.className, res);
			id2binders.put(res.classId, res);
		}
		return res;
	}

	/**
	 * Retrieves a PBinder that manages polymorphic identifiers for the class
	 * with the given identifier. If it does not exist, creates it.
	 * @param id	The class identifier.
	 */
	public PBinder getPBinder(long id) throws PException {
		return getPBinder(id, null);
	}

	/**
	 * Retrieves a PBinder that manages polymorphic identifiers for the class
	 * with the given identifier. If it does not exist, creates it.
	 * @param id	The class name.
	 * @param conn	The connection to access the data store.
	 */
	public PBinder getPBinder(long id, Object conn) throws PException {
		PolymorphIdBinderInfo res = (PolymorphIdBinderInfo) id2binders.get(new Long(id));
		if (res == null) {
			try {
				res = (PolymorphIdBinderInfo) Class.forName(POLYMORPHIDBINDERCLASS)
						.newInstance();
			} catch (Exception e) {
				throw new PException(e,
									 "Cannot allocate the Binder (probably a ClassLoader problem).");
			}
			res.init(lgManager.getPMapper(), id, this, conn);
			cn2binders.put(res.className, res);
			id2binders.put(res.classId, res);
		}
		return res;
	}

	/**
	 * Retrieves the mapper associated with this PolymorphIdMgr.
	 */
	public PMapper getPMapper() {
		return lgManager.getPMapper();
	}

	/**
	 * Retrieves a PNamingContext that manages polymorphic references for the
	 * class with the given name. This means that such references can point to
	 * object of this class or any of its sub-classes. If it does not exist,
	 * creates it.
	 * @param cn	The class name.
	 */
	public PNamingContext getRefNC(String cn) {
		PolymorphRefNC res = (PolymorphRefNC) refNC.get(cn);
		if (res == null) {
			res = new PolymorphRefNC();
			res.init(this);
			refNC.put(cn, res);
		}
		return res;
	}

	/**
	 * Initializes a polymorphic identifier manager by mapping relevant classes
	 * to the given mapper, and setting up relevant initial persistent
	 * information.
	 */
	public void init(PMapper pm, byte clact) throws PException {
		synchronized (LongGenMgrRegistry.class) {
			lgManager = LongGenMgrRegistry.getLongGenMgr(pm);
			if (lgManager == null) {
				String cn = pm.cn2mn(LONGGENMGRCLASS);
				try {
					lgManager = (LongGenMgr) Class.forName(cn).newInstance();
				} catch (Exception e) {
					throw new PException(e, "Cannot create LongGenMgr (probably a ClassLoader problem): " + cn);
				}
				lgManager.init(pm, clact);
				LongGenMgrRegistry.registerLonGenMgr(lgManager);
			}
		}
		binderLG = lgManager.getLongGen(this.getClass().getPackage().getName() +
										"." + this.getClass().getName());
		String cn = "??";
		try {
			PBinder binder = new BasidBinder(PNamingContext.CTSTRING);
			cn = pm.cn2mn(PolymorphIdBinderInfo.CLASSIDN) + PMapper.PCLASSMAPPINGAPPENDER;
			PClassMapping pcm = (PClassMapping) Class.forName(cn).newInstance();
			pcm.setPBinder(binder);
			binder.setPClassMapping(pcm);
			pm.map(pcm);
			PMapCluster cl = pm.getPMapCluster(pcm.getClassName());
			switch (clact) {
			case PClassMapping.CREATE_STRUCTURE_IF_NEEDED:
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEALL:
				cl.deleteMappingStructures();
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEDATA:
				cl.deleteData();
				break;
			case PClassMapping.CLEANUP_DONOTHING:
				break;
			}

			binder = new BasidBinder(PNamingContext.CTLONG);
			cn = pm.cn2mn(PolymorphIdBinderInfo.IDCLASSN) + PMapper.PCLASSMAPPINGAPPENDER;
			pcm = (PClassMapping) Class.forName(cn).newInstance();
			pcm.setPBinder(binder);
			binder.setPClassMapping(pcm);
			pm.map(pcm);
			cl = pm.getPMapCluster(pcm.getClassName());
			switch (clact) {
			case PClassMapping.CREATE_STRUCTURE_IF_NEEDED:
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEALL:
				cl.deleteMappingStructures();
				cl.createMappingStructures(false);
				break;
			case PClassMapping.CLEANUP_REMOVEDATA:
				cl.deleteData();
				break;
			case PClassMapping.CLEANUP_DONOTHING:
				break;
			}
		} catch (InstantiationException e) {
			throw new PException(e,
								 "Cannot create PClassMapping for [" + cn + "].");
		} catch (IllegalAccessException e) {
			throw new PException(e,
								 "Cannot access PClassMapping class [" + cn + "].");
		} catch (ClassNotFoundException e) {
			throw new PException(e,
								 "Cannot load PClassMapping class [" + cn + "].");
		} catch (PException e) {
			throw e;
		}
	}

	/**
	 * Allocates a new persisttent class identifier.
	 */
	public long newClassId() throws PException {
		return binderLG.genId();
	}

	/**
	 * Notifies the manager that a PType has been associated with this binder.
	 */
	public void ncTypeDef(PolymorphRefNC p) {
		if (Debug.ON && logger != null) {
			if (p.getPType() == null)
				logger.log(BasicLevel.DEBUG, p.getPType().getJormName());
			else
				logger.log(BasicLevel.DEBUG, "Null type of the pnc: " + p);
		}
		Iterator it = cn2binders.values().iterator();
		while (it.hasNext()) {
			PolymorphIdBinderInfo b = (PolymorphIdBinderInfo) it.next();
			if (b.getPType() == null)
				continue;
			if (Debug.ON && logger != null)
				logger.log(BasicLevel.DEBUG, "compare to " + b.getPType().getJormName());
			if (b.getPType().isa(p.getPType())) {
				p.addValidBinder(b);
				if (Debug.ON && logger != null) logger.log(BasicLevel.DEBUG, "added");
			}
		}
	}
}
