/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.objectweb.jorm.compiler.lib.JormCompilerParameter;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.generator.api.MOPFactory;
import org.objectweb.jorm.generator.api.Generator;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.app.Velocity;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.io.File;
import java.io.InputStream;

/**
 * This class start the generations of the jorm objects and user object. The
 * jorm objects are the mapping, the binding and the accessor. It is needed to
 * specify a MOPFactory before start the generation.The generate method
 * starts the generation. To generate user objects it is necessary to specify
 * their GeneratorFactory by the addGeneratorFactory method.
 */
public class JormGenerator implements Generator, Loggable {
	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_PROP
			= "class.resource.loader.description";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE
			= "Velocity Classpath Resource Loader";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_PROP
			= "class.resource.loader.class";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_VALUE
			= "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader";

	private Manager metaInfoManager = null;
	private Logger logger = null;
	private LoggerFactory loggerFactory = null;
	/**
	 * The mopFactories which specialize jorm for different supports
	 */
	private HashMap mopFactories = new HashMap();
	private PathExplorer pathExplorer = null;

	private AccessorGenerator accessorGenerator = null;
	private BindingGenerator bindingGenerator = null;
	private MappingGenerator mappingGenerator = null;
	private PNameGetterGenerator pNameGetterGenerator = null;
	private PNameGenerator pNameGenerator = null;
	private BinderGenerator binderGenerator = null;
	private VelocityEngine velocityEngine = null;

	private JormCompilerParameter compilerParameter = null;
	private JormConfigurator compilerConfigurator = null;

	/**
	 * Initializes the generator. Should be called after setCompilerParameter
	 * and setCompilerConfigurator.
	 * @throws PException
	 */
	public void init() throws PException {
		velocityEngine = new VelocityEngine();
		velocityEngine.setProperty(Velocity.RESOURCE_LOADER, "class");
		velocityEngine.setProperty(CLASS_RESOURCE_LOADER_DESCRIPTION_PROP,
								   CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE);
		velocityEngine.setProperty(CLASS_RESOURCE_LOADER_CLASS_PROP,
								   CLASS_RESOURCE_LOADER_CLASS_VALUE);
		// Connect the velocity log system to monolog
		velocityEngine.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogger(
				(loggerFactory == null
				 ? logger
				 : loggerFactory.getLogger(logger.getName() + ".velocity"))
		));
		StringBuffer strlibs = new StringBuffer();
		String libs[][][];
		libs = setupClassGenerators(compilerConfigurator, compilerParameter);
		String sep = "";
		for (int i = 0; i < libs.length; i++) {
			for (int j = 0; j < libs[i].length; j++) {
				for (int k = 0; k < libs[i][j].length; k++) {
					strlibs.append(sep);
					sep = ",";
					strlibs.append(libs[i][j][k]);
					logger.log(BasicLevel.DEBUG, "VM Library: " + libs[i][j][k]);
				}
			}
		}
		libs = setupCompositeNameGenerators();
		for (int i = 0; i < libs.length; i++) {
			for (int j = 0; j < libs[i].length; j++) {
				for (int k = 0; k < libs[i][j].length; k++) {
					strlibs.append(sep);
					sep = ",";
					strlibs.append(libs[i][j][k]);
					logger.log(BasicLevel.DEBUG, "VM Library: " + libs[i][j][k]);
				}
			}
		}
		velocityEngine.setProperty(Velocity.VM_LIBRARY, strlibs.toString());
		try {
			logger.log(BasicLevel.DEBUG, "initializes the velocity engine");
			velocityEngine.init();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while initializing velocity");
		}
	}

	private Map getContextMap(Class co, JormCompilerParameter cp) throws PException {
		ClassProject cpr = null;
		Iterator it = co.getClassProjects().iterator();
		while (it.hasNext()) {
			cpr = (ClassProject) it.next();
			if (cpr.getProjectName().equals(cp.projectName)) {
				break;
			} else {
				cpr = null;
			}
		}
		if (cpr == null) {
			throw new PExceptionCompiler("Cannot find the relevant project!");
		}
		HashMap res = new HashMap();
		it = cpr.getMappings().iterator();
		while (it.hasNext()) {
			Mapping mo = (Mapping) it.next();
			MOPFactory mopf = (MOPFactory) mopFactories.get(mo.getMapperName());
			res.put(mo.getMapperName(), mopf.createClassGenerationContext(mo));
		}
		return res;
	}

	private String[][][] setupClassGenerators(JormConfigurator jcc, JormCompilerParameter cp) throws PException {
		logger.log(BasicLevel.DEBUG, "setupClassGenerators");
		String[][][] res = new String[3][][];
		if (accessorGenerator == null) {
			accessorGenerator = new AccessorGenerator();
			accessorGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				accessorGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".accessor"));
				accessorGenerator.setLoggerFactory(loggerFactory);
			} else {
				accessorGenerator.setLogger(logger);
			}
		}
		res[0] = accessorGenerator.getTemplateLibraries();
		if (bindingGenerator == null) {
			bindingGenerator = new BindingGenerator(jcc, cp);
			bindingGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				bindingGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".binding"));
				bindingGenerator.setLoggerFactory(loggerFactory);
			} else {
				bindingGenerator.setLogger(logger);
			}
		}
		res[1] = bindingGenerator.getTemplateLibraries();
		if (mappingGenerator == null) {
			mappingGenerator = new MappingGenerator(jcc, cp);
			mappingGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				mappingGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".mapping"));
				mappingGenerator.setLoggerFactory(loggerFactory);
			} else {
				mappingGenerator.setLogger(logger);
			}
		}
		res[2] = mappingGenerator.getTemplateLibraries();
		return res;
	}

	private String[][][] setupCompositeNameGenerators() {
		logger.log(BasicLevel.DEBUG, "setupCompositeNameGenerators");
		String[][][] res = new String[3][][];
		if (pNameGetterGenerator == null) {
			pNameGetterGenerator = new PNameGetterGenerator();
			pNameGetterGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				pNameGetterGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".pnamegetter"));
				pNameGetterGenerator.setLoggerFactory(loggerFactory);
			} else {
				pNameGetterGenerator.setLogger(logger);
			}
		}
		res[0] = pNameGetterGenerator.getTemplateLibraries();
		if (pNameGenerator == null) {
			pNameGenerator = new PNameGenerator();
			pNameGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				pNameGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".pname"));
				pNameGenerator.setLoggerFactory(loggerFactory);
			} else {
				pNameGenerator.setLogger(logger);
			}
		}
		res[1] = pNameGenerator.getTemplateLibraries();
		if (binderGenerator == null) {
			binderGenerator = new BinderGenerator();
			binderGenerator.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				binderGenerator.setLogger(
						loggerFactory.getLogger(logger.getName() + ".binder"));
				binderGenerator.setLoggerFactory(loggerFactory);
			} else {
				binderGenerator.setLogger(logger);
			}
		}
		res[2] = binderGenerator.getTemplateLibraries();
		return res;
	}

	private JormCompilerParameter overloadCompilerParameter(String filename, JormCompilerParameter jcp) {
		logger.log(BasicLevel.DEBUG, "overloadCompilerParameter");
		JormCompilerParameter res;
		File fn = new File(filename);
		try {
			logger.log(BasicLevel.DEBUG, "look for parameter file: " + fn.getParent() + File.separator + "jormcOpts.properties");
			res = (JormCompilerParameter) jcp.clone();
			InputStream is = pathExplorer.getInputStream(fn.getParent() + File.separator + "jormcOpts.properties");
			if (is == null) {
				logger.log(BasicLevel.DEBUG, "no overloading");
				res = jcp;
			} else {
				res.loadConfFile(is, compilerConfigurator.knownMappers());
				logger.log(BasicLevel.DEBUG, "parameter overloaded");
			}
		} catch (PException e) {
			logger.log(BasicLevel.DEBUG, "no overloading");
			res = jcp;
		}
		return res;
	}

	// IMPLEMENTATION OF METHODS FROM THE Generator INTERFACE

	/**
	 * Assigns the compiler parameter to this generator.
	 * @param jcp   the compiler parameter.
	 */
	public void setCompilerParameter(JormCompilerParameter jcp) {
		compilerParameter = jcp;
	}

	/**
	 * Assigns the compiler configurator to this generator.
	 * @param jcc   the compiler configurator.
	 */
	public void setCompilerConfigurator(JormConfigurator jcc) {
		compilerConfigurator = jcc;
	}

	/**
	 * Assigns the meta information manager to this generator.
	 * @param mim   the meta information manager to use.
	 */
	public void setMetaInfoManager(Manager mim) {
		metaInfoManager = mim;
	}

	/**
	 * Assigns a PathExplorer object for locating files that have to be parsed.
	 *
	 * @param pathexpl	the PathExplorer to be used for file location
	 */
	public void setPathExplorer(PathExplorer pathexpl) {
		pathExplorer = pathexpl;
		if (pathExplorer == null) {
			pathExplorer = new DirJavaExplorer();
		}
		logger.log(BasicLevel.DEBUG, "jorm classpath=" + pathExplorer.getClassPath());
	}

	/**
	 * Adds a mapping generator which generates a specific mapping part.
	 * @param mappername		The name of the mapper.
	 * @param mappinggenerator	The Mapping Verifier object.
	 */
	public void addMappingGenerator(String mappername, MOPFactory mappinggenerator) {
		mopFactories.put(mappername, mappinggenerator);
	}

	/**
	 * This method generates the jorm objects by delegation to the
	 * AccessorGenerator, the BindingGenerator and the MappingGenerator. Its
	 * second task is to launch the user objects generator.
	 * @param mo		This meta object is a Class.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		The compiler parameter.
	 * @exception PException if an error occurs during the generation.
	 */
	public void generate(Class mo, TargetHolder holder,
						 JormCompilerParameter cp, JormConfigurator jcc)
			throws PException {
		logger.log(BasicLevel.DEBUG, "class generation: " + mo.getFQName()
									 + " / filename=[" + mo.getFileName() + "]");
		JormCompilerParameter ocp = overloadCompilerParameter(mo.getFileName(), cp);
		logger.log(BasicLevel.DEBUG, "ClassMappingInheritance = " + ocp.classMappingInheritance);
		logger.log(BasicLevel.DEBUG, "BindingInheritance = " + ocp.bindingInheritance);
		logger.log(BasicLevel.DEBUG, "Project Name = " + ocp.projectName);
		ClassProject cpr = mo.getClassProject(ocp.projectName);
		if (cpr == null) {
			logger.log(BasicLevel.WARN, "No project '" + ocp.projectName
										+ "' defined: no generation of binding and mapping done.");
		} else if (!mo.isAbstract()) {
			logger.log(BasicLevel.DEBUG, "let's generate!!");
			// create the contexts map for generation steps to come
			Map cmap = getContextMap(mo, ocp);
			logger.log(BasicLevel.DEBUG, mo.getName() + "Map Contexts generated");
			// generate the mappings
			mappingGenerator.generate(mo, holder, ocp, jcc, cmap);
			logger.log(BasicLevel.DEBUG, mo.getName() + "Mappings generated");
			// generate the bindings
			bindingGenerator.generate(mo, holder, ocp, jcc, cmap);
			logger.log(BasicLevel.DEBUG, mo.getName() + "Bindings generated");
			cmap = null;
		}
		// generate the accessor
		accessorGenerator.generate(mo, holder, ocp);
		logger.log(BasicLevel.DEBUG, mo.getName() + "Accessor generated");
	}

	/**
	 * This method generates the jorm objects by delegation to the
	 * PNameGenerator, BinderGenerator and CompositeNameGenerator.
	 * @param mo		This meta object is a CompositeName.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		The compiler parameter.
	 * @exception PException if an error occurs during the generation.
	 */
	public void generate(CompositeName mo,
						 TargetHolder holder,
						 JormCompilerParameter cp) throws PException {
		logger.log(BasicLevel.DEBUG, "composite name generation: " + mo.getFQName());
		// generate the pnamegetter
		pNameGetterGenerator.generate(mo, holder, cp);
		logger.log(BasicLevel.DEBUG, "XXXpnamegetter generated");
		// generate the pname
		pNameGenerator.generate(mo, holder, cp);
		logger.log(BasicLevel.DEBUG, "XXXpname generated");
		// generate the binder
		binderGenerator.generate(mo, holder, cp);
		logger.log(BasicLevel.DEBUG, "XXXbinder generated");
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}
}
