/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.adapter;

import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.mapper.rdb.lib.RdbBlob;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.sql.PreparedStatement;
import java.sql.Blob;
import java.sql.Connection;
import java.util.Date;
import java.util.List;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Reader;

/*
 * @author	<A HREF="mailto:sebastien.chassande@inrialpes.fr><b>
 *			Sebastien Chassande-Barrioz
 *			</b></A>
 */

public class BasicRdbAdapter implements RdbAdapter {

	private final static String SUBSTRING = "substring";
	private final static String CONCAT = "concat";
	private final static String FIRSTLOCATE = "position";
	private final static String INDEXEDLOCATE = "instr";

	public boolean supportBatchPreparedStatement() {
		return true;
	}

	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	public int getSqlTypeCode(PType pt) throws PExceptionProtocol {
		return getSqlTypeCode(pt, null);
	}

	/**
	 * This methods return the SQL type code linked to the PType
	 * specified in parameter.
	 * @return An int constant defined in the java.sql.Types class
	 */
	public int getSqlTypeCode(PType pt, String sqlColumnType) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type code associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
			return Types.BIT;
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_OBJCHAR:
			return Types.CHAR;
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_OBJBYTE:
			return Types.TINYINT;
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_OBJSHORT:
			return Types.SMALLINT;
		case PType.TYPECODE_INT:
		case PType.TYPECODE_OBJINT:
			return Types.INTEGER;
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_OBJLONG:
			return Types.BIGINT;
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_OBJFLOAT:
			return Types.REAL;
		case PType.TYPECODE_DOUBLE:
		case PType.TYPECODE_OBJDOUBLE:
			return Types.DOUBLE;
		case PType.TYPECODE_STRING:
			return Types.VARCHAR;
		case PType.TYPECODE_DATE:
			if (sqlColumnType == null) {
				return Types.TIMESTAMP;
			} else {
				String str = sqlColumnType.toUpperCase();
				if (str.startsWith("DATE")) {
					return Types.DATE;
				} else if (str.startsWith("TIMESTAMP")) {
					return Types.TIMESTAMP;
				} else if (str.startsWith("TIME")) {
					return Types.TIME;
				} else {
					return Types.TIMESTAMP;
				}
			}
		case PType.TYPECODE_CHARARRAY:
			return Types.VARCHAR;
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_SERIALIZED:
			return Types.VARBINARY;
		case PType.TYPECODE_BIGINTEGER:
			return Types.DECIMAL;
		case PType.TYPECODE_BIGDECIMAL:
			return Types.DECIMAL;
		default:
			throw new PExceptionProtocol("No sql type exist for the "
										 + pt.getJormName() + " PType");
		}
	}

	public String getSqlType(PType pt, boolean usedInPK, int size) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql Type name associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
			return "BIT";
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_OBJCHAR:
			return "CHAR(1)";
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_OBJBYTE:
			return "TINYINT";
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_OBJSHORT:
			return "SMALLINT";
		case PType.TYPECODE_INT:
		case PType.TYPECODE_OBJINT:
			return "INTEGER";
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_OBJLONG:
			return "BIGINT";
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_OBJFLOAT:
			return "REAL";
		case PType.TYPECODE_DOUBLE:
		case PType.TYPECODE_OBJDOUBLE:
			return "DOUBLE";
		case PType.TYPECODE_STRING:
		case PType.TYPECODE_CHARARRAY:
			if (size != PType.NOSIZE) {
				return "VARCHAR(" + size + ")";
			}
			return "VARCHAR";
		case PType.TYPECODE_DATE:
			return "TIMESTAMP";
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_SERIALIZED:
			if (size != PType.NOSIZE) {
				return "VARBINARY(" + size + ")";
			}
			return "VARBINARY";
		case PType.TYPECODE_BIGINTEGER:
		case PType.TYPECODE_BIGDECIMAL:
			return "NUMERIC";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
										 + pt.getJormName() + " PType");
		}
	}

	public String getSqlGetterName(PType pt, String resultSet, int idx)
			throws PExceptionProtocol {
		return getSqlGetterName(pt, resultSet, idx, null);
	}

	public String getSqlGetterName(PType pt, String resultSet, int idx,
								   String sqlColumnType) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql getter associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return resultSet + ".getBoolean(" + idx + ")";
		case PType.TYPECODE_CHAR:
			return resultSet + ".getString(" + idx + ").charAt(0)";
		case PType.TYPECODE_BYTE:
			return resultSet + ".getByte(" + idx + ")";
		case PType.TYPECODE_SHORT:
			return resultSet + ".getShort(" + idx + ")";
		case PType.TYPECODE_INT:
			return resultSet + ".getInt(" + idx + ")";
		case PType.TYPECODE_LONG:
			return resultSet + ".getLong(" + idx + ")";
		case PType.TYPECODE_FLOAT:
			return resultSet + ".getFloat(" + idx + ")";
		case PType.TYPECODE_DOUBLE:
			return resultSet + ".getDouble(" + idx + ")";
		case PType.TYPECODE_STRING:
			return resultSet + ".getString(" + idx + ")";
		case PType.TYPECODE_DATE:
			if (sqlColumnType == null) {
				return resultSet + ".getTimestamp(" + idx + ")";
			} else {
				String str = sqlColumnType.toUpperCase();
				if (str.startsWith("DATE")) {
					return resultSet + ".getDate(" + idx + ")";
				} else if (str.startsWith("TIMESTAMP")) {
					return resultSet + ".getTimestamp(" + idx + ")";
				} else if (str.startsWith("TIME")) {
					return resultSet + ".getTime(" + idx + ")";
				} else {
					return resultSet + ".getTimestamp(" + idx + ")";
				}
			}
		case PType.TYPECODE_BIGINTEGER:
			return "((fieldval = " + resultSet + ".getBigDecimal(" + idx + "))==null ? null : ((BigDecimal) fieldval).toBigInteger())";
		case PType.TYPECODE_BIGDECIMAL:
			return resultSet + ".getBigDecimal(" + idx + ")";
		case PType.TYPECODE_CHARARRAY:
			return resultSet + ".getCharacterStream(" + idx + ")";
		case PType.TYPECODE_BYTEARRAY:
			return resultSet + ".getBytes(" + idx + ")";
		case PType.TYPECODE_OBJSHORT:
			return "new Short(" + resultSet + ".getShort(" + idx + "))";
		case PType.TYPECODE_OBJBOOLEAN:
			return "new Boolean(" + resultSet + ".getBoolean(" + idx + "))";
		case PType.TYPECODE_OBJCHAR:
			return "((fieldval = " + resultSet + ".getString(" + idx + "))==null ? null : new Character(((String) fieldval).charAt(0)))";
		case PType.TYPECODE_OBJBYTE:
			return "new Byte(" + resultSet + ".getByte(" + idx + "))";
		case PType.TYPECODE_OBJINT:
			return "new Integer(" + resultSet + ".getInt(" + idx + "))";
		case PType.TYPECODE_OBJLONG:
			return "new Long(" + resultSet + ".getLong(" + idx + "))";
		case PType.TYPECODE_OBJFLOAT:
			return "new Float(" + resultSet + ".getFloat(" + idx + "))";
		case PType.TYPECODE_OBJDOUBLE:
			return "new Double(" + resultSet + ".getDouble(" + idx + "))";
		case PType.TYPECODE_SERIALIZED:
			return "resultSet.getBinaryStream(" + idx + ")";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
										 + pt.getJormName() + " PType");
		}
	}

	public String getSqlSetterName(PType pt, String value, int idx) throws PExceptionProtocol {
		return getSqlSetterName(pt, value, idx, null);
	}

	public String getSqlSetterName(PType pt, String value, int idx,
								   String sqlColumnType) throws PExceptionProtocol {
		if (pt == null) {
			throw new PExceptionProtocol("No Sql setter associated to a null PType");
		}
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "setBoolean(" + idx + ", " + value + ")";
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_OBJCHAR:
			return "setString(" + idx + ", " + "String.valueOf(" + value + "))";
		case PType.TYPECODE_BYTE:
			return "setByte(" + idx + ", " + value + ")";
		case PType.TYPECODE_SHORT:
			return "setShort(" + idx + ", " + value + ")";
		case PType.TYPECODE_INT:
			return "setInt(" + idx + ", " + value + ")";
		case PType.TYPECODE_LONG:
			return "setLong(" + idx + ", " + value + ")";
		case PType.TYPECODE_FLOAT:
			return "setFloat(" + idx + ", " + value + ")";
		case PType.TYPECODE_DOUBLE:
			return "setDouble(" + idx + ", " + value + ")";
		case PType.TYPECODE_STRING:
			return "setString(" + idx + ", " + value + ")";
		case PType.TYPECODE_DATE:
			String type = "java.sql.Timestamp";
			String sqlSetterName = "setTimestamp";
			if (sqlColumnType == null) {
				return sqlSetterName + "(" + idx + ", "
						+ "(" + value + "== null ? null : " +
						"(" + value + " instanceof " + type
						+ " ? (" + type + ")" + value
						+ " : new " + type + "(" + value + ".getTime()))))";
			} else {
				String str = sqlColumnType.toUpperCase();
				if (str.startsWith("DATE")) {
					type = "java.sql.Date";
					sqlSetterName = "setDate";
				} else if (str.startsWith("TIMESTAMP")) {
					type = "java.sql.Timestamp";
					sqlSetterName = "setTimestamp";
				} else if (str.startsWith("TIME")) {
					type = "java.sql.Time";
					sqlSetterName = "setTime";
				}
			}
			return sqlSetterName + "(" + idx + ", "
					+ "(" + value + "== null ? null : " +
					"(" + value + " instanceof " + type
					+ " ? (" + type + ")" + value
					+ " : new " + type + "(" + value + ".getTime()))))";
		case PType.TYPECODE_BIGINTEGER:
			return "setBigDecimal(" + idx + ", new BigDecimal(" + value + "))";
		case PType.TYPECODE_BIGDECIMAL:
			return "setBigDecimal(" + idx + ", " + value + ")";
		case PType.TYPECODE_CHARARRAY:
			return "setCharacterStream(" + idx + ", ((chararray = " + value + ") == null ? null : new java.io.CharArrayReader(chararray)), (chararray == null ? 0 : chararray.length))";
		case PType.TYPECODE_BYTEARRAY:
			return "setBytes(" + idx + ", " + value + ")";
		case PType.TYPECODE_SERIALIZED:
			return "setBinaryStream(" + idx + ", (blob = new RdbBlob(" + value + ")).getBinaryStream(), (int) blob.length())";
		case PType.TYPECODE_OBJBOOLEAN:
			return "setObject(" + idx + ", " + value + ", Types.BIT)";
		case PType.TYPECODE_OBJBYTE:
			return "setObject(" + idx + ", " + value + ", Types.TINYINT)";
		case PType.TYPECODE_OBJSHORT:
			return "setObject(" + idx + ", " + value + ", Types.SMALLINT)";
		case PType.TYPECODE_OBJINT:
			return "setObject(" + idx + ", " + value + ", Types.INTEGER)";
		case PType.TYPECODE_OBJLONG:
			return "setObject(" + idx + ", " + value + ", Types.BIGINT)";
		case PType.TYPECODE_OBJFLOAT:
			return "setObject(" + idx + ", " + value + ", Types.REAL)";
		case PType.TYPECODE_OBJDOUBLE:
			return "setObject(" + idx + ", " + value + ", Types.DOUBLE)";
		default:
			throw new PExceptionProtocol("No sql type exist for the "
										 + pt.getJormName() + " PType");
		}
	}

	/**
	 * This calculates the size of a result set. If this feature is not supported
	 * by a database or its jdbc driver, this method returns -1.
	 * @param rs the result set which the size must be returned
	 */
	public int fetchResultSetSize(ResultSet rs) throws SQLException {
		try {
			return rs.getFetchSize();
		} catch (SQLException e) {
			return -1;
		}
	}

	public String getValueAsSQLString(Object value, PType type) {
		if (value == null || type == null)
			return null;
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_OBJBOOLEAN:
			return (((Boolean) value).booleanValue() ? "1" : "0");

		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_OBJFLOAT:
		case PType.TYPECODE_OBJDOUBLE:
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_INT:
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_DOUBLE:
			return value.toString();

		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_STRING:
		case PType.TYPECODE_DATE:
		case PType.TYPECODE_BIGINTEGER:
		case PType.TYPECODE_BIGDECIMAL:
			return "'" + value.toString() + "'";

		case PType.TYPECODE_CHARARRAY:
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_SERIALIZED:
		default:
			return null;
		}
	}

	public String getValueAsSQLString(boolean value) {
		return (value ? "1" : "0");
	}

	public String getValueAsSQLString(byte value) {
		return "" + value;
	}

	public String getValueAsSQLString(char value) {
		return "'" + value + "'";
	}

	public String getValueAsSQLString(short value) {
		return "" + value;
	}

	public String getValueAsSQLString(int value) {
		return "" + value;
	}

	public String getValueAsSQLString(long value) {
		return "" + value;
	}

	public String getValueAsSQLString(float value) {
		return "" + value;
	}

	public String getValueAsSQLString(double value) {
		return "" + value;
	}

	public boolean getBoolean(ResultSet rs, int idx) throws SQLException {
		return rs.getBoolean(idx);
	}

	public Boolean getOboolean(ResultSet rs, int idx) throws SQLException {
		boolean res = rs.getBoolean(idx);
		return (rs.wasNull() ? null : new Boolean(res));
	}

	public char getChar(ResultSet rs, int idx) throws SQLException {
		return rs.getString(idx).charAt(0);
	}

	public Character getOchar(ResultSet rs, int idx) throws SQLException {
		String s = rs.getString(idx);
		return (rs.wasNull() ? null : new Character(s.charAt(0)));
	}

	public byte getByte(ResultSet rs, int idx) throws SQLException {
		return rs.getByte(idx);
	}

	public Byte getObyte(ResultSet rs, int idx) throws SQLException {
		byte res = rs.getByte(idx);
		return (rs.wasNull() ? null : new Byte(res));
	}

	public short getShort(ResultSet rs, int idx) throws SQLException {
		return rs.getShort(idx);
	}

	public Short getOshort(ResultSet rs, int idx) throws SQLException {
		short res = rs.getShort(idx);
		return (rs.wasNull() ? null : new Short(res));
	}

	public int getInt(ResultSet rs, int idx) throws SQLException {
		return rs.getInt(idx);
	}

	public Integer getOint(ResultSet rs, int idx) throws SQLException {
		int res = rs.getInt(idx);
		return (rs.wasNull() ? null : new Integer(res));
	}

	public long getLong(ResultSet rs, int idx) throws SQLException {
		return rs.getLong(idx);
	}

	public Long getOlong(ResultSet rs, int idx) throws SQLException {
		long res = rs.getLong(idx);
		return (rs.wasNull() ? null : new Long(res));
	}

	public float getFloat(ResultSet rs, int idx) throws SQLException {
		return rs.getFloat(idx);
	}

	public Float getOfloat(ResultSet rs, int idx) throws SQLException {
		float res = rs.getFloat(idx);
		return (rs.wasNull() ? null : new Float(res));
	}

	public double getDouble(ResultSet rs, int idx) throws SQLException {
		return rs.getDouble(idx);
	}

	public Double getOdouble(ResultSet rs, int idx) throws SQLException {
		double res = rs.getDouble(idx);
		return (rs.wasNull() ? null : new Double(res));
	}

	public String getString(ResultSet rs, int idx) throws SQLException {
		return rs.getString(idx);
	}

	public Date getDate(ResultSet rs, int idx) throws SQLException {
		return rs.getTimestamp(idx);
	}

	public Date getDate(ResultSet rs, int idx, String sqlColumnType) throws SQLException {
		Date res = null;
		if (sqlColumnType == null || sqlColumnType.length() == 0) {
			res = rs.getTimestamp(idx);
		} else {
			String str = sqlColumnType.toUpperCase();
			if (str.startsWith("DATE")) {
				res = rs.getDate(idx);
			} else if (str.startsWith("TIMESTAMP")) {
				res = rs.getTimestamp(idx);
			} else if (str.startsWith("TIME")) {
				res = rs.getTime(idx);
			} else {
				res = rs.getTimestamp(idx);
			}
		}
		if (rs.wasNull()) {
			return null;
		} else {
			return res;
		}
	}

	public char[] getCharArray(ResultSet rs, int idx) throws SQLException {
		Reader r = rs.getCharacterStream(idx);
		if (rs.wasNull()) {
			return null;
		} else {
			try {
				return getCharArray(r);
			} catch (IOException e) {
				throw new SQLException(
						"Impossible to build a char[]: " + e.getMessage());
			}
		}
	}

	private static char[] getCharArray(java.io.Reader reader) throws java.io.IOException {
		int size = 0;
		long skipped = 0;
		while ((skipped = reader.skip(50)) == 50) {
			size += 50;
		}
		size += (int) skipped;
		char[] res = new char[size];
		reader.reset();
		reader.read(res);
		return res;
	}

	public byte[] getByteArray(ResultSet rs, int idx) throws SQLException {
		return rs.getBytes(idx);
	}

	public Serializable getSerialized(ResultSet rs, int idx)
			throws SQLException, IOException, ClassNotFoundException {
		return (Serializable) new ObjectInputStream(rs.getBinaryStream(idx)).readObject();
	}

	public BigInteger getBigInteger(ResultSet rs, int idx) throws SQLException {
		return rs.getBigDecimal(idx).toBigInteger();
	}

	public BigDecimal getBigDecimal(ResultSet rs, int idx) throws SQLException {
		return rs.getBigDecimal(idx);
	}

	public void setBoolean(PreparedStatement ps, int idx, boolean o)
			throws SQLException {
		ps.setBoolean(idx, o);
	}

	public void setOboolean(PreparedStatement ps, int idx, Boolean o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.BIT);
		else
			ps.setBoolean(idx, o.booleanValue());
	}

	public void setChar(PreparedStatement ps, int idx, char c)
			throws SQLException {
		ps.setString(idx, String.valueOf(c));
	}

	public void setOchar(PreparedStatement ps, int idx, Character o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.CHAR);
		else {
			ps.setString(idx, String.valueOf(o.charValue()));
		}
	}

	public void setByte(PreparedStatement ps, int idx, byte o)
			throws SQLException {
		ps.setByte(idx, o);
	}

	public void setObyte(PreparedStatement ps, int idx, Byte o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.SMALLINT);
		else
			ps.setByte(idx, o.byteValue());
	}

	public void setShort(PreparedStatement ps, int idx, short o)
			throws SQLException {
		ps.setShort(idx, o);
	}

	public void setOshort(PreparedStatement ps, int idx, Short o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.SMALLINT);
		else
			ps.setShort(idx, o.shortValue());
	}

	public void setInt(PreparedStatement ps, int idx, int o)
			throws SQLException {
		ps.setInt(idx, o);
	}

	public void setOint(PreparedStatement ps, int idx, Integer o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.INTEGER);
		else
			ps.setInt(idx, o.intValue());
	}

	public void setLong(PreparedStatement ps, int idx, long o)
			throws SQLException {
		ps.setLong(idx, o);
	}

	public void setOlong(PreparedStatement ps, int idx, Long o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.BIGINT);
		else
			ps.setLong(idx, o.longValue());
	}

	public void setFloat(PreparedStatement ps, int idx, float o)
			throws SQLException {
		ps.setFloat(idx, o);
	}

	public void setOfloat(PreparedStatement ps, int idx, Float o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.FLOAT);
		else
			ps.setFloat(idx, o.floatValue());
	}

	public void setDouble(PreparedStatement ps, int idx, double o)
			throws SQLException {
		ps.setDouble(idx, o);
	}

	public void setOdouble(PreparedStatement ps, int idx, Double o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.DOUBLE);
		else
			ps.setDouble(idx, o.doubleValue());
	}

	public void setString(PreparedStatement ps, int idx, String o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.VARCHAR);
		else
			ps.setString(idx, o);
	}

	public void setDate(PreparedStatement ps, int idx, Date o, String columnType)
			throws SQLException {
		if (columnType == null) {
//default is a Timestamp
			if (o == null)
				ps.setNull(idx, java.sql.Types.TIMESTAMP);
			else if (o instanceof java.sql.Timestamp) {
				ps.setTimestamp(idx, (java.sql.Timestamp) o);
			} else {
				ps.setTimestamp(idx, new java.sql.Timestamp(o.getTime()));
			}
		} else if (columnType.startsWith("DATE")) {
			if (o == null)
				ps.setNull(idx, java.sql.Types.DATE);
			else if (o instanceof java.sql.Date) {
				ps.setDate(idx, (java.sql.Date) o);
			} else {
				ps.setDate(idx, new java.sql.Date(o.getTime()));
			}
		} else if (columnType.startsWith("TIMESTAMP")) {
			if (o == null)
				ps.setNull(idx, java.sql.Types.TIMESTAMP);
			else if (o instanceof java.sql.Timestamp) {
				ps.setTimestamp(idx, (java.sql.Timestamp) o);
			} else {
				ps.setTimestamp(idx, new java.sql.Timestamp(o.getTime()));
			}
		} else if (columnType.startsWith("TIME")) {
			if (o == null)
				ps.setNull(idx, java.sql.Types.TIME);
			else if (o instanceof java.sql.Time) {
				ps.setTime(idx, (java.sql.Time) o);
			} else {
				ps.setTime(idx, new java.sql.Time(o.getTime()));
			}
		}
	}

	public void setCharArray(PreparedStatement ps, int idx, char[] o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.VARBINARY);
		else {
			ps.setCharacterStream(idx, new java.io.CharArrayReader(o), o.length);
		}
	}

	public void setByteArray(PreparedStatement ps, int idx, byte[] o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.VARBINARY);
		else
			ps.setBytes(idx, o);
	}

	public void setBigInteger(PreparedStatement ps, int idx, BigInteger o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.DECIMAL);
		else
			ps.setBigDecimal(idx, new BigDecimal(o));
	}

	public void setBigDecimal(PreparedStatement ps, int idx, BigDecimal o)
			throws SQLException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.DECIMAL);
		else
			ps.setBigDecimal(idx, o);
	}

	public void setSerialized(PreparedStatement ps, int idx, Serializable o)
			throws SQLException, IOException {
		if (o == null)
			ps.setNull(idx, java.sql.Types.VARBINARY);
		else {
			Blob blob = new RdbBlob(o);
			ps.setBinaryStream(idx, blob.getBinaryStream(), (int) blob.length());
		}
	}

	public String getSubstringName() {
		return SUBSTRING;
	}

	public String getConcatExpression(String op1, String op2) {
		return CONCAT + "(" + op1 + ", " + op2 + ")";
	}

	public String getFirstLocateExpression(String substring, String instring)
			throws PException {
		return FIRSTLOCATE + "(" + substring + " in " + instring + ")";
	}

	public String getIndexedLocateExpression(String instring,
											 String substring,
											 String fromIndex)
			throws PException {
		return INDEXEDLOCATE + "(" + instring + ", " + substring + ", " +
				fromIndex + ")";
	}

	public String getQuery(String selectClause, List jts, String whereclause) {
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT ");
		sb.append(selectClause);
		sb.append(" FROM ");
		String sep = "";
		for (int i = 0; i < jts.size(); i++) {
			JoinedTable jt = (JoinedTable) jts.get(i);
			sb.append(sep);
			sep = ", ";
			getFromClause(jt, sb);
		}
		getWhereClause(jts, sb, whereclause);
		return sb.toString();
	}

	protected void getFromClause(JoinedTable jt, StringBuffer sb) {
		sb.append(jt.tableName);
		if (jt.alias != null && jt.alias.length() > 0) {
			writeTableAlias(jt.alias, sb);
		}
		for (int i = 0; i < jt.joinedTables.size(); i++) {
			getFromClause(jt,
						  (JoinedTable) jt.joinedTables.get(i),
						  (JoinedTable.Join) jt.joins.get(i),
						  sb);
		}
	}

	public void writeTableAlias(String alias, StringBuffer sb) {
		sb.append(" as ");
		sb.append(alias);
	}

	public String handleOrderBy(String query) {
		return query;
	}

	public String getCreateSequence(String seqName) {
		return "CREATE SEQUENCE " + seqName;
	}

	public String getNextValInSequence(String seqName) {
		return "select nextVal('" + seqName + "')";
	}


	public boolean existTable(Connection connection, String tableName) throws SQLException {
		boolean existtable = false;
		ResultSet rs = null;
		try {
			rs = connection.getMetaData().getTables(null, null, tableName, new String[]{"TABLE"});
			while (rs.next() && !existtable) {
				existtable = tableName.equalsIgnoreCase(rs.getString(3));
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
		}
		return existtable;
	}

	public boolean existSequence(Connection connection, String seqName) throws SQLException {
		return existTable(connection, seqName);
	}


	protected void getFromClause(JoinedTable jtMain,
								 JoinedTable jt, JoinedTable.Join join,
								 StringBuffer sb) {
		sb.append(" left outer join ");
		sb.append(jt.tableName);
		if (jt.alias != null && jt.alias.length() > 0) {
			sb.append(" as ");
			sb.append(jt.alias);
		}
		sb.append(" on ");
		String sep = "";
		String main = (jtMain.alias != null && jtMain.alias.length() > 0
				? jtMain.alias
				: jtMain.tableName );
		String ext = (jt.alias != null && jt.alias.length() > 0
				? jt.alias
				: jt.tableName );
		for (int i = 0; i < join.colT1.size(); i++) {
			sb.append(sep);
			sep = " AND ";
			sb.append(main);
			sb.append(".");
			sb.append(join.colT1.get(i));
			sb.append(" = ");
			sb.append(ext);
			sb.append(".");
			sb.append(join.colT2.get(i));
		}
		for (int i = 0; i < jt.joinedTables.size(); i++) {
			getFromClause(jt,
						  (JoinedTable) jt.joinedTables.get(i),
						  (JoinedTable.Join) jt.joins.get(i),
						  sb);
		}
	}

	protected void getWhereClause(List jts,
								  StringBuffer sb,
								  String whereclause) {
		if (whereclause != null && whereclause.length() > 0) {
			sb.append(" WHERE ");
			sb.append(whereclause);
		}
	}

	public static char[] getCharArray(Object o) throws Exception {
		if (o instanceof java.io.Reader) {
			java.io.Reader reader = (java.io.Reader) o;
			int size = 0;
			long skipped = 0;
			while ((skipped = reader.skip(50)) == 50) {
				size += 50;
			}
			size += (int) skipped;
			char[] res = new char[size];
			reader.reset();
			reader.read(res);
			return res;
		} else if (o instanceof String) {
			return ((String) o).toCharArray();
		} else {
			throw new PException("Unknown type to convert to char[]");
		}
	}

	public String getSubqueryAlias() {
		return "";
	}

}
