/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.generator;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.generator.lib.CommonHelper;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.mapper.rdb.adapter.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.JoinedTable;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.Collection;
import java.util.Collections;

/**
 * This is the root class that contains all the required information for
 * generating Java code specific to the RDB mapping. It allows the support of
 * all elaborated mapping rules.
 * @author  P. Dechamboux
 */
public class RdbGenInfos {
	/**
	 * The set of all tables used to store the generated class. It also contains
	 * the reference table that is supposed to hold the object identifier (see
	 * referenceTable).
	 */
	public Map genTables = new HashMap();
	/**
	 * The reference table used in this mapping. It entirely stores the
	 * identifier of the related JORM object, as specified by variable genId.
	 */
	public RdbGenTable referenceTable = null;
	/**
	 * The set of all columns used to store the generated class. It defines the
	 * order with which the columns will be used within all kinds of queries.
	 * This set is the union of all columns specified with each table involved
	 * in this mapping.
	 */
	public List readableColumns = new ArrayList();
	/**
	 * The structure of the identifier of the related JORM object.
	 */
	public RdbGenRef genId = null;
	/**
	 * The set of references defined by the generated class.
	 * There is one RdbGenRef object registered for each reference field,
	 * in association with its field name into this map.
	 */
	public Map genRefs = new HashMap();

	public static CommonHelper commonHelper = new CommonHelper();

	public Map getGenTables() {
		return genTables;
	}

	public RdbGenTable getReferenceTable() {
		return referenceTable;
	}

	public List getReadableColumns() {
		return readableColumns;
	}

	public RdbGenRef getGenId() {
		return genId;
	}

	public Map getGenRefs() {
		return genRefs;
	}

	/**
	 * Computes the list of references (i.e., their definition) stored into this
	 * particular table.
	 * @param rgt   The table for which to find the reference definitions.
	 * @return  The computed list of definitions.
	 */
	public List getTableCompositeGenRefs(RdbGenTable rgt) {
		return getTableCompositeGenRefs(rgt, null);
	}

	public List getTableCompositeGenRefs(RdbGenTable rgt, RdbGenJoin rgj) {
		ArrayList res = new ArrayList();
		Iterator it = genRefs.values().iterator();
		while (it.hasNext()) {
			RdbGenRef rgr = (RdbGenRef) it.next();
			if (rgr.refColumn == null && rgr.join == rgj && !res.contains(rgr)) {
				res.add(rgr);
			}
		}
		return res;
	}

	/**
	 * Gets the position (within [1..n] of the given column into the overall
	 * list defined into this RdbGenInfos.
	 * @param irgc  The column for which to search the position.
	 * @return  Its position into the genColumns list. -1 if not found (BUG:
	 *          should never appear).
	 */
	public int getColumnPosition(RdbGenColumn irgc) {
		return getColumnPosition(irgc, null);
	}

	public int getColumnPosition(RdbGenColumn irgc, RdbGenJoin rgj) {
		int idx = 1;
		for (int i = 0; i < readableColumns.size(); i++) {
			RdbGenColumn rgc = (RdbGenColumn) readableColumns.get(i);
			if (rgc == irgc) {
				return (rgj == null ? idx : idx + rgj.joinIdx);
			} else if (rgc.joins != null) {
				idx += rgc.joins.size();
			} else {
				idx++;
			}
		}
		return -1;
	}

	/**
	 * Constructs the string representing the SQL query used for testing the
	 * existence of a JORM object.
	 * @return  The query string.
	 */
	public String getExistStatement() {
		StringBuffer sb = new StringBuffer("SELECT ");
		String sep = "";
		Iterator it;
		String tabname = null;
		if (genId.refColumn != null) {
			sb.append(sep);
			appendFQColName(sb, genId.refColumn);
			tabname = genId.refColumn.columnTable.tableName;
		} else /* genCNId must not be null */ {
			it = genId.cnFieldColumns.keySet().iterator();
			while (it.hasNext()) {
				String cnfn = (String) it.next();
				RdbGenColumn rgc = (RdbGenColumn) genId.cnFieldColumns.get(cnfn);
				sb.append(sep);
				sep = ", ";
				appendFQColName(sb, rgc);
				if (tabname == null) {
					tabname = rgc.columnTable.tableName;
				}
			}
		}
		sb.append(" FROM ");
		sb.append(tabname);
		sb.append(" WHERE ");
		sep = "";
		if (genId.refColumn != null) {
			sb.append(sep);
			appendFQColName(sb, genId.refColumn);
			sb.append(" = ?");
		} else /* genCNId must not be null */ {
			it = genId.cnFieldColumns.keySet().iterator();
			while (it.hasNext()) {
				String cnfn = (String) it.next();
				RdbGenColumn rgc = (RdbGenColumn) genId.cnFieldColumns.get(cnfn);
				sb.append(sep);
				sep = " AND ";
				appendFQColName(sb, rgc);
				sb.append(" = ?");
			}
		}
		return sb.toString();
	}

	/**
	 * Constructs the string representing the SQL query used to read a JORM
	 * object.
	 * @return  The query string.
	 */
	public String getReadStatement(RdbAdapter adapter) {
		//build the select clause
		StringBuffer select = new StringBuffer();
		String sep = "";
		for (int i = 0; i < readableColumns.size(); i++) {
			RdbGenColumn rgc = (RdbGenColumn) readableColumns.get(i);
			if (rgc.joins != null && rgc.joins.size() > 1) {
				String[] s = new String[rgc.joins.size()];
				for (int j = 0; j < rgc.joins.size(); j++) {
					RdbGenJoin rgj = (RdbGenJoin) rgc.joins.get(j);
					s[rgj.joinIdx] = getFQColName(rgc, "_J" + rgj.joinIdx);
				}
				for (int j = 0; j < s.length; j++) {
					select.append(sep);
					sep = ", ";
					select.append(s[j]);
				}
			} else {
				select.append(sep);
				sep = ", ";
				appendFQColName(select, rgc);
			}
		}

		// build the data structure representing the from clause and the joins
		JoinedTable jt = new JoinedTable(referenceTable.tableName);
		Iterator it = genTables.values().iterator();
		while (it.hasNext()) {
			RdbGenTable rgt = (RdbGenTable) it.next();
			if (rgt.joins != null) {
				boolean multiJoin = rgt.joins.size() > 1;
				for (int i = 0; i < rgt.joins.size(); i++) {
					JoinedTable.Join jtj;
					RdbGenJoin join = (RdbGenJoin) rgt.joins.get(i);
					String tableSuffix = null;
					if (multiJoin) {
						tableSuffix = "_J" + ((RdbGenJoin) rgt.joins.get(i)).joinIdx;
						jtj = jt.createChildren(rgt.tableName,
												rgt.tableName + tableSuffix);
					} else {
						jtj = jt.createChildren(rgt.tableName);
					}
					for (int j = 0; j < join.joinColumnsInMain.size(); j++) {
						jtj.addJoinColumn(
								((RdbGenColumn) join.joinColumnsInMain.get(j)).columnName,
								((RdbGenColumn) join.joinColumnsInExt.get(j)).columnName);
					}
				}
			}
		}

		//build the where clause with only the identifier matching
		StringBuffer where = new StringBuffer();
		sep = "";
		// apply the identifier matching
		if (genId.refColumn != null) {
			where.append(sep);
			appendFQColName(where, genId.refColumn);
			where.append(" = ?");
		} else {
			it = genId.cnFieldColumns.keySet().iterator();
			while (it.hasNext()) {
				String cnfn = (String) it.next();
				RdbGenColumn rgc = (RdbGenColumn) genId.cnFieldColumns.get(cnfn);
				where.append(sep);
				sep = " AND ";
				appendFQColName(where, rgc);
				where.append(" = ?");
			}
		}

		//Delegate to the adapter the bulding of the query
		return adapter.getQuery(select.toString(),
								Collections.singletonList(jt), where.toString());
	}

	/**
	 * Constructs the string representing the SQL query used to get the extent
	 * of a JORM class (the ID PName of all objects of this class).
	 * @return  The query string.
	 */
	public String getPNameIteratorStatement() {
		StringBuffer sb = new StringBuffer("SELECT ");
		if (genId.getCnFieldColumns() == null) {
			appendFQColName(sb, genId.getRefColumn());
		} else {
			String sep = "";
			Iterator it = genId.getCnFieldColumns().keySet().iterator();
			while (it.hasNext()) {
				RdbGenColumn rgc = (RdbGenColumn) genId.getCnFieldColumns().get(it.next());
				if (!colInGenId(rgc)) {
					continue;
				}
				sb.append(sep);
				sep = ", ";
				appendFQColName(sb, rgc);
			}
		}
		sb.append(" FROM ");
		sb.append(referenceTable.tableName);
		return sb.toString();
	}

	/**
	 * Constructs the string representing the SQL query used to update the part
	 * of a JORM object stored into the given table.
	 * @param rgt   The description of the table to be modified.
	 * @return  The query string.
	 */
	public String getUpdateStatement(RdbGenTable rgt) throws PException {
		StringBuffer sb = new StringBuffer("UPDATE ");
		sb.append(rgt.tableName);
		sb.append(" SET ");
		String sep = "";
		for (int i = 0; i < rgt.tableColumns.size(); i++) {
			if (updatable((RdbGenColumn) rgt.tableColumns.get(i))) {
				sb.append(sep);
				sep = ", ";
				sb.append(((RdbGenColumn) rgt.tableColumns.get(i)).columnName);
				sb.append(" = ?");
			}
		}
		appendModificationWhereClause(sb, rgt);
		return sb.toString();
	}

	public String getUpdateStatement(RdbGenJoin rgj) throws PException {
		StringBuffer sb = new StringBuffer("UPDATE ");
		sb.append(rgj.table.tableName);
		sb.append(" SET ");
		String sep = "";
		if (rgj.table.colocatedTable) {
			for (int i = 0; i < rgj.joinColumnsInExt.size(); i++) {
				sb.append(sep);
				sep = ", ";
				sb.append(((RdbGenColumn) rgj.joinColumnsInExt.get(i)).columnName);
				sb.append(" = ?");
			}

		} else {
			for (int i = 0; i < rgj.table.tableColumns.size(); i++) {
				RdbGenColumn rgc = (RdbGenColumn) rgj.table.tableColumns.get(i);
				if (rgc.joinCol == null) {
					sb.append(sep);
					sep = ", ";
					sb.append(rgc.columnName);
					sb.append(" = ?");
				}
			}
		}
		appendModificationWhereClause(sb, rgj, rgj.table.colocatedTable);
		return sb.toString();
	}

	public String getUpdateNullStatement(RdbGenTable rgt) throws PException {
		if (rgt.joins != null && rgt.joins.size() > 0) {
			throw new PException("umanaged case, must use " +
								 "getUpdateNullStatement(RdbGenJoin) for each join.");
		}
		StringBuffer sb = new StringBuffer("UPDATE ");
		sb.append(rgt.tableName);
		sb.append(" SET ");
		String sep = "";
		for (int i = 0; i < rgt.tableColumns.size(); i++) {
			RdbGenColumn rgc = (RdbGenColumn) rgt.tableColumns.get(i);
			if (updatable(rgc)) {
				if (rgt.colocatedColumn(rgc)) {
					continue;
				}
				sb.append(sep);
				sep = ", ";
				sb.append(rgc.columnName);
				sb.append(" = null");
			}
		}
		appendModificationWhereClause(sb, rgt);
		return sb.toString();
	}

	public String getUpdateNullStatement(RdbGenJoin rgj) throws PException {
		StringBuffer sb = new StringBuffer("UPDATE ");
		sb.append(rgj.table.tableName);
		sb.append(" SET ");
		String sep = "";
		for (int i = 0; i < rgj.joinColumnsInExt.size(); i++) {
			sb.append(sep);
			sep = ", ";
			sb.append(((RdbGenColumn) rgj.joinColumnsInExt.get(i)).columnName);
			sb.append(" = null");
		}
		appendModificationWhereClause(sb, rgj, false);
		return sb.toString();
	}

	/**
	 * Constructs the string representing the SQL query used to insert the part
	 * of a JORM object stored into the given table.
	 * @param rgt   The description of the table to be modified.
	 * @return  The query string.
	 */
	public String getInsertStatement(RdbGenTable rgt) {
		String sep = "";
		StringBuffer sb = new StringBuffer("INSERT INTO ");
		sb.append(rgt.tableName);
		sb.append(" (");
		for (int i = 0; i < rgt.tableColumns.size(); i++) {
			sb.append(sep);
			sep = ", ";
			sb.append(((RdbGenColumn) rgt.tableColumns.get(i)).columnName);
		}
		sb.append(") VALUES (");
		sep = "";
		for (int i = 0; i < rgt.tableColumns.size(); i++) {
			sb.append(sep);
			sep = ", ";
			sb.append("?");
		}
		sb.append(")");
		return sb.toString();
	}

	public String getInsertStatement(RdbGenJoin rgj) {
		RdbGenTable rgt = rgj.getTable();
		int i = 0;
		try {
			String sep = "";
			StringBuffer sb = new StringBuffer("INSERT INTO ");
			sb.append(rgt.tableName);
			sb.append(" (");
			for (i = 0; i < rgt.tableColumns.size(); i++) {
				RdbGenColumn col = (RdbGenColumn) rgt.tableColumns.get(i);
				if (col.isInJoin(rgj)) {
					sb.append(sep);
					sep = ", ";
					sb.append(((RdbGenColumn) rgt.tableColumns.get(i)).columnName);
				}
			}
			sb.append(") VALUES (");
			sep = "";
			for (i = 0; i < rgt.tableColumns.size(); i++) {
				RdbGenColumn col = (RdbGenColumn) rgt.tableColumns.get(i);
				if (col.isInJoin(rgj)) {
					sb.append(sep);
					sep = ", ";
					sb.append("?");
				}
			}
			sb.append(")");
			return sb.toString();
		} catch (RuntimeException e) {
			System.out.println(rgt.tableName);
			System.out.println(rgt.tableColumns.get(i));
			System.out.println(((RdbGenColumn) rgt.tableColumns.get(i)).joins);
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Constructs the string representing the SQL query used to delete the part
	 * of a JORM object stored into the given table.
	 * @param rgt   The description of the table to be modified.
	 * @return  The query string.
	 */
	public String getDeleteStatement(RdbGenTable rgt) throws PException {
		StringBuffer sb = new StringBuffer("DELETE FROM ");
		sb.append(rgt.tableName);
		appendModificationWhereClause(sb, rgt);
		return sb.toString();
	}

	public String getDeleteStatement(RdbGenJoin rgj) throws PException {
		return getDeleteStatement(rgj.getTable());
	}

	/**
	 * Constructs the string representing the SQL query used to delete the part
	 * of a JORM object stored into the given table.
	 * @param rgj   The description of the join between the external table and
	 * the main table
	 * @return  The query string.
	 */
	public String getExtDeleteStatement(RdbGenJoin rgj) throws PException {
		StringBuffer sb = new StringBuffer("DELETE FROM ");
		sb.append(rgj.table.tableName);
		appendModificationWhereClause(sb, rgj, rgj.table.colocatedTable);
		return sb.toString();
	}

	public String setIdPnParam(String cnfn,
							   int idx,
							   RdbAdapter adapter,
							   boolean isSpecific) throws PException {
		RdbGenColumn rgc = (RdbGenColumn) genId.cnFieldColumns.get(cnfn);
		String value;
		if (isSpecific) {
			value = "((" + genId.getCnFQName() + "PNG) _pngId).pnGet"
					+ commonHelper.upperFL(cnfn) + "(null)";
		} else {
			value = "((PNameGetter) _pngId)."
					+ commonHelper.getPNameGetterGetFunction(rgc.columnType)
					+ "(\"" + cnfn + "\", null)";
		}
		return rgc.getSqlSet(value, idx, adapter);
	}

	/**
	 * Searches the RdbGenRef object, from the genRefs list, which the given
	 * column belongs to.
	 * @param rgc   The column for which to find a RdbGenRef.
	 * @return  The relevant RdbGenRef or null if none found.
	 */
	public RdbGenRef getGenRefOfColumn(RdbGenColumn rgc) {
		return getGenRefOfColumn(rgc, null);
	}

	public RdbGenRef getGenRefOfColumn(RdbGenColumn rgc, RdbGenJoin rgj) {
		if (rgc.joinCol != null) {
			return getGenRefOfColumn(rgc.joinCol, rgj);
		}
		PrimitiveElement pe = null;
		if (rgj != null && rgc.joins != null) {
			int idx = rgc.joins.indexOf(rgj);
			if (idx != -1) {
				pe = (PrimitiveElement) rgc.pes.get(idx);
			}
		}
		Iterator it = genRefs.values().iterator();
		while (it.hasNext()) {
			RdbGenRef rgr = (RdbGenRef) it.next();
			if (rgr.refColumn != null) {
				if (rgr.refColumn == rgc && (pe == null
						|| rgr.nd.getFieldName().equals(pe.getName()))) {
					//if (rgj !=null) System.out.println("getGenRefOfColumn(" + rgc.columnName + ", " + rgj.joinIdx + ") return " + rgr.fieldName);
					return rgr;
				}
			} else {
				Iterator it2 = rgr.cnFieldColumns.values().iterator();
				Collection classfields = rgr.nd.getNameRef().getProjection().values();
				while (it2.hasNext()) {
					RdbGenColumn currentRgc = (RdbGenColumn) it2.next();
					if (currentRgc == rgc && (pe == null
							|| classfields.contains(pe.getName()))) {
						//if (rgj !=null) System.out.println("getGenRefOfColumn(" + rgc.columnName + ", " + rgj.joinIdx + ") return " + rgr.fieldName);
						return rgr;
					}
				}
			}
		}
		//if (rgj !=null) System.out.println("getGenRefOfColumn(" + rgc.columnName + ", " + rgj.joinIdx + ") return null");
		return null;
	}

	/**
	 * Verifies if the given column is used in the identifier of this class,
	 * which is defined by genId.
	 * @param rgc   The column for which to verify its belonging to the ID.
	 * @return  true if it belongs to genId.
	 */
	public boolean colInGenId(RdbGenColumn rgc) {
		if (rgc.joinCol != null) {
			return colInGenId(rgc.joinCol);
		}
		if (genId.refColumn != null) {
			if (genId.refColumn == rgc) {
				return true;
			}
		} else {
			Iterator it2 = genId.cnFieldColumns.values().iterator();
			while (it2.hasNext()) {
				if (it2.next() == rgc) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Searches the name of the field which defines the given reference.
	 * @param gr    The reference definition.
	 * @return  The relevant field name.
	 */
	public String getGenRefFieldName(RdbGenRef gr) {
		Iterator it = genRefs.keySet().iterator();
		while (it.hasNext()) {
			String res = (String) it.next();
			if (genRefs.get(res) == gr) {
				return res;
			}
		}
		return null;
	}

	/**
	 * Searches the name of the field in the composite name associated with the
	 * identifier, which is mapped to the given column.
	 * @param rgc   The column to search.
	 * @return  The name of the mapped field.
	 */
	public String getPnFieldName(RdbGenColumn rgc) {
		Iterator it = genId.cnFieldColumns.keySet().iterator();
		while (it.hasNext()) {
			String res = (String) it.next();
			if (rgc == genId.cnFieldColumns.get(res)) {
				return res;
			}
		}
		return null;
	}

	// PRIVATE METHODS

	/**
	 * Appends the definition of a fully qualified name of a field of a table
	 * (example: field F of table T => appends "T.F").
	 * @param sb
	 * @param rgc
	 */
	private void appendFQColName(StringBuffer sb, RdbGenColumn rgc) {
		sb.append(rgc.columnTable.tableName);
		sb.append(".");
		sb.append(rgc.columnName);
	}

	private String getFQColName(RdbGenColumn rgc, String tableSuffix) {
		return rgc.columnTable.tableName + tableSuffix + "." + rgc.columnName;
	}

	/**
	 * Appends the where clause for a given table for the prupose of a
	 * modification of this table (i.e., an update or a delete).
	 * @param sb    The string buffer into which to produce the clause.
	 * @param rgt   The involved table.
	 */
	private void appendModificationWhereClause(StringBuffer sb,
											   RdbGenTable rgt)
			throws PException {
		String sep = "";
		sb.append(" WHERE ");
		if (rgt != referenceTable) {
			throw new PException("umanaged case, the table "
								 + rgt.tableName + " is an external table");
		}
		if (genId.refColumn != null) {
			sb.append(genId.refColumn.columnName);
			sb.append(" = ?");
		} else {
			Iterator it = genId.cnFieldColumns.values().iterator();
			while (it.hasNext()) {
				RdbGenColumn cur = (RdbGenColumn) it.next();
				sb.append(sep);
				sep = " AND ";
				sb.append(cur.columnName);
				sb.append(" = ?");
			}
		}
	}

	/**
	 * Appends the where clause for a given table for the prupose of a
	 * modification of this table (i.e., an update or a delete).
	 * @param sb    The string buffer into which to produce the clause.
	 * @param rgj   The involved join.
	 */
	private void appendModificationWhereClause(StringBuffer sb,
											   RdbGenJoin rgj, boolean isNotJoinColumn) {
		String sep = "";
		sb.append(" WHERE ");
		sep = "";
		if (isNotJoinColumn) {
			for (int i = 0; i < rgj.table.tableColumns.size(); i++) {
				RdbGenColumn rgc = (RdbGenColumn) rgj.table.tableColumns.get(i);
				if (rgc.joinCol == null) {
					sb.append(sep);
					sep = " AND ";
					sb.append(rgc.columnName);
					sb.append(" = ?");
				}
			}
		} else {
			for (int i = 0; i < rgj.joinColumnsInExt.size(); i++) {
				sb.append(sep);
				sep = " AND ";
				sb.append(((RdbGenColumn) rgj.joinColumnsInExt.get(i)).columnName);
				sb.append(" = ?");
			}
		}
	}

	public boolean updatable(RdbGenColumn rgc) {
		return !rgc.hiddenField || rgc.joinCol != null || !colInGenId(rgc);
	}

	public boolean updatable(RdbGenColumn rgc, RdbGenJoin rgj) {
		if (rgj == null) {
			return updatable(rgc);
		}
		if (rgj.table.colocatedTable) {
			return rgc.joinCol != null && rgc.isInJoin(rgj);
		} else {
			return rgc.joinCol == null;
		}
	}
}
