/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.generator;

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbExternalTable;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbJoin;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbPrimitiveElementMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbTable;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.HashMap;
import java.util.Map;

/**
 * Defines one of the table that is used to mapped a particular JORM class.
 * In case of the reference table (see RdbGenInfos), no join information is
 * defined.
 * @author  P. Dechamboux
 */
public class RdbGenTable {
	public RdbGenInfos genInfos = null;
	/**
	 * The name of the table involved in this mapping.
	 */
	public String tableName = null;
	/**
	 * The list of columns of this table used in this mapping.
	 */
	public List tableColumns = new ArrayList();
	/**
	 * The list of columns of this table that are shared with a colocated
	 * object (i.e., an object stored into the same table along with this
	 * object.
	 */
	public List colocatedColumns = null;
	/**
	 * Tells that this table stores colocated objects.
	 */
	public boolean colocatedTable = false;

	public List joins = null;
	public Map jn2join = null;

	public boolean readOnly = false;

	public boolean generateUpdate = true;

	public boolean isMainTable;

	public RdbGenTable() {
	}

	public RdbGenTable(RdbExternalTable mo,
					   RdbGenTable mainTable) throws PException {
		this(mo);
		isMainTable = false;
		readOnly = mo.isReadOnly();
		joins = new ArrayList();
		jn2join = new HashMap();
		int joinIdx = 0;
		for (Iterator itj = mo.getRdbJoins().iterator(); itj.hasNext();) {
			RdbJoin j = (RdbJoin) itj.next();
			RdbGenJoin rgj = new RdbGenJoin(mainTable, this, j, joinIdx);
			joinIdx++;
			jn2join.put(j.getName(), rgj);
			joins.add(rgj);
		}
		for (Iterator it = mo.getPrimitiveElementMappings().iterator(); it.hasNext();) {
			RdbPrimitiveElementMapping rpem = (RdbPrimitiveElementMapping) it.next();
			Iterator entryIt = rpem.getPrimitiveElementByRdbJoin().entrySet().iterator();
			if (!entryIt.hasNext()) {
				continue;
			}
			if (rpem.getName() == null) {
				throw new PException("Primitive element "
									 + ((PrimitiveElement) rpem.getLinkedMO()).getName()
									 + " has a null column name");
			}
			RdbGenColumn rgc = getColumn(rpem.getName());
			//System.out.println("Column " + rgc.columnName);
			rgc.pes = new ArrayList();
			rgc.joins = new ArrayList();
			PrimitiveElement pe = null;
			while (entryIt.hasNext()) {
				Map.Entry me = (Map.Entry) entryIt.next();
				RdbJoin j = (RdbJoin) me.getKey();
				pe = (PrimitiveElement) me.getValue();
				//System.out.println("pe: " + pe.getName() + " / join: " + j);
				rgc.pes.add(pe);
				rgc.joins.add(jn2join.get(j.getName()));
			}
			rgc.hiddenField = pe instanceof ScalarField;
			rgc.columnType = pe.getType();
		}
	}

	public RdbGenTable(RdbTable mo) throws PException {
		tableName = mo.getName();
		isMainTable = true;
		//System.out.println("Definition of the table: " + tableName);
		colocatedTable = mo.isColocated() && !mo.isColocatedMaster();
		//System.out.println("colocatedTable: " + colocatedTable);
		for (Iterator it = mo.getPrimitiveElementMappings().iterator(); it.hasNext();) {
			RdbPrimitiveElementMapping rpem = (RdbPrimitiveElementMapping) it.next();
			RdbGenColumn rgc = new RdbGenColumn();
			rgc.columnName = rpem.getName();
			if (rgc.columnName == null) {
				throw new PException("Primitive element "
									 + ((PrimitiveElement) rpem.getLinkedMO()).getName()
									 + " has a null column name");
			}
			//System.out.println("contains the column: " + rgc.columnName);
			rgc.columnNotNull = rpem.isNotNull();
			rgc.columnTable = this;
			PrimitiveElement pe = (PrimitiveElement) rpem.getLinkedMO();
			if (pe != null) {
				rgc.columnType = pe.getType();
				rgc.columnSize = pe.getSize();
				rgc.hiddenField = pe instanceof ScalarField;
				rgc.fieldName = pe.getName();
			}
			rgc.columnSqlType = rpem.getType();
			if (rgc.columnSqlType != null && rgc.columnSqlType.length() == 0) {
				rgc.columnSqlType = null;
			}
			tableColumns.add(rgc);
		}
	}

	public void setGenerateUpdate(RdbGenInfos rgi) {
		generateUpdate = false;
		for (int i = 0; i < tableColumns.size() && !generateUpdate; i++) {
			RdbGenColumn rgc = (RdbGenColumn) tableColumns.get(i);
			if (rgc.joinCol != null) {
				continue;
			}
			if (joins == null && rgi.colInGenId(rgc)) {
				continue;
			}
			generateUpdate = true;
		}
	}

	public boolean getGenerateUpdate() {
		return generateUpdate;
	}

	public boolean getReadOnly() {
		return readOnly;
	}

	public List getJoins() {
		return joins;
	}

	public RdbGenInfos getGenInfos() {
		return genInfos;
	}

	public String getTableName() {
		return tableName;
	}

	public String getTableNameNoDot() {
		return tableName.replace('.', '_');
	}

	public List getTableColumns() {
		return tableColumns;
	}

	public List getColocatedColumns() {
		return colocatedColumns;
	}

	public boolean getColocatedTable() {
		return colocatedTable;
	}

	public boolean isMainTable() {
		return isMainTable;
	}

	/**
	 * Specifies if only prepared statement can be used with this table mapping.
	 */
	private Boolean psMand = null;

	/**
	 * Tells if only PreparedStatement can be used to store data associated
	 * with this table.
	 * @return  true if the use of PreparedStatement is mandatory.
	 */
	public boolean preparedStatementMandatory() {
		if (psMand == null) {
			for (int i = 0; i < tableColumns.size(); i++) {
				if (((RdbGenColumn) tableColumns.get(i)).columnType == PTypeSpace.SERIALIZED) {
					psMand = new Boolean(true);
					return true;
				}
			}
			psMand = new Boolean(false);
		}
		return psMand.booleanValue();
	}

	/**
	 * Tells if the given column belongs to a colocated ones, which means that
	 * it is shared with the colocated object and must not be touched (e.g.
	 * should not be nullified).
	 * @param rgc   The involved column.
	 * @return  true if it is a colocated column.
	 */
	boolean colocatedColumn(RdbGenColumn rgc) {
		if (colocatedColumns != null) {
			for (int i = 0; i < colocatedColumns.size(); i++) {
				if (colocatedColumns.get(i) == rgc) {
					return true;
				}
			}
		}
		return false;
	}

	public RdbGenColumn getColumn(String name) {
		RdbGenColumn rgc = null;
		for (int i = 0; i < tableColumns.size(); i++) {
			rgc = (RdbGenColumn) tableColumns.get(i);
			if (rgc.columnName.equals(name)) {
				return rgc;
			}
		}
		return null;
	}
}
