/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.api;

import java.util.Iterator;

/**
 * The NameDef interface describes the mapping of a reference field,
 * ut can be the PName of a Class. It can be a Field which is an attribute
 * of the class and is a reference; in this case, the reference can be
 * to another Class (ClassObject) or to a GenClass.
 * A NameDef object is composed of ScalarField(s) and/or FieldName(s),
 * or system defined.
 * @author	X. Spengler
 */
public interface NameDef extends MetaObject {

	/**
	 * Allows to know if the namedef is describe by a composite name.
	 * @return true the namedef is a composite one.
	 */
	boolean isNameRef();

	/**
	 * Allows to know if the namedef is describe by a single field.
	 * @return true the namedef is a single one.
	 */
	boolean isFieldName();

	/**
	 * Builds a reference toward a composite name describing the namedef.
	 * @param cn_Class the meta object representing the composite name.
	 * @return the nameref metaobject describing the reference.
	 */
	NameRef createNameRef(CompositeName cn_Class);

	/**
	 * Retrieves the nameref if the namedef is a composite one.
	 * @return the nameref or null if the namedef is a single one or system.
	 */
	NameRef getNameRef();

	/**
	 * Retrieves the name of the field describing the namedef.
	 * The fieldname can be the name of a field or an hiddenfield.
	 * @return the fieldname or null is the namedef is a system or a composite one.
	 */
	String getFieldName();

	/**
	 * Allows to set the fieldname describing the namedef single field.
	 * @param fieldname the fieldname
	 */
	void setFieldName(String fieldname);

	/**
	 * Returns the name of the NameDef object.
	 * @return the name of the NameDef object.
	 */
	public String getName();

	/**
	 * Sets the name of the NameDef object.
	 * @param name the name of the NameDef object.
	 */
	public void setName(String name);

	/**
	 * Allows to know if the mapping of the referenced field is defined by
	 * another system (could be defined by Versant system for instance).
	 * @return	true, if the mapping is defined by another system, else false,
	 *          if the mapping is defined inside JORM (by ScalarField(s) and
	 *          FieldName(s)).
	 */
	boolean isSystem();

	/**
	 * Allows to set if the current NameDef is system defined (defined outside
	 * jorm) or not.
	 * @param system	true, if the mapping is defined by another system, else
	 *	        		false, if the mapping is defined inside JORM (by
	 *                  ScalarField(s) and FieldName(s)).
	 */
	void setSystem(boolean system);

	/**
	 * Returns the number of  fields which composed the current object.
	 * these fields are hidden or not
	 * @return	the number of  fields
	 */
	int getFieldNumber();

	/**
	 * Returns an iterator on all fields (HiddenFields and FieldRef).
	 * This iterator contains FieldName . If no object
	 * fields exists, an empty iterator is returned.
	 * @return  an iterator on field
	 */
	Iterator iterateField();

}
