/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.metainfo.api.Extension;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.*;

public class BasicExtension extends BasicMetaObject
		implements Extension, Loggable {

    /**
     * the name of the class reference
     */
    private Class inheritedClass;

    /**
     * a simple logger to log
     */
    private transient Logger logger;

    /**
     * the list of the name definition declared by the user in the
     * current reference to the class.
     * value: a NameDef object
     */
    private ArrayList nameDefs;


    /**
     * Builds a new BasicClassRef object.
     * This object is defined by its name, the reference to the class and
     * its parent.
     *
     * @param inheritedClass	the reference to the class
     * @param parent	the parent of the current object
     */
    public BasicExtension(Class inheritedClass, MetaObject parent) {
        super(parent);
        this.inheritedClass = inheritedClass;
        nameDefs = new ArrayList();
    }

	///////////////////////////////////////////////////////////////////
	//                      from ClassRef interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the Class object which describes the ClassRef entity.
	 * This object is used as the type of a field.
	 *
	 * @return  the Class Object contained by the current class ref object
	 */
	public Class getMOClass() {
		return inheritedClass;
	}

	/**
	 * Returns the name of the class ref.
	 *
	 * @return the name of the class reference meta object
	 */
	public String getClassName() {
		return inheritedClass.getName();
	}

	/**
	 * Returns the fully qualified name of the class ref.
	 *
	 * @return the name of the class reference meta object
	 */
	public String getFQName() {
		return inheritedClass.getFQName();
	}

	///////////////////////////////////////////////////////////////////
	//                      from TypedElement interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the name of the current typed element.
	 *
	 * @return	the string representation of the typed element
	 */
	public String getName() {
		return "";
	}

	/**
	 * Allows to know the type of the field.
	 *
	 * @return	a PType object which defines the type of the current field
	 */
	public PType getType() {
		try {
			if (inheritedClass.getInheritedClassNumber() == 0) {
				// no super class, just use the name of the class.
				return getManager().getPTypeSpace().createPType(inheritedClass.getFQName());
			} else { // minimum 1 super class
				Map table = new HashMap();
				inheritedClass.addAllInheritedClass(table);
				String allNames [][] = new String[table.size()][];
				int count = 0;

				Iterator iter = table.values().iterator();
				while (iter.hasNext()) {
					String[] names = (String[]) iter.next();
					allNames[count][0] = names[0];
					allNames[count] = names;
				}
				return getManager().getPTypeSpace().createPType(
						inheritedClass.getFQName(),
						allNames);
			}
		} catch (Exception e) {

		}
		return null;
	}

	///////////////////////////////////////////////////////////////////
	//                      from Reference interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Creates a new NameDef object for the current reference class object.
	 * If this NameDef already exists for the current object, it is returned,
	 * in other case, a new NameDef object is created and returned.
	 *
	 * @return	a new object used to describe the name projection for
	 *	      	the current reference class, or an existing one if it
	 *          already exists
	 */

	public NameDef createRefNameDef() {
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "create a new NameDef () for the current Extension (" +
                        inheritedClass.getFQName() + ")");
		// in the case it does not exist, we create it.
		BasicNameDef newNameDef = new BasicNameDef(this);
		newNameDef.setLogger(logger);
		nameDefs.add(newNameDef);
		return newNameDef;
	}

	/**
	 * Returns a NameDef object.
	 * If the namedef does not exist, null is returned.
	 *
	 * @param name   the name of the NameDef object.
	 * @return  an existing namedef object, else null is returned.
	 */
	public NameDef getRefNameDef(String name) {
        if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "get a NameDef () " + name +  " for the current Extension (" +
                        inheritedClass.getFQName() + ")");
		for (Iterator iter = nameDefs.iterator(); iter.hasNext();) {
			NameDef nameDef = (NameDef) iter.next();
			if (nameDef.getName().equals(name)) {
				return nameDef;
			}
		}
		return null;
	}

	/**
	 * Returns an iterator on existing NameDef for the current Class.
	 * This method returns an empty iterator if no namedef exists in this Class.
	 *
	 * @return	an iterator on NameDef, or an empty iterator if no namedef is
	 *          defined
	 */

	public Collection getRefNameDef() {
		return nameDefs;
	}


	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 *
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
		for (Iterator it = getRefNameDef().iterator(); it.hasNext();) {
			((Loggable) it.next()).setLogger(logger);
		}
	}

	/**
	 * Defines the logger factory to obtain new logger.
	 *
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory) {
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return null;
	}
}
