/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.metainfo.api.GenClass;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

/**
 * BasicGenClass is the implementation of the GenClass
 * interface defined in the Meta information system.
 * This object is defines by a name, a list of index fields,
 * and its parent.
 * @author	X. Spengler
 */
public class BasicGenClass implements GenClass, Loggable {
	/**
	 * name is the string representation of the name of the Generic Class
	 */
	private String name;

	/**
	 * the isAbstract property defines if the current generic
	 * class is abstract or not.
	 */
	private boolean isAbstract;

	/**
	 * the parent meta object is the parent of the current generic class.
	 */
	private MetaObject parent;

	/**
	 * indexFields is a list of index fields for the generic class.
	 * key: the name of the index field
	 * value: a PrimitiveElement
	 */
	private Map indexFields;

	/**
	 * a simple logger to log
	 */
	private transient Logger logger;

	/**
	 * Builds a new BasicGenClass object.
	 * This object is built with its name, and its parent.
	 *
	 * @param name	        the name of the generic class
	 * @param isAbstract	true, the generic class is an abstract class, else
	 *	        			false, the generic class is not an abstract class
	 * @param parent    the parent meta-object of the current GenClass
	 */
	public BasicGenClass(String name, boolean isAbstract, MetaObject parent) {
		this.name = name;
		this.isAbstract = isAbstract;
		this.parent = parent;
		indexFields = new HashMap();
	}

	///////////////////////////////////////////////////////////////////
	//                      from GenClass interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Allows to know the name of the current generic class object.
	 *
	 * @return	the string representation of the generic class name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns a new PrimitiveElement object, created with its name
	 * and its type (PType).
	 * As the current Class is generic, the field of the class is an
	 * index field. If the field already exists, it is returned, else a new
	 * one is created.
	 *
	 * @param indexFieldName	the name of the index field
	 * @param type	            the type of the index field
	 * @return	a new PrimitiveField for the current generic class
	 */
	public PrimitiveElement createIndexField(String indexFieldName,
											 PType type) {
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG,
					   "create a new PrimitiveElement (index field (" +
					   indexFieldName + ")) for the current " + "GenClass (" +
					   this.name + ")");

		// verify if the object already exists in the system
		PrimitiveElement result = (PrimitiveElement)
				indexFields.get(indexFieldName);
		if (result != null) {
			logger.log(BasicLevel.WARN,
					   "attention, try to create an existing PrimitiveElement (" +
					   name + "), return existing one");
		} else {
			// else create it
			result = new BasicPrimitiveElement(indexFieldName, type, PType.NOSIZE, this);
			((Loggable) result).setLogger(logger);
			indexFields.put(indexFieldName, result);
		}
		return result;
	}

	/**
	 * Returns the PrimitiveElement corresponding to its name.
	 * If no field corresponds to this name, null is returned.
	 *
	 * @param name	the name of the index field
	 * @return	an existing index field. If the index field does not exist,
	 *	    	null is returned.
	 */
	public PrimitiveElement getIndexField(String indexFieldName) {
		return (PrimitiveElement) indexFields.get(indexFieldName);
	}

	/**
	 * Allows to know all the existing index field for the current generic
	 * class.
	 * This iterator contains PrimitiveElement objects. If no index field
	 * exists, an empty iterator is returned.
	 *
	 * @return	an Iterator on index field (PrimitiveElement). If there is no
	 *          index field to return, an empty iterator is returned.
	 */
	public Iterator iterateIndexField() {
		return indexFields.values().iterator();
	}

	/**
	 * Returns the number of index fields for the current generic class.
	 *
	 * @return  the integer number of the index fields for the generic class
	 */
	public int getIndexFieldNumber() {
		return indexFields.size();
	}

	///////////////////////////////////////////////////////////////////
	//                      from MetaObject interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Allows to know the parent MetaObject of the current MetaObject.
	 *
	 * @return	the MetaObject corresponding to the parent of the
	 *  		current object. If there is no parent, null is returned.
	 */
	public MetaObject getParent() {
		return parent;
	}

	/**
	 * Set the parent of the current meta object if it is not yet done
	 * by the constructor of the meta object
	 * @param itsParent	the parent MetaObject of the current object
	 */
	public void setParent(MetaObject itsParent) {
		this.parent = itsParent;
	}

	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 *
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
		for (Iterator it = this.iterateIndexField(); it.hasNext();) {
			((Loggable) it.next()).setLogger(logger);
		}
	}

	/**
	 * Defines the logger factory to obtain new logger.
	 *
	 * @param loggerFactory	the LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerFactory) {
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return null;
	}
}

