/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.HelperClassMapping;
import org.objectweb.jorm.metainfo.api.IdentifierMapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.PrimitiveElementMapping;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.ReferenceMapping;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Collection;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Collections;

/**
 * Description
 */
public class BasicHelperClassMapping extends BasicMappingStructure
		implements HelperClassMapping {
	/**
	 * The name of the rule used to map the class.
	 */
	private String ruleName;

	/**
	 * A Map of PrimitiveElementMapping objects.
	 * key = field name
	 * value = the PrimitiveElementMapping
	 */
	private Map primitiveElementMappings;

	/**
	 * A NameDef object.
	 * This is the NameDef object of the class.
	 */
	private IdentifierMapping identifierMapping;

	private Set dependencies;

	/**
	 * Builds a new BasicHelperClassMapping object.
	 *
	 * @param ruleName   the name of the rule used to map the class,
	 *        linkedMO	 the object referenced by the current object,
	 *        parent	 the parent of the current object.
	 */
	public BasicHelperClassMapping(String ruleName, MetaObject linkedMO,
								   MetaObject parent) {
		super(parent, linkedMO);
		this.ruleName = ruleName;
		primitiveElementMappings = new HashMap();
		identifierMapping = null;
		dependencies = new HashSet();
	}

	///////////////////////////////////////////////////////////////////
	//                      from HelperClassMapping interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the  name of the rule used to map the class.
	 * @return	 the name of the rule.
	 */
	public String getRuleName() {
		return this.ruleName;
	}

	/**
	 * Sets the name of the rule used to map the class.
	 * @param rulename the name of the rule.
	 */
	public void setRuleName(String rulename) {

		this.ruleName = rulename;
	}

	/**
	 * Returns the IdentifierMapping object.
	 * @return	an IdentifierMapping object.
	 */
	public IdentifierMapping getIdentifierMapping() {
		return this.identifierMapping;
	}

	/**
	 * Sets the IdentifierMapping object.
	 * @param idmapping the IdentifierMapping object.
	 */
	public void setIdentifierMapping(IdentifierMapping idmapping) {
		this.identifierMapping = idmapping;
	}

	/**
	 * Returns a collection of PrimitiveElementMapping objects.
	 * @return  a collection.
	 */
	public Collection getPrimitiveElementMappings() {
		List list = null;
		try {
			list = new ArrayList(primitiveElementMappings.values());
			Collections.sort(list, FieldComparator.instance);
		} catch (Exception e) {
			e.printStackTrace();  //To change body of catch statement use Options | File Templates.
		}
		return list;
	}

	public PrimitiveElementMapping getPrimitiveElementMapping(String fieldName) {
		return (PrimitiveElementMapping)
				primitiveElementMappings.get(fieldName);
	}

	/**
	 * Adds a PrimitiveElementMapping object.
	 * @param  peMapping  the PrimitiveElementMapping object to add to the list.
	 */
	public void addPrimitiveElementMapping(PrimitiveElementMapping peMapping) {
		addPrimitiveElementMapping(
				((PrimitiveElement) peMapping.getLinkedMO()).getName(), peMapping);
	}

	/**
	 * Adds a PrimitiveElementMapping object.
	 * @param  peMapping  the PrimitiveElementMapping object to add to the list.
	 */
	public void addPrimitiveElementMapping(String fieldName, PrimitiveElementMapping peMapping) {
		primitiveElementMappings.put(fieldName, peMapping);
	}

	/**
	 * Returns an Iterator over PrimitiveElementMapping objects.
	 * @return  an Iterator.
	 */
	public Iterator iteratePrimitiveElementMappings() {
		return primitiveElementMappings.values().iterator();
	}

	/**
	 * Creates a new IdentifierMapping object.
	 * @param   nd	the NameDef object that defines an object identifier.
	 * @return	an IdentifierMapping object.
	 */
	public IdentifierMapping createIdentifierMapping(NameDef nd) {
		IdentifierMapping res = getIdentifierMapping();
		if (res == null) {
			res = new BasicIdentifierMapping(nd, this);
			setIdentifierMapping(res);
		}
		return res;
	}

	/**
	 * Creates a new ReferenceMapping object.
	 * @param ruleName  the name of the rule used to map the reference,
	 *        nd	    the NameDef object that defines an object reference.
	 * @return	a ReferenceMapping object.
	 */
	public ReferenceMapping createReferenceMapping(String ruleName, NameDef nd) {
		return new BasicReferenceMapping(ruleName, nd, this);
	}

	public void addDependency(String jormClassName) {
		dependencies.add(jormClassName);
	}

	public void removeDependency(String jormClassName) {
		dependencies.remove(jormClassName);
	}

	public Collection getDependencies() {
		return dependencies;
	}
}
