/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mi2xml.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.util.io.api.TargetHolder;
//import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.lib.JormConfigurator;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.mi2xml.api.MappingDomtreeBuilder;
import org.objectweb.jorm.mi2xml.api.DomtreeBuilder;
import org.objectweb.jorm.mi2xml.api.Writer;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Iterator;

import org.w3c.dom.Document;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;

import java.io.*;

/**
 * BasicDomWriter implements the Writer interface.
 * BasicDomWriter generates the .pd file of a Class or a CompositeName object.
 */
public class BasicDomWriter implements Writer, Loggable {

	//TODO
	private static final String ENCODING = "ISO-8859-1";
	private static final String DOCTYPE_PUBLIC =
			"-//ObjectWeb Consortium//DTD JORM DEFINITIONS 2.0//EN";
	private static final String JORM_DTD =
			"http://www.objectweb.org/jorm/dtds/jorm2.dtd";

	/**
	 * Default indentation for xml viewer.
	 */
	public static final String XML_INDENT = "4";

	//private PathExplorer pathExplorer = null;

	private Logger logger = null;
	private LoggerFactory loggerFactory = null;

	private DomtreeBuilder domtreeBuilder;

	/**
	 * Initializes a BasicDomWriter object.
	 * @param compilerConfigurator  a JormConfigurator object.
	 */
	public void init(JormConfigurator compilerConfigurator)
			throws PException {
		domtreeBuilder = new BasicDomtreeBuilder();
		((Loggable) domtreeBuilder).setLoggerFactory(loggerFactory);
		((Loggable) domtreeBuilder).setLogger(loggerFactory.getLogger("org.objectweb.jorm.mi2xml."));
		Iterator it = compilerConfigurator.knownMappers();
		while (it.hasNext()) {
			String mappername = (String) it.next();
			logger.log(BasicLevel.DEBUG,
					   "Register specific mapping domtree builder for mapper [" + mappername + "]");
			MappingDomtreeBuilder mdb = compilerConfigurator.getMappingDomtreeBuilder(mappername);
			((Loggable) mdb).setLoggerFactory(loggerFactory);
			((Loggable) mdb).setLogger(loggerFactory.getLogger("org.objectweb.jorm.mi2xml." + mappername));
			domtreeBuilder.addMappingDomtreeBuilder(mappername, mdb);
		}
	}

	/**
	 * This method first builds the DOMTree of a Class or a CompositeName object and
	 * then generates the .pd file by delegation to the XMLSerializer.
	 * @param mo		a Class or a CompositeName object.
	 * @param holder	The target holder which allows to create files.
	 * @exception PException if an error occurs during the generation.
	 */
	public void write(MetaObject mo, TargetHolder holder)
			throws PException {
		String moFQName = null;
		String moFileName = null;
		String name = null;
		if (mo instanceof Class) {
			moFQName = ((Class) mo).getFQName();
			moFileName = ((Class) mo).getFileName();
			name = ((Class) mo).getName();
		} else if (mo instanceof CompositeName) {
			moFQName = ((CompositeName) mo).getFQName();
			//fileName = ((CompositeName) mo).getFileName();
			name = ((CompositeName) mo).getName();
		}
		logger.log(BasicLevel.DEBUG, "FQName():" + moFQName
									 + " / filename=[" + moFileName + "]");
		logger.log(BasicLevel.DEBUG, "let's build the domtree !");
		// generate the DOMTree for a Class or a CompositeName object.
		Document domTree = domtreeBuilder.build(mo);
		logger.log(BasicLevel.DEBUG, name + "DOMTree generated");
		// File name
		String fileName = File.separatorChar + name;
		String packName = ((Package) mo.getParent()).getName();
		if (packName != null && packName.length() > 0) {
			fileName = packName + File.separatorChar + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate " + fileName + ".pd");
		write(domTree, holder, fileName);
	}

	/**
	 * This method generates a .pd file from a DOMTree object.
	 * @param domTree	the DOMTree of a Class or a CompositeName object.
	 * @param holder	The target holder which allows to create files.
	 * @exception PException if an error occurs during the generation.
	 */
	private void write(Document domTree, TargetHolder holder, String fileName)
			throws PException {
		try {
			// outputformat properties
			OutputFormat outf = new OutputFormat(domTree);
			outf.setIndenting(true);
			outf.setIndent(Integer.parseInt(XML_INDENT));
			outf.setOmitXMLDeclaration(false);
			//TODO
			outf.setEncoding(ENCODING);
			outf.setDoctype(DOCTYPE_PUBLIC, JORM_DTD);
			FileWriter fw = holder.getFileWriter(fileName + ".pd");
			BufferedWriter buf = new BufferedWriter(fw);
			// serialize
			XMLSerializer xmls = new XMLSerializer(buf, outf);
			xmls.asDOMSerializer();
			xmls.serialize(domTree);
			fw.flush();
			fw.close();
			logger.log(BasicLevel.DEBUG, fileName + ".pd generated");
		} catch (Exception e) {
			e.printStackTrace();
			throw new PException(e, "Problem while writing .pd file.");
		}
	}

	///////////////////////////////////////////////////////////////////
	//                      from Loggable interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Defines a logger object.
	 *
	 * @param logger	the logger object
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Defines the logger factory to obtain new loggers.
	 *
	 * @param loggerfactory	The LoggerFactory object to obtain a logger object
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		this.loggerFactory = loggerfactory;
		if (logger == null && loggerFactory != null) {
			logger = loggerFactory.getLogger("org.objectweb.jorm.mi2xml.rdb");
		}
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}
