/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.naming.api;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.type.api.PType;

import java.util.Date;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * The <b>PName</b> interface defines the behaviour of a persistent name as
 * it is managed by its associated naming context. Indeed, such a behaviour
 * is supported partly within the name implementation, and partly within its
 * naming context.
 * @author	R. Basset, P. D?chamboux
 */
public interface PName extends Cloneable, Serializable {
	/**
	 * It tests if a given coding type is supported for this PName.
	 * @param codingtype	One of the coding type defined by
	 * 						PNamingContext.CTxxx constants defined within the
	 * 						PNamingContext interface.
	 * @return	It returns true if the coding type is supported for this PName.
	 */
	boolean codingSupported(short codingtype);

	/**
	 * The <b>encode</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is an array of bytes.
	 * @return	The array of bytes corresponding to the encoded representation.
	 */
	byte[] encode() throws PExceptionNaming;

	/**
	 * The <b>encodeAbstract</b> method produces an encoded representation of
	 * this PName. The encoding format is the case of this method is a Java
	 * object of any complexity. Example of such object is a getter interface
	 * giving access of the fields of a composite relational primary key.
	 * @return	The Object corresponding to the encoded representation.
	 */
	Object encodeAbstract() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeByte</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a byte.
	 * @return	The byte corresponding to the encoded representation.
	 */
	byte encodeByte() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeObyte</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Byte.
	 * @return	The Byte corresponding to the encoded representation.
	 */
	Byte encodeObyte() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeChar</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a char.
	 * @return	The char corresponding to the encoded representation.
	 */
	char encodeChar() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOchar</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Character.
	 * @return	The Character corresponding to the encoded representation.
	 */
	Character encodeOchar() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeInt</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a int.
	 * @return	The int corresponding to the encoded representation.
	 */
	int encodeInt() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOint</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Integer.
	 * @return	The Integer corresponding to the encoded representation.
	 */
	Integer encodeOint() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeLong</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a long.
	 * @return	The long corresponding to the encoded representation.
	 */
	long encodeLong() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOlong</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Long.
	 * @return	The Long corresponding to the encoded representation.
	 */
	Long encodeOlong() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeShort</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a short.
	 * @return	The short corresponding to the encoded representation.
	 */
	short encodeShort() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeOshort</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Short.
	 * @return	The Short corresponding to the encoded representation.
	 */
	Short encodeOshort() throws PExceptionNaming, UnsupportedOperationException;

	/**
	 * The <b>encodeString</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a String.
	 * @return	The String corresponding to the encoded representation.
	 */
	String encodeString() throws PExceptionNaming;

	/**
	 * The <b>encodeCharArray</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a char[].
	 * @return	The char[] corresponding to the encoded representation.
	 */
	char[] encodeCharArray() throws PExceptionNaming;

	/**
	 * The <b>encodeDate</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a Date.
	 * @return	The Date corresponding to the encoded representation.
	 */
	Date encodeDate() throws PExceptionNaming;

	/**
	 * The <b>encodeBigInteger</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a BigInteger.
	 * @return	The BigInteger corresponding to the encoded representation.
	 */
	BigInteger encodeBigInteger() throws PExceptionNaming;

	/**
	 * The <b>encodeBigDecimal</b> method produces an encoded representation of this
	 * PName within its naming context. The encoding format in the case of this
	 * method is a BigDecimal.
	 * @return	The BigDecimal corresponding to the encoded representation.
	 */
	BigDecimal encodeBigDecimal() throws PExceptionNaming;

	/**
	 * It associates a PName to an information item within a naming context.
	 * @param conn	The connection that can be used to access the data store.
	 * @param pnc	Thhe naming context which must provide a PName for
	 * 				this PName.
	 * @return	The new PName associated to this PName.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	PName export(Object conn, PNamingContext pnc) throws PException;

	/**
	 * It associates a PName to an information item within a naming context.
	 * @param conn	The connection that can be used to access the data store.
	 * @param pnc	Thhe naming context which must provide a PName for
	 * 				this PName.
	 * @param hints	Any information relevant for name creation.
	 * @return	The new PName associated to this PName.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	PName export(Object conn, PNamingContext pnc, Object hints)
			throws PException;

	/**
	 * It retrieves the name manager associated to this PName.
	 * @return	The naming context which provide this pname
	 */
	PNameManager getPNameManager();

	/**
	 * It retrieves the PType associated to this PName if any exists. If none
	 * is associated, it returns null.
	 * @return	The PType associated to this PName.
	 */
	PType getPType();

	/**
	 * It tells if this name corresponds to "null" representation within a
	 * its naming context.
	 * @return	true if it represents a "null" name.
	 */
	boolean isNull();

	/**
	 * It retrieves the information item associated to the given PName (if any
	 * exists).
	 * @param conn	The connection that can be used to access the data store.
	 * @return	The information item associated with the given PName within
	 * 			a naming context.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	PName resolve(Object conn) throws UnsupportedOperationException, PException;

	/**
	 * It removes this PName and all the associated information within a naming
	 * context (i.e., the association between the PName and its information
	 * item).
	 * @param conn	The connection that can be used to access the data store.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	void unexport(Object conn) throws PException;

	/**
	 * It removes this PName and all the associated information within a naming
	 * context (i.e., the association between the PName and its information
	 * item).
	 * @param conn	The connection that can be used to access the data store.
	 * @param hints	Any information relevant for name removing.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	void unexport(Object conn, Object hints) throws PException;
}
