/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.naming.api;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;

/**
 * The <b>PNameManager</b> interface provides a means to create, remove
 * and manage PName. It ensures that each of the names it manages unambiguously
 * designates a particular information item (another persistent name or a
 * binding for instance).
 * @author	R. Basset, P. D?chamboux
 */
public interface PNameManager extends PNameCoder {
	/**
	 * It associates a PName to an information item within a naming context.
	 * @param conn		The connection that can be used to access the data store.
	 * @param infoitem	The information item that must be accessible through a
	 * 					PName within a naming context.
	 * @return	The <code>PName</code> associated to the given information
	 * 			item.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 *							the data store.
	 */
	PName export(Object conn, Object infoitem) throws PException;

	/**
	 * It associates a PName to an information item within a naming context.
	 * @param conn		The connection that can be used to access the data store.
	 * @param infoitem	The information item that must be accessible through a
	 * 					PName within a naming context.
	 * @param hints		Any information relevant for name creation.
	 * @return	The <code>PName</code> associated to the given information
	 * 			item.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 *							the data store.
	 */
	PName export(Object conn, Object infoitem, Object hints)
			throws PException;

	/**
	 * It removes the PName and all the associated information within a naming
	 * context (i.e., the association between the PName and its information
	 * item).
	 * @param conn	The connection that can be used to access the data store.
	 * @param pn	A PName valid in this naming context. In case of a PBinder,
	 * 				export is done on a PBinding.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	void unexport(Object conn, PName pn) throws PException;

	/**
	 * It removes the PName and all the associated information within a naming
	 * context (i.e., the association between the PName and its information
	 * item).
	 * @param conn	The connection that can be used to access the data store.
	 * @param pn	A PName valid in this naming context. In case of a PBinder,
	 * 				export is done on a PBinding.
	 * @param hints		Any information relevant for name removing.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @exception PExceptionIO	Raised in case of problem during the access of
	 * the data store
	 */
	void unexport(Object conn, PName pn, Object hints) throws PException;
}
