

/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNameCoding;
import org.objectweb.jorm.naming.api.PExceptionExistingName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PNCStringCoder;
import org.objectweb.jorm.naming.lib.BasicPNamingContext;
import org.objectweb.jorm.naming.lib.BasicStringCoder;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.cache.api.FixableCacheEntry;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.Iterator;

public class PolymorphIdBinder
	extends BasicPNamingContext
	implements PBinder, PNamingContext {
    protected CacheManager bindingcache = null;
	private PClassMapping pcmbinder = null; //pclassmapping using this pbinder
    private PName nullPName = null;

	private long jormclassId = (long) -1;
	private long jormobjectId = (long) -1;

    /**
	 * A constructor that must be initialized through the setPClassMapping method.
	 */
    public PolymorphIdBinder() {
	}

    /**
	 * A constructor that need not to be initialized through the init method.
	 */
    public PolymorphIdBinder(PClassMapping pcm) {
		this.pcmbinder = pcm;
	}

    /**
	 * It initialize this pbinder by  setting the pclassmapping for which it is the binder.
	 * @param pcm    The pclassmapping related to the pbinder.
	 */
	public void setPClassMapping(PClassMapping pcm) {
		this.pcmbinder = pcm;
	}

    public void setNullPName(Object o) throws PException {
        if (o instanceof PolymorphIdPName) {
            nullPName = (PolymorphIdPName) o;
        } else if (o instanceof  PolymorphIdPNG) {
            nullPName = new PolymorphIdPName(((PolymorphIdPNG)o).pnGetClassId(null),((PolymorphIdPNG)o).pnGetObjectId(null), this);
        } else if (o instanceof  PNameGetter) {
            nullPName = new PolymorphIdPName(((PNameGetter)o).pngetLongField("classId", null),((PNameGetter)o).pngetLongField("objectId", null), this);
        }
    }
    // IMPLEMENTATION OF METHODS FROM THE PBinder INTERFACE

	public PBinding lookup(PName pn) throws PException {
		if (pn == null)
			throw new PExceptionNaming("This pname is null");
		if (pn.isNull())
			return null;
        if (bindingcache == null) {
            throw new PExceptionProtocol("No internal cache: should be managed externally!");
        }
        CacheEntry ce = bindingcache.lookup(pn);
        if (ce == null) {
            return null;
        }
        return (PBinding) ce.getCeObject();
	}

	public void bind(PName pn, PBindingCtrl pb) throws PException {
		if (pn == null)
			throw new PExceptionNaming("This pname is null");
		if (pb == null)
			throw new PExceptionNaming("This pbinding is null");
		if (!(pn instanceof PolymorphIdPName))
			throw new PExceptionNaming("Can only bind PolymorphIdPName");
		if (!pn.getPNameManager().equals(this))
			throw new PExceptionNaming("This pname is not valid in this binder");
		if (!pb.getPClassMapping().equals(pcmbinder))
			throw new PExceptionNaming("This pbinding is not valid in this binder");
		if (pn.isNull())
			throw new PExceptionNaming("This PName represent the null value");
		byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
			PBinding.ACTION_BIND);
		// Performs the transition
        if (bindingcache != null) {
            synchronized (bindingcache) {
                try {
                    bindingcache.fix(bindingcache.bind(pn, pb));
                } catch (CacheException e) {
                    throw new PException(e, "[" + pcmbinder.getClassName() + "]: problem with cache management");
                }
            }
        }
		pb.setPName(pn);
		pb.setStatus(nextstate);
	}

	public PClassMapping getBinderClassMapping() {
		return this.pcmbinder;
	}

	public void unbind(PBindingCtrl pb) throws PException {
		if (pb == null)
			throw new PExceptionNaming("This pbinding is null");
		byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
			PBinding.ACTION_UNBIND);
		// If the state does not change, do nothing
		if (nextstate == pb.getStatus())
			return;
		// Performs the transition
		PName pn = pb.getPName();
		if (pn == null)
			throw new PExceptionNaming("This PBinding is not bounded with a pname");
		if (!pb.getPClassMapping().equals(pcmbinder))
			throw new PExceptionNaming("This pbinding is not valid in this binder");
        if (bindingcache != null) {
            synchronized (bindingcache) {
                try {
                    if (((FixableCacheEntry) bindingcache).getCeFixCount() == 0) {
                        // the cache replacement has decided to remove this entry
                        pb.setPName(null);
                        pb.setStatus(nextstate);
                    } else {
                        // this entry becomes eligeable for replacement
                        bindingcache.unfix(bindingcache.lookup(pn));
                    }
                } catch (CacheException e) {
                    throw new PException(e, "[" + pcmbinder.getClassName() + "]: problem with cache management");
                }
            }
        } else {
            pb.setPName(null);
            pb.setStatus(nextstate);
        }
	}

	// IMPLEMENTATION OF METHODS FROM THE PNameManager INTERFACE

	public boolean codingSupported(int codingtype) {
		switch (codingtype) {
		case PNamingContext.CTCOMPOSITE:
		case PNamingContext.CTSTRING:
			return true;
		}
		return false;
	}

	public PName decodeAbstract(Object en, Object ctx) throws PExceptionNaming, UnsupportedOperationException {
        if (en instanceof PolymorphIdPNG) {
            // there is a specific PNamegetter
            try {
            	PolymorphIdPNG png = (PolymorphIdPNG) en;
            	if ((png.pnGetClassId(ctx) == jormclassId) && (png.pnGetObjectId(ctx) == jormobjectId)) {
            		return getNull();
            	} else {
					return new PolymorphIdPName(((PolymorphIdPNG)en).pnGetClassId(ctx),((PolymorphIdPNG)en).pnGetObjectId(ctx), this);
                }
            } catch (PException e) {
                throw new PExceptionNaming(e, "Wrong name structure!!");
            }
        } else if(en instanceof PNameGetter){
            // we use the generic PNameGetter
            try {
            	PNameGetter png = (PNameGetter) en;
            	if ((png.pngetLongField("classId", ctx) == jormclassId) && (png.pngetLongField("objectId", ctx) == jormobjectId)) {
            		return getNull();
            	} else {
                	return new PolymorphIdPName(((PNameGetter)en).pngetLongField("classId", ctx),((PNameGetter)en).pngetLongField("objectId", ctx), this);
                }
            } catch (PException e) {
                throw new PExceptionNaming(e, "Wrong name structure!!");
            }
        } else {
            throw new PExceptionNaming("A PNamegetter is required to decode this object :" + en);
        }
	}

	public Object encodeAbstract(PName pn) throws PExceptionNaming, UnsupportedOperationException {
        if (!(pn instanceof PolymorphIdPName))
            throw new PExceptionNaming("This binder decode only PolymorphIdPName");
        if (!pn.getPNameManager().equals(this))
            throw new PExceptionNaming("This pname is not valid in this pnc");
        return pn;
	}

	public boolean supportDynamicComposite() {
		return true;
	}

	public boolean supportCompositeField(String fn, PType ft) {
    // TO DO : I can not generate a dynamic type checking
    // if (ClassId.equals(fn)) && (ft.isa(org.objectweb.jorm.type.api.PType@da88c2))
    // if (ObjectId.equals(fn)) && (ft.isa(org.objectweb.jorm.type.api.PType@da88c2))
        return true;
	}

	public boolean supportStaticComposite() {
		return true;
	}

    public PName export(Object conn, Object en) throws PException {
		if (en instanceof PolymorphIdPName) {
		    if (((PName) en).getPNameManager()==this) {
			    return (PName) en;
			} else {
			    return new PolymorphIdPName(this, (PName) en);
			}
	    }
        throw new PExceptionNaming("Not supported. You must the export method with a ctx parameter which is the accessor");
    }

    public PName export(Object conn, Object en, Object ctx)
		throws PException {
        PName pn = null;
        if (en == null) {
			throw new PExceptionNaming("Cannot export null!");
		}
		if (en instanceof PolymorphIdPName) {
		    if (((PName) en).getPNameManager()==this) {
			    return (PName) en;
			} else {
			    return new PolymorphIdPName(this, (PName) en);
			}
	    }
        if (!(en instanceof PBindingCtrl))
            throw new PExceptionNaming("The item cannot be exported by this binder: " + en);
		PBindingCtrl pb = (PBindingCtrl) en;
		byte nextstate = PStateGraph.nextStatePBinding(pb.getStatus(),
			PBinding.ACTION_EXPORT);
		if (nextstate == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		// Performs the transition
        if (ctx instanceof PName) {
            pn = new PolymorphIdPName(this, (PName) ctx);
        } else if ((en instanceof PolymorphIdPNG) && (ctx instanceof PAccessor)) {
            pn = new PolymorphIdPName(((PolymorphIdPNG)en).pnGetClassId(ctx),((PolymorphIdPNG)en).pnGetObjectId(ctx), this);
        } else if (ctx instanceof PolymorphIdPNG) {
            // there is a specific PNamegetter
            pn = new PolymorphIdPName(((PolymorphIdPNG)ctx).pnGetClassId(null),((PolymorphIdPNG)ctx).pnGetObjectId(null), this);
        } else if (ctx instanceof PNameGetter) {
            // we use the generic PNameGetter
            pn = new PolymorphIdPName(((PNameGetter)ctx).pngetLongField("classId", null),((PNameGetter)ctx).pngetLongField("objectId", null), this);
        } else {
            throw new PExceptionNaming("This item cannot be exported by this binder");
        }
        if (bindingcache != null) {
            synchronized (bindingcache) {
                if (bindingcache.lookup(pn) != null) {
                    throw new PExceptionExistingName(
                            "[" + pcmbinder.getClassName() + "]: an object has been already export with the same identifier");
                }
                try {
                    bindingcache.fix(bindingcache.bind(pn, en));
                } catch (CacheException e) {
                    throw new PException(e, "[" + pcmbinder.getClassName() + "]: problem with cache management");
                }
            }
        }
		pb.setPName(pn);
		pb.setStatus(nextstate);
        return pn;
	}

    public PName getNull() {
        if (nullPName == null)
            nullPName = PolymorphIdPName.nullPName(this);
        return nullPName;
    }

	/**
	 * It retrieves the information item associated to the given PName (if any
	 * exists). The pname must be a StrName and must support the hashcode method.
	 * @param conn	The connection that can be used to access the data store.
	 * @param pn	The PName to be looked up.
	 * @return	The PBinding associated with the given PName within
	 * 		this binder
	 * @exception PExceptionNaming	Raised if the Pname is not registered into this binder.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */

	public PName resolve(Object conn, PName pn) throws PException {
        if (pn == null)
            throw new PExceptionNaming("This pname is null");
        if (!pn.getPNameManager().equals(this))
            throw new PExceptionNaming("This pname is not valid in this binder");
        return pn;
	}

    /**
	 * It removes the PName and all the associated information within a naming
	 * context (i.e., the association between the PName and its information
	 * item). in the case of the StrBinder, the unexport operation do nothing more
	 * than the unbind operation.
	 * @param conn	The connection that can be used to access the data store.
	 * @param pn	A PName valid in this naming context.
	 * @exception PExceptionNaming	Raised in case of problem within the
	 * 								naming context.
	 * @see org.objectweb.jorm.naming.api.PNamingContext
	 */
    public void unexport(Object conn, PName pn) throws PException {
        if (pn.isNull()) {
            throw new PExceptionProtocol("[" + pcmbinder.getClassName() + "]: cannot unexport with a null pname.");
        }
        if (bindingcache != null) {
            CacheEntry ce = bindingcache.lookup(pn);
            if (ce == null) {
                throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
            }
            PBindingCtrl pb = (PBindingCtrl) ce.getCeObject();
            byte nextstate = PStateGraph.nextStatePBinding(
                pb.getStatus(), PBinding.ACTION_UNEXPORT);
            if (nextstate == PBinding.LIFECYCLE_ERROR) {
                throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
            }
            pb.setStatus(nextstate);
        } else {
            throw new PExceptionProtocol("Unauthorized operation: No Pbinding can be found without a CacheManager");
        }
    }

    public void unexport(Object conn, PName pn, Object hints) throws PException {
        if (pn.isNull()) {
            throw new PExceptionProtocol("[" + pcmbinder.getClassName() + "]: cannot unexport with a null pname.");
        }
        PBindingCtrl pb = null;
        if (hints instanceof PBindingCtrl) {
            pb = (PBindingCtrl) hints;
        } else if (bindingcache != null) {
            CacheEntry ce = bindingcache.lookup(pn);
            if (ce != null) {
                pb = (PBindingCtrl) ce.getCeObject();
            }
        }
        if (pb == null) {
            throw new PExceptionProtocol("Unauthorized operation: No Pbinding found");
        }
        byte nextstate = PStateGraph.nextStatePBinding(
            pb.getStatus(), PBinding.ACTION_UNEXPORT);
        if (nextstate == PBinding.LIFECYCLE_ERROR) {
            throw new PExceptionProtocol("Unauthorized operation: status =" + pb.getStatus());
        }
        pb.setStatus(nextstate);
    }

    /**
     * Returns the cache manager associated with this binder.
     * @return  The cache manager.
     */
    public CacheManager getCacheManager() {
        return bindingcache;
    }

    /**
     * Assigns the cache manager associated with this binder.
     * @param cm    The cache manager.
     * @throws PException
     */
    public void setCacheManager(CacheManager cm) throws PException {
        bindingcache = cm;
    }

	public String encodeString(PName pn) throws PExceptionNaming {
        if (!(pn instanceof PolymorphIdPName)) {
            throw new PExceptionNaming("This binder decode only PolymorphIdPName");
		}
        if (!pn.getPNameManager().equals(this)) {
            throw new PExceptionNaming("This pname is not valid in this pnc");
		}
		PolymorphIdPName rpn = (PolymorphIdPName) pn;
		PNCStringCoder cnsc = new BasicStringCoder();
		cnsc.putLong(rpn.classId);
		cnsc.putLong(rpn.objectId);
		return cnsc.getStringCode();
	}

	public PName decodeString(String en) throws PExceptionNaming {
		try {
			PNCStringCoder cnsc = new BasicStringCoder(en);
			PolymorphIdPName pn = new PolymorphIdPName(this);
			pn.classId = cnsc.getLong();
			pn.objectId = cnsc.getLong();
			return pn;
		} catch (PException e) {
			throw new PExceptionNaming(e, "Wrong name structure!!");
		}
	}
}
