/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.naming.lib.BasicPName;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;

public class PolymorphIdPName extends BasicPName implements PName, PNameGetter, PolymorphIdPNG {
	public static long NULLVALUE = -1;
	public static char NULL_CHAR_VALUE = '0';
	public static Object NULLOVALUE = null;

    protected long classId;
    protected long objectId;

    public PolymorphIdPName(PNameManager pnc) {
		this.pnc = pnc;
    }


    public PolymorphIdPName(long classId,long objectId, PNameManager pnc) {
        this.classId = classId;
        this.objectId = objectId;
        this.pnc = pnc;
    }

    public PolymorphIdPName(PNameManager pnc, PName pn)
        throws PException {
        if (pn instanceof PolymorphIdPName) {
            classId = ((PolymorphIdPName) pn).classId;
            objectId = ((PolymorphIdPName) pn).objectId;
        } else {
            PNameGetter png = (PNameGetter) pn.encodeAbstract();
            if (png instanceof PolymorphIdPNG) {
                classId = ((PolymorphIdPNG) png).pnGetClassId(null);
                objectId = ((PolymorphIdPNG) png).pnGetObjectId(null);
            } else {
                throw new UnsupportedOperationException("");
            }
        }
        this.pnc = pnc;
    }

    public void init(long classId,long objectId, PNameManager pnc) {
        this.classId = classId;
        this.objectId = objectId;
        this.pnc = pnc;
    }

    public static PName nullPName(PNameManager _pnm) {
        return new PolymorphIdPName((long)NULLVALUE, (long)NULLVALUE, _pnm);
    }

	public String toString() {
		return super.toString()
			+ " / classId=" + classId
			+ " / objectId=" + objectId
			;
	}


    // IMPLEMENTATION OF THE PName INTERFACE //
    //---------------------------------------//

    public boolean isNull() {
        return this == pnc.getNull();
    }

    // METHODS USED BY THE CACHE //
    //---------------------------//

    public int hashCode() {
        return 0 + ((int) classId) + ((int) objectId);
    }

    public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof PolymorphIdPName))
			return false;
		PolymorphIdPName pn = (PolymorphIdPName) obj;
		if (pn.pnc != this.pnc)
			return false;
		// check the other composite name fields and the pnc
		return (classId == pn.classId) && (objectId == pn.objectId);
    }


    // IMPLEMENTATION OF THE GENERIC PNameGetter INTERFACE //
    //-----------------------------------------------------//

    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=byte, fn=" + fn);
    }

    public Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.Byte, fn=" + fn);
    }

    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=char, fn=" + fn);
    }

    public Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.Character, fn=" + fn);
    }

    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=short, fn=" + fn);
    }

    public Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.Short, fn=" + fn);
    }

    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=int, fn=" + fn);
    }

    public Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.Integer, fn=" + fn);
    }

    public long pngetLongField(String fn, Object ctx) throws PException {
        if (fn.equals("classId")) return classId;
        if (fn.equals("objectId")) return objectId;
        throw new PExceptionTyping("Unsupported field type: type=long, fn=" + fn);
    }

    public Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.Long, fn=" + fn);
    }

    public String pngetStringField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.lang.String, fn=" + fn);
    }

    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=byte[], fn=" + fn);
    }

    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=char[], fn=" + fn);
    }

    public Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.util.Date, fn=" + fn);
    }

    public BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.math.BigInteger, fn=" + fn);
    }

    public BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("Unsupported field type: type=java.math.BigDecimal, fn=" + fn);
    }

    // IMPLEMENTATION OF THE SPECIFIC PNameGetter INTERFACE //
    //------------------------------------------------------//

    public long pnGetClassId(Object ctx) throws PException {
        return classId;
    }
    public long pnGetObjectId(Object ctx) throws PException {
        return objectId;
    }
}

