/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid.fos;

// Import clauses required by jorm
import org.objectweb.jorm.api.ConnectionHolder;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.tuple.api.Tuple;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
// Import clauses required by the object ClassId
import org.objectweb.jorm.facility.naming.polymorphid.ClassIdAccessor;
// Imports needed by the fos mapper
import org.objectweb.perseus.connector.ra.fos.FosConnection;
import org.objectweb.perseus.fos.api.FosAccess;
import org.objectweb.perseus.fos.api.FosException;
import org.objectweb.perseus.fos.api.FosStructure;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;

public class ClassIdBinding
	implements PBindingCtrl
 , FosStructure
    {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status;
	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	public ClassIdMapping pClassMapping;
	/**
	 * The PName of this PBinding object.
	 */
	protected PName pname;
// none

	public ClassIdBinding() {
// nothing
	}

	public ClassIdBinding(PClassMapping pcm) {
		pClassMapping = (ClassIdMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (ClassIdMapping) pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	    pname = null;
	}

	public void write(Object conn, PAccessor pa)
		throws PException {
		ClassIdAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof ClassIdAccessor)
			paccessor = (ClassIdAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		FosConnection connection;
		if (conn == null)
			throw new PExceptionProtocol("Cannot write without a connection");
		else if (conn instanceof FosConnection)
			connection = (FosConnection) conn;
		else
			throw new PExceptionProtocol("Cannot write with this kind of connection");
		try {
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
				connection.delete(ClassIdMapping.FOS_DIRNAME, pname.encodeString());
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
				connection.write(ClassIdMapping.FOS_DIRNAME, pname.encodeString(), this, pa);
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
				connection.write(ClassIdMapping.FOS_DIRNAME, pname.encodeString(), this, pa);
		    }
		} catch(FosException fe) {
			throw new PExceptionIO(fe);
		}
		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Read");
		ClassIdAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof ClassIdAccessor)
			paccessor = (ClassIdAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		FosConnection connection;
		if (conn == null)
			throw new PExceptionProtocol("Cannot read without a connection");
		else if (conn instanceof FosConnection)
			connection = (FosConnection) conn;
		else
			throw new PExceptionProtocol("Cannot read with this kind of connection");
		try {
            String enc_pname = pname.encodeString();
            if (!connection.exist(ClassIdMapping.FOS_DIRNAME, enc_pname)) {
                throw new PExceptionNoDSI(
                    "No object matching with the following identifier: " + enc_pname);
            }
			connection.read(ClassIdMapping.FOS_DIRNAME, enc_pname, this, pa);
		} catch(FosException fe) {
			throw new PExceptionIO(fe);
		}
		status = nextState;
	}

	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = pClassMapping.getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(conn, pa);
	        return;
	    }
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, pClassMapping, txctx);
	        if (tuple == null) {
	            read(conn, pa);
	            return;
	        }
		    ClassIdAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof ClassIdAccessor)
			    paccessor = (ClassIdAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            if (paccessor != null) {
                paccessor.paSetClassId(tuple.getLong(1));
            } else if (paccessorGen != null) {
                paccessorGen.paSetLongField(pClassMapping.classId_NAME, tuple.getLong(1));
            }
		    status = nextState;
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Exist");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		status = nextState;
		FosConnection connection;
		if (conn == null)
			throw new PExceptionProtocol("Cannot check the existence without a connection");
		else if (conn instanceof FosConnection)
			connection = (FosConnection) conn;
		else
			throw new PExceptionProtocol("Cannot check the existence with this kind of connection");
		try {
			return connection.exist(ClassIdMapping.FOS_DIRNAME, pname.encodeString());
		} catch(FosException fe) {
			throw new PExceptionIO(fe);
		}
	}

	public PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export with hints");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unexport");
        pClassMapping.getPBinder().unexport(conn, this.pname, this);
	}

	public void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Bind");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unbind");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}

	public void readFile(ObjectInputStream ois, FosAccess conn, Object ctxt) throws Exception {
		if (ctxt instanceof ClassIdAccessor) {
			ClassIdAccessor paccessor = (ClassIdAccessor) ctxt;
			paccessor.paSetClassId(ois.readLong());
		} else {
			PAccessorGen paccessorGen = (PAccessorGen) ctxt;
			paccessorGen.paSetLongField(pClassMapping.classId_NAME, ois.readLong());
 		}
	}

	public void writeFile(ObjectOutputStream oos, FosAccess conn, Object ctxt) throws Exception {
		if (ctxt instanceof ClassIdAccessor) {
			ClassIdAccessor paccessor = (ClassIdAccessor) ctxt;
			oos.writeLong(paccessor.paGetClassId());
		} else {
			PAccessorGen paccessorGen = (PAccessorGen) ctxt;
			oos.writeLong(paccessorGen.paGetLongField(pClassMapping.classId_NAME));
 		}
	}
}
