/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.facility.naming.polymorphid.rdb.postgres;

// Import clauses required by jorm
import org.objectweb.jorm.api.ConnectionHolder;
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.tuple.api.Tuple;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
// Import clauses required by the object IdClass
import org.objectweb.jorm.facility.naming.polymorphid.IdClassAccessor;
// Imports needed by the rdb mapper
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.io.IOException;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.mapper.rdb.adapter.BasicRdbAdapter;

public class IdClassBinding
	implements PBindingCtrl
      {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status;
	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	public IdClassMapping pClassMapping;
	/**
	 * The PName of this PBinding object.
	 */
	protected PName pname;

	public IdClassBinding() {
// nothing
	}

	public IdClassBinding(PClassMapping pcm) {
		pClassMapping = (IdClassMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (IdClassMapping) pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	    pname = null;
	}

	public void write(Object conn, PAccessor pa)
		throws PException {
		IdClassAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof IdClassAccessor)
			paccessor = (IdClassAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection;
		if (conn==null) {
			throw new PExceptionProtocol("Impossible to write without connection");
		} else if (conn instanceof Connection) {
			connection = (Connection) conn;
		} else if (conn instanceof ConnectionHolder) {
			connection = (Connection) ((ConnectionHolder) conn).getCHConnection();
		} else if (conn instanceof org.objectweb.perseus.persistence.api.ConnectionHolder) {
			try {
				connection = (Connection)
					((org.objectweb.perseus.persistence.api.ConnectionHolder) conn)
					.getCHConnectionForWrite();
			} catch(org.objectweb.perseus.persistence.api.PersistenceException e) {
				throw new PExceptionProtocol(e, "Impossible to write with this connection: ");
			}
		} else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else {
			throw new PExceptionProtocol("Impossible to write with this connection type.");
		}
		try {
			String sqlquery;
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
//Table jf_ClassId : readOnly=false generateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deletejf_ClassIdTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [IdClass]: problematic SQL query -> {" + sqlquery + "}");
                }
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertjf_ClassIdTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [IdClass]: problematic SQL query -> {" + sqlquery + "}");
                }
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updatejf_ClassIdTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [IdClass]: problematic SQL query -> {" + sqlquery + "}");
                }
		    }
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe);
		}
		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Read");
		IdClassAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof IdClassAccessor)
			paccessor = (IdClassAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		Connection connection;
		if (conn==null) {
			throw new PExceptionProtocol("Impossible to read without connection");
		} else if (conn instanceof Connection) {
			connection = (Connection) conn;
		} else if (conn instanceof ConnectionHolder) {
			connection = (Connection) ((ConnectionHolder) conn).getCHConnection();
		} else if (conn instanceof org.objectweb.perseus.persistence.api.ConnectionHolder) {
			try {
				connection = (Connection)
					((org.objectweb.perseus.persistence.api.ConnectionHolder) conn)
					.getCHConnectionForWrite();
			} catch(org.objectweb.perseus.persistence.api.PersistenceException e) {
				throw new PExceptionProtocol(e, "Impossible to read with this connection: ");
			}
		} else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else {
			throw new PExceptionProtocol("Impossible to read with this connection type.");
		}
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
		try {
            pstmt = connection.prepareStatement("SELECT jf_ClassId.CID, jf_ClassId.CN FROM jf_ClassId WHERE jf_ClassId.CID = ?");
            pstmt.setLong(1, pname.encodeLong());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
                fieldval = resultSet.getString(2);
                if (resultSet.wasNull()) {
                    fieldval = null;
                } else {
                }
                paccessor.paSetClassName((java.lang.String) fieldval);
			} else if (paccessorGen != null) {
                fieldval = resultSet.getString(2);
                if (resultSet.wasNull()) {
                    fieldval = null;
                } else {
                }
                paccessorGen.paSetStringField(pClassMapping.className_NAME, (java.lang.String) fieldval);
			}
		} catch (PException pe) {
			throw pe;
		} catch (Exception pe) {
			throw new PExceptionIO(pe);
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (java.sql.SQLException se) {
			    throw new PExceptionIO(se);
			}
		}
		status = nextState;
	}

	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = pClassMapping.getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(conn, pa);
	        return;
	    }
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, pClassMapping, txctx);
	        if (tuple == null) {
	            read(conn, pa);
	            return;
	        }
		    IdClassAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof IdClassAccessor)
			    paccessor = (IdClassAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            if (paccessor != null) {
                paccessor.paSetClassName(tuple.getString(2));
            } else if (paccessorGen != null) {
                paccessorGen.paSetStringField(pClassMapping.className_NAME, tuple.getString(2));
            }
		    status = nextState;
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Exist");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		status = nextState;
		Connection connection;
		if (conn==null) {
			throw new PExceptionProtocol("Impossible to check the existence without connection");
		} else if (conn instanceof Connection) {
			connection = (Connection) conn;
		} else if (conn instanceof ConnectionHolder) {
			connection = (Connection) ((ConnectionHolder) conn).getCHConnection();
		} else if (conn instanceof org.objectweb.perseus.persistence.api.ConnectionHolder) {
			try {
				connection = (Connection)
					((org.objectweb.perseus.persistence.api.ConnectionHolder) conn)
					.getCHConnectionForWrite();
			} catch(org.objectweb.perseus.persistence.api.PersistenceException e) {
				throw new PExceptionProtocol(e, "Impossible to check the existence with this connection: ");
			}
		} else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else {
			throw new PExceptionProtocol("Impossible to check the existence with this connection type.");
		}
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT jf_ClassId.CID FROM jf_ClassId WHERE jf_ClassId.CID = ?");
            pstmt.setLong(1, pname.encodeLong());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} catch (PException e) {
			throw new PExceptionIO(e);
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (java.sql.SQLException se) {
			    throw new PExceptionIO(se);
			}
		}
		return exist;
	}

	public PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export with hints");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unexport");
        pClassMapping.getPBinder().unexport(conn, this.pname, this);
	}

	public void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Bind");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unbind");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}
    private String updatejf_ClassIdTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        IdClassAccessor paccessor)
        throws PException, SQLException, IOException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE jf_ClassId SET CN = ? WHERE CID = ?");
                nullField = paccessor.paGetClassName();
                if (nullField == null) {
                    pstmt.setNull(1, 12);
                } else {
                    pstmt.setString(1, ((java.lang.String) nullField));
                }
                pstmt.setLong(2, pname.encodeLong());
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE jf_ClassId SET CN = ? WHERE CID = ?");
                nullField = paccessorGen.paGetStringField(pClassMapping.className_NAME);
                if (nullField == null) {
                    pstmt.setNull(1, 12);
                } else {
                    pstmt.setString(1, ((java.lang.String) nullField));
                }
                pstmt.setLong(2, pname.encodeLong());
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertjf_ClassIdTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                IdClassAccessor paccessor)
        throws PException, SQLException, IOException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO jf_ClassId (CID, CN) VALUES (?, ?)");
                pstmt.setLong(1, pname.encodeLong());
                nullField = paccessor.paGetClassName();
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO jf_ClassId (CID, CN) VALUES (?, ?)");
                pstmt.setLong(1, pname.encodeLong());
                nullField = paccessorGen.paGetStringField(pClassMapping.className_NAME);
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deletejf_ClassIdTable(Connection connection, Object conn, PAccessorGen paccessorGen, IdClassAccessor paccessor) throws PException, SQLException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM jf_ClassId WHERE CID = ?");
                pstmt.setLong(1, pname.encodeLong());
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
}
