/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import junit.framework.TestCase;
import org.objectweb.jorm.api.PException;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;


/**
 * Code for testing the RDB mapping structures implementation. It only tests
 * the definition of mapping structures and does not test operations that
 * modify the database schema.
 * @author P. Dechamboux
 */
public class TestRdbPMappingStructuresManager extends TestCase {
	private final static String JCN1 = "org.objectweb.test.C1";
	private final static String JCN2 = "org.objectweb.test.C2";

	private final static String TN1 = "T1";
	private final static String T1CN1 = "C11";
	private final static String T1CT1 = "INTEGER";
	private final static boolean T1CNN1 = true;
	private final static String T1CN2 = "C12";
	private final static String T1CT2 = "CHAR(20)";
	private final static boolean T1CNN2 = false;
	private final static String TN2 = "T2";
	private final static String T2CN1 = "C21";
	private final static String T2CT1 = "INTEGER";
	private final static boolean T2CNN1 = true;
	private final static String T2CN2 = "C22";
	private final static String T2CT2 = "CHAR(20)";
	private final static boolean T2CNN2 = false;
    private Logger logger = new LoggerImpl();

	public TestRdbPMappingStructuresManager(String testname) {
		super(testname);
	}

	protected void setUp() {

	}

	protected void tearDown() {

	}

	public void testClusterWith1Class1Table1Column() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Column " + T1CN1 + " into table " + JCN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class1Table1ColumnDupOK() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, false);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Column " + T1CN1 + " into table " + JCN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class1Table1ColumnDupKOnotnull() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, ! T1CNN1, false, true);
			fail("Duplicate column definition with incompatible notnull");
		} catch (PException e) {
			assertTrue(e.getMessage(), true);
		}
	}

	public void testClusterWith1Class1Table1ColumnDupKOtype() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT2, T1CNN1, false, true);
			fail("Duplicate column definition with incompatible type");
		} catch (PException e) {
			assertTrue(e.getMessage(), true);
		}
	}

	public void testClusterWith1Class1Table2Column() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN2, T1CT2, T1CNN2, false, false);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Column " + T1CN1 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
			assertTrue("Column " + T1CN2 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN2, T1CT2, T1CNN2));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class2Table2Column() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN2, T1CT2, T1CNN2, false, true);
			man.addTableColumn(JCN1, TN2, T2CN1, T2CT1, T2CNN1, false, true);
			man.addTableColumn(JCN1, TN2, T2CN2, T2CT2, T2CNN2, false, true);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Table " + TN2 + " does not exist in cluster", cl.containTable(TN2));
			assertTrue("Column " + T1CN1 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
			assertTrue("Column " + T1CN2 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN2, T1CT2, T1CNN2));
			assertTrue("Column " + T2CN1 + " into table " + TN2 + " does not exist in cluster",
					   cl.containColumn(TN2, T2CN1, T2CT1, T2CNN1));
			assertTrue("Column " + T2CN2 + " into table " + TN2 + " does not exist in cluster",
					   cl.containColumn(TN2, T2CN2, T2CT2, T2CNN2));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith2Class2Table2ColumnNoMerge() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN2, T1CT2, T1CNN2, false, true);
			man.addTableColumn(JCN2, TN2, T2CN1, T2CT1, T2CNN1, false, true);
			man.addTableColumn(JCN2, TN2, T2CN2, T2CT2, T2CNN2, false, true);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			RdbPMapCluster cl2 = (RdbPMapCluster) man.getPMapCluster(JCN2);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Cluster has not been created for " + JCN2, cl2 != null);
			assertTrue("The two classes are into the same cluster", cl != cl2);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Table " + TN2 + " does not exist in cluster", cl2.containTable(TN2));
			assertTrue("Column " + T1CN1 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
			assertTrue("Column " + T1CN2 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN2, T1CT2, T1CNN2));
			assertTrue("Column " + T2CN1 + " into table " + TN2 + " does not exist in cluster",
					   cl2.containColumn(TN2, T2CN1, T2CT1, T2CNN1));
			assertTrue("Column " + T2CN2 + " into table " + TN2 + " does not exist in cluster",
					   cl2.containColumn(TN2, T2CN2, T2CT2, T2CNN2));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith2Class2Table2ColumnMerge() {
		try {
			RdbPMappingStructuresManager man = new RdbPMappingStructuresManager();
			man.setLogger(logger);
			man.addTableColumn(JCN1, TN1, T1CN1, T1CT1, T1CNN1, false, true);
			man.addTableColumn(JCN1, TN1, T1CN2, T1CT2, T1CNN2, false, true);
			man.addTableColumn(JCN1, TN2, T2CN2, T2CT2, T2CNN2, false, true);
			man.addTableColumn(JCN2, TN2, T2CN1, T2CT1, T2CNN1, false, true);
			man.addTableColumn(JCN2, TN2, T2CN2, T2CT2, T2CNN2, false, false);
			RdbPMapCluster cl = (RdbPMapCluster) man.getPMapCluster(JCN1);
			RdbPMapCluster cl2 = (RdbPMapCluster) man.getPMapCluster(JCN2);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Cluster has not been created for " + JCN2, cl2 != null);
			assertTrue("The two classes are not into the same cluster", cl == cl2);
			assertTrue("Table " + TN1 + " does not exist in cluster", cl.containTable(TN1));
			assertTrue("Table " + TN2 + " does not exist in cluster", cl.containTable(TN2));
			assertTrue("Column " + T1CN1 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN1, T1CT1, T1CNN1));
			assertTrue("Column " + T1CN2 + " into table " + TN1 + " does not exist in cluster",
					   cl.containColumn(TN1, T1CN2, T1CT2, T1CNN2));
			assertTrue("Column " + T2CN1 + " into table " + TN2 + " does not exist in cluster",
					   cl.containColumn(TN2, T2CN1, T2CT1, T2CNN1));
			assertTrue("Column " + T2CN2 + " into table " + TN2 + " does not exist in cluster",
					   cl.containColumn(TN2, T2CN2, T2CT2, T2CNN2));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}
}
