/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.basictype;

import junit.framework.Assert;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.naming.api.PName;

import java.util.Date;
import java.util.Map;
import java.util.HashMap;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * This class is an implementation of the PAccessorGen interface for a
 * persistent object which have only one field. Only the primitive type (no
 * reference) are supported.
 *
 * @author S.Chassande-Barrioz
 */
public class GenericAccessor extends Assert implements PAccessorGen {

    private Map fn2tnv;

    /**
     * Builds a generic accessor which contains a value. Then it is usefull
     * during a write operation.
     * @param type is the PType of the field (not null)
     * @param fieldName is the name of the field (not null)
     * @param value is the value of the field (can be null). If the type is
     * a java primitive type (boolean, byte, short, int, long, float, double)
     * then this param must be an instance of the corresponding java.lang.XXX
     * class.
     */
    public GenericAccessor(PType type, String fieldName, Object value) {
        fn2tnv = new HashMap();
        fn2tnv.put(fieldName, new Object[]{type, value});
    }

    /**
     * Builds a generic accessor without value. Then it is usefull during a
     * read operation.
     * @param type is the PType of the field (not null)
     * @param fieldName is the name of the field (not null)
     */
    public GenericAccessor(PType type, String fieldName) {
        fn2tnv = new HashMap();
        fn2tnv.put(fieldName, new Object[]{type, null});
    }

    // IMPLEMENTATION OF THE PAccessorGen INTERFACE //
    //----------------------------------------------//

    public boolean paGetBooleanField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.BOOLEAN.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: boolean");
        }
        return ((Boolean) tnv[1]).booleanValue();
    }

    public Boolean paGetObooleanField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJBOOLEAN.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Boolean");
        }
        return (Boolean) tnv[1];
    }

    public byte paGetByteField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.BYTE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: byte");
        }
        return ((Byte) tnv[1]).byteValue();
    }

    public Byte paGetObyteField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJBYTE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Byte");
        }
        return (Byte) tnv[1];
    }

    public char paGetCharField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.CHAR.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: char");
        }
        return ((Character) tnv[1]).charValue();
    }

    public Character paGetOcharField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJCHAR.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Char");
        }
        return (Character) tnv[1];
    }

    public short paGetShortField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.SHORT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: short");
        }
        return ((Short) tnv[1]).shortValue();
    }

    public Short paGetOshortField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJSHORT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Short");
        }
        return (Short) tnv[1];
    }

    public int paGetIntField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.INT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " found: int");
        }
        return ((Integer) tnv[1]).intValue();
    }

    public Integer paGetOintField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJINT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Int");
        }
        return (Integer) tnv[1];
    }

    public long paGetLongField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.LONG.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: long");
        }
        return ((Long) tnv[1]).longValue();
    }

    public Long paGetOlongField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJLONG.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Long");
        }
        return (Long) tnv[1];
    }

    public float paGetFloatField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.FLOAT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " found: float");
        }
        return ((Float) tnv[1]).floatValue();
    }

    public Float paGetOfloatField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJFLOAT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Float");
        }
        return (Float) tnv[1];
    }

    public double paGetDoubleField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.DOUBLE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: double");
        }
        return ((Double) tnv[1]).doubleValue();
    }

    public Double paGetOdoubleField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJDOUBLE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Double");
        }
        return (Double) tnv[1];
    }

    public String paGetStringField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.STRING.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: string");
        }
        return (String) tnv[1];
    }

    public byte[] paGetByteArrayField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.BYTEARRAY.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: byte[]");
        }
        return (byte[]) tnv[1];
    }

    public char[] paGetCharArrayField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.CHARARRAY.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: char[]");
        }
        return (char[]) tnv[1];
    }

    public Date paGetDateField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.DATE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: date");
        }
        return (Date) tnv[1];
    }

    public Serializable paGetSerializedField(String fn) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }

        PType type = (PType) tnv[0];
        if (PTypeSpace.SERIALIZED.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: serialized");
        }
        return (Serializable) tnv[1];
    }

	public BigInteger paGetBigIntegerField(String fn) throws PException {
		Object[] tnv = (Object[]) fn2tnv.get(fn);
		if (tnv==null) {
			fail("Unmanaged field" + fn);
		}

		PType type = (PType) tnv[0];
		if (PTypeSpace.BIGINTEGER.getTypeCode() != type.getTypeCode()) {
			fail("Bad type expected: "
					+ type.getJormName()
					+ " asked: BigInteger");
		}
		return (BigInteger) tnv[1];
	}

	public BigDecimal paGetBigDecimalField(String fn) throws PException {
		Object[] tnv = (Object[]) fn2tnv.get(fn);
		if (tnv==null) {
			fail("Unmanaged field" + fn);
		}

		PType type = (PType) tnv[0];
		if (PTypeSpace.BIGDECIMAL.getTypeCode() != type.getTypeCode()) {
			fail("Bad type expected: "
					+ type.getJormName()
					+ " asked: BigDecimal");
		}
		return (BigDecimal) tnv[1];
	}

    public PName paGetRefField(String fn, Object connection) throws PException {
        fail("No reference in this test: fn" + fn);
        return null;
    }

    public void paSetBooleanField(String fn, boolean value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.BOOLEAN.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: boolean");
        }
        tnv[1] = new Boolean(value);
    }

    public void paSetObooleanField(String fn, Boolean value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJBOOLEAN.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Boolean");
        }
        tnv[1] = value;
    }

    public void paSetByteField(String fn, byte value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.BYTE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: byte");
        }
        tnv[1] = new Byte(value);
    }

    public void paSetObyteField(String fn, Byte value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJBYTE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Byte");
        }
        tnv[1] = value;
    }

    public void paSetCharField(String fn, char value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.CHAR.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: char");
        }
        tnv[1] = new Character(value);
    }

    public void paSetOcharField(String fn, Character value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJCHAR.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Char");
        }
        tnv[1] = value;
    }

    public void paSetShortField(String fn, short value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.SHORT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: short");
        }
        tnv[1] = new Short(value);
    }

    public void paSetOshortField(String fn, Short value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJSHORT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Short");
        }
        tnv[1] = value;
    }

    public void paSetIntField(String fn, int value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.INT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: int");
        }
        tnv[1] = new Integer(value);
    }

    public void paSetOintField(String fn, Integer value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJINT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Int");
        }
        tnv[1] = value;
    }

    public void paSetLongField(String fn, long value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.LONG.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: long");
        }
        tnv[1] = new Long(value);
    }

    public void paSetOlongField(String fn, Long value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJLONG.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Long");
        }
        tnv[1] = value;
    }

    public void paSetFloatField(String fn, float value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.FLOAT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: float");
        }
        tnv[1] = new Float(value);
    }

    public void paSetOfloatField(String fn, Float value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJFLOAT.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Float");
        }
        tnv[1] = value;
    }

    public void paSetDoubleField(String fn, double value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.DOUBLE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: double");
        }
        tnv[1] = new Double(value);
    }

    public void paSetOdoubleField(String fn, Double value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.OBJDOUBLE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: Double");
        }
        tnv[1] = value;
    }

    public void paSetStringField(String fn, String value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.STRING.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: string");
        }
        tnv[1] = value;
    }

    public void paSetByteArrayField(String fn, byte[] value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.BYTEARRAY.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: byte[]");
        }
        tnv[1] = value;
    }

    public void paSetCharArrayField(String fn, char[] value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.CHARARRAY.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: char[]");
        }
        tnv[1] = value;
    }

    public void paSetDateField(String fn, Date value) throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.DATE.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: date");
        }
        tnv[1] = value;
    }

    public void paSetSerializedField(String fn, Serializable value)
            throws PException {
        Object[] tnv = (Object[]) fn2tnv.get(fn);
        if (tnv==null) {
            fail("Unmanaged field" + fn);
        }
        PType type = (PType) tnv[0];
        if (PTypeSpace.SERIALIZED.getTypeCode() != type.getTypeCode()) {
            fail("Bad type expected: "
                    + type.getJormName()
                    + " asked: serialized");
        }
        tnv[1] = value;
    }

	public void paSetBigIntegerField(String fn, BigInteger value)
			throws PException {
		Object[] tnv = (Object[]) fn2tnv.get(fn);
		if (tnv==null) {
			fail("Unmanaged field" + fn);
		}
		PType type = (PType) tnv[0];
		if (PTypeSpace.BIGINTEGER.getTypeCode() != type.getTypeCode()) {
			fail("Bad type expected: "
					+ type.getJormName()
					+ " asked: BigInteger");
		}
		tnv[1] = value;
	}

	public void paSetBigDecimalField(String fn, BigDecimal value)
			throws PException {
		Object[] tnv = (Object[]) fn2tnv.get(fn);
		if (tnv==null) {
			fail("Unmanaged field" + fn);
		}
		PType type = (PType) tnv[0];
		if (PTypeSpace.BIGDECIMAL.getTypeCode() != type.getTypeCode()) {
			fail("Bad type expected: "
					+ type.getJormName()
					+ " asked: BigDecimal");
		}
		tnv[1] = value;
	}

    public void paSetRefField(String fn, PName value, Object connection)
            throws PException {
        fail("No reference in this test: fn" + fn + " / pn=" + value);
    }


    // IMPLEMENTATION OF THE PAccessor INTERFACE //
    //-------------------------------------------//

    public Object getMemoryInstance() {
        return this;
    }
}


