/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.runtime.colocated;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PGenClassAccessor;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Date;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

import junit.framework.Assert;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class GenClass_C implements PGenClassAccessor {

    public List elements;
    public int expectedSize;

    public GenClass_C(List pnames) {
        elements = new ArrayList(pnames.size());
        for(int i=0; i<pnames.size(); i++) {
            elements.add(new RefElement((PName) pnames.get(i)));
        }
        expectedSize = elements.size();
    }

    public GenClass_C(int expectedsize) {
        expectedSize = expectedsize;
        elements = new ArrayList();
    }

    public boolean equals(Object obj) {
        Assert.assertNotNull("null object", obj);
        Assert.assertEquals("Bad class", obj.getClass(), getClass());
        GenClass_C o = (GenClass_C) obj;
        Assert.assertEquals("Bad size", elements.size(), o.elements.size());
        for(int i=0; i<elements.size(); i++) {
            Assert.assertTrue("Expected element <"
                + ((RefElement) elements.get(i)).element
                + "> is not present: in " + o.elements,
                o.elements.contains(elements.get(i)));
        }
        for(int i=0; i<o.elements.size(); i++) {
            Assert.assertTrue("Unexpected element <"
                + ((RefElement) o.elements.get(i)).element
                + "> is present",
                elements.contains(o.elements.get(i)));
        }
        return true;
    }

    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE //
    //---------------------------------------------------//

    public void paAdd(PIndexedElem elem, Object conn) throws PException {
        elements.add(elem);
    }

    public boolean paDeltaSupported() {
        return true;
    }

    public int paGetNbElem() {
        return elements.size();
    }

    public Iterator paIterator() {
        return elements.iterator();
    }

    public void paSetNbElem(int nbelem) {
        if (nbelem != -1) {
            Assert.assertEquals("Bad expected size", expectedSize, nbelem);
        }
        elements.clear();
    }

    public PIndexedElem createPIndexedElem() throws PException {
        return new RefElement();
    }


    // IMPLEMENTATION OF THE PGenClassAccessor INTERFACE //
    //---------------------------------------------------//

    public Object getMemoryInstance() {
        return this;
    }


    // INNER CLASS IMPLEMENTING THE PIndexedElem INTERFACE //
    //-----------------------------------------------------//

    public class RefElement implements PIndexedElem {
        public byte status;
        public PName element;

        public RefElement() {
            status = PIndexedElem.ELEM_UNMODIFIED;
        }

        public RefElement(PName element) {
            this.status = PIndexedElem.ELEM_CREATED;
            this.element = element;
        }

        public RefElement(byte status, PName element) {
            this.status = status;
            this.element = element;
        }

        public boolean equals(Object obj) {
            return element.equals(((RefElement) obj).element);
        }

        public String toString() {
            return "RefElemen(" + element.toString() + ")";
        }

        public byte getElemStatus() {
            return status;
        }

        public void pieSetRefElem(PName value) throws PException {
            element = value;
        }

        public PName pieGetRefElem() throws PException {
            return element;
        }

        public boolean pieGetBooleanElem() throws PException {
            Assert.fail("The element is a reference");
            return false;
        }

        public Boolean pieGetObooleanElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public byte pieGetByteElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Byte pieGetObyteElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public byte pieGetByteIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return 0;
        }

        public Byte pieGetObyteIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public char pieGetCharElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Character pieGetOcharElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public char pieGetCharIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return 0;
        }

        public Character pieGetOcharIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public short pieGetShortElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Short pieGetOshortElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public short pieGetShortIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return 0;
        }

        public Short pieGetOshortIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public int pieGetIntElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Integer pieGetOintElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public int pieGetIntIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return 0;
        }

        public Integer pieGetOintIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public long pieGetLongElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Long pieGetOlongElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public long pieGetLongIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return 0;
        }

        public Long pieGetOlongIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public float pieGetFloatElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Float pieGetOfloatElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public double pieGetDoubleElem() throws PException {
            Assert.fail("The element is a reference");
            return 0;
        }

        public Double pieGetOdoubleElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public String pieGetStringElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public String pieGetStringIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public Date pieGetDateElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

        public Date pieGetDateIndexField(String fn) throws PException {
            Assert.fail("There is no index");
            return null;
        }

        public char[] pieGetCharArrayElem() throws PException {
            Assert.fail("The element is a reference");
            return new char[0];
        }

        public byte[] pieGetByteArrayElem() throws PException {
            Assert.fail("The element is a reference");
            return new byte[0];
        }

        public Serializable pieGetSerializedElem() throws PException {
            Assert.fail("The element is a reference");
            return null;
        }

		public BigDecimal pieGetBigDecimalElem() throws PException {
			Assert.fail("The element is a reference");
			return null;
		}

		public BigInteger pieGetBigIntegerElem() throws PException {
			Assert.fail("The element is a reference");
			return null;
		}

        public void pieSetBooleanElem(boolean value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetObooleanElem(Boolean value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetByteElem(byte value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetObyteElem(Byte value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetByteIndexField(String fn, byte value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetObyteIndexField(String fn, Byte value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetCharElem(char value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOcharElem(Character value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetCharIndexField(String fn, char value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetOcharIndexField(String fn, Character value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetShortElem(short value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOshortElem(Short value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetShortIndexField(String fn, short value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetOshortIndexField(String fn, Short value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetIntElem(int value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOintElem(Integer value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetIntIndexField(String fn, int value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetOintIndexField(String fn, Integer value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetLongElem(long value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOlongElem(Long value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetLongIndexField(String fn, long value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetOlongIndexField(String fn, Long value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetFloatElem(float value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOfloatElem(Float value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetDoubleElem(double value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetOdoubleElem(Double value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetStringElem(String value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetStringIndexField(String fn, String value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetDateElem(Date value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetDateIndexField(String fn, Date value) throws PException {
            Assert.fail("There is no index");
        }

        public void pieSetCharArrayElem(char[] value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetByteArrayElem(byte[] value) throws PException {
            Assert.fail("The element is a reference");
        }

        public void pieSetSerializedElem(Serializable value) throws PException {
            Assert.fail("The element is a reference");
        }

		public void pieSetBigDecimalElem(BigDecimal value) throws PException {
			Assert.fail("The element is a reference");
		}

		public void pieSetBigIntegerElem(BigInteger value) throws PException {
			Assert.fail("The element is a reference");
		}
    }
}
