/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.gcbasictypeelem;

import org.objectweb.jorm.api.PGenClassAccessor;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.util.Date;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

import junit.framework.Assert;

/**
 * @author  P. Dechamboux
 */
public class GcBasicTypeAccessor implements PGenClassAccessor {
    private boolean[] boolean_value;
    private byte[] byte_value;
    private char[] char_value;
    private short[] short_value;
    private int[] int_value;
    private long[] long_value;
    private float[] float_value;
    private double[] double_value;
    private Object[] object_value;
    private int typeCode, nbElem, nextToAdd;

    public GcBasicTypeAccessor(int typecode, int nbelem) throws PException {
        typeCode = typecode;
        nbElem = nbelem;
        switch (typeCode) {
        case PType.TYPECODE_BOOLEAN:
            boolean_value = new boolean[nbelem];
            break;
        case PType.TYPECODE_BYTE:
            byte_value = new byte[nbelem];
            break;
        case PType.TYPECODE_CHAR:
            char_value = new char[nbelem];
            break;
        case PType.TYPECODE_SHORT:
            short_value = new short[nbelem];
            break;
        case PType.TYPECODE_INT:
            int_value = new int[nbelem];
            break;
        case PType.TYPECODE_LONG:
            long_value = new long[nbelem];
            break;
        case PType.TYPECODE_FLOAT:
            float_value = new float[nbelem];
            break;
        case PType.TYPECODE_DOUBLE:
            double_value = new double[nbelem];
            break;
        case PType.TYPECODE_OBJBOOLEAN:
        case PType.TYPECODE_OBJBYTE:
        case PType.TYPECODE_OBJCHAR:
        case PType.TYPECODE_OBJSHORT:
        case PType.TYPECODE_OBJINT:
        case PType.TYPECODE_OBJLONG:
        case PType.TYPECODE_OBJFLOAT:
        case PType.TYPECODE_OBJDOUBLE:
        case PType.TYPECODE_STRING:
        case PType.TYPECODE_DATE:
        case PType.TYPECODE_BYTEARRAY:
        case PType.TYPECODE_CHARARRAY:
        case PType.TYPECODE_BIGDECIMAL:
        case PType.TYPECODE_SERIALIZED:
            object_value = new Object[nbelem];
            break;
        default :
            throw new PExceptionTyping("Wrong field type.");
        }
    }

    public int getTypeCode() {
        return typeCode;
    }

    public void setElem(int pos, Object o) {
        object_value[pos] = o;
    }

    public void setElem(int pos, boolean o) {
        boolean_value[pos] = o;
    }

    public void setElem(int pos, byte o) {
        byte_value[pos] = o;
    }

    public void setElem(int pos, char o) {
        char_value[pos] = o;
    }

    public void setElem(int pos, short o) {
        short_value[pos] = o;
    }

    public void setElem(int pos, int o) {
        int_value[pos] = o;
    }

    public void setElem(int pos, long o) {
        long_value[pos] = o;
    }

    public void setElem(int pos, float o) {
        float_value[pos] = o;
    }

    public void setElem(int pos, double o) {
        double_value[pos] = o;
    }

    public boolean equals(Object other) {
        switch (typeCode) {
        case PType.TYPECODE_BOOLEAN:
            return assertEqualArray("boolean values error (1)", boolean_value
                    , ((GcBasicTypeAccessor) other).boolean_value)
                    && assertEqualArray("boolean values error (2)",
                            ((GcBasicTypeAccessor) other).boolean_value,
                            boolean_value);
        case PType.TYPECODE_BYTE:
            return assertEqualArray("byte values error (1)", byte_value
                    , ((GcBasicTypeAccessor) other).byte_value, true)
                    && assertEqualArray("byte values error (2)",
                            ((GcBasicTypeAccessor) other).byte_value,
                            byte_value, true);
        case PType.TYPECODE_CHAR:
            return assertEqualArray("char values error (1)", char_value
                    , ((GcBasicTypeAccessor) other).char_value)
                    && assertEqualArray("char values error (2)",
                            ((GcBasicTypeAccessor) other).char_value,
                            char_value);
        case PType.TYPECODE_SHORT:
            return assertEqualArray("short values error (1)", short_value
                    , ((GcBasicTypeAccessor) other).short_value)
                    && assertEqualArray("short values error (2)",
                            ((GcBasicTypeAccessor) other).short_value,
                            short_value);
        case PType.TYPECODE_INT:
            return assertEqualArray("int values error (1)", int_value
                    , ((GcBasicTypeAccessor) other).int_value)
                    && assertEqualArray("int values error (2)",
                            ((GcBasicTypeAccessor) other).int_value,
                            int_value);
        case PType.TYPECODE_LONG:
            return assertEqualArray("long values error (1)", long_value
                    , ((GcBasicTypeAccessor) other).long_value)
                    && assertEqualArray("long values error (2)",
                            ((GcBasicTypeAccessor) other).long_value,
                            long_value);
        case PType.TYPECODE_FLOAT:
            return assertEqualArray("float values error (1)", float_value
                    , ((GcBasicTypeAccessor) other).float_value)
                    && assertEqualArray("float values error (2)",
                            ((GcBasicTypeAccessor) other).float_value,
                            float_value);
        case PType.TYPECODE_DOUBLE:
            return assertEqualArray("double values error (1)", double_value
                    , ((GcBasicTypeAccessor) other).double_value)
                    && assertEqualArray("double values error (2)",
                            ((GcBasicTypeAccessor) other).double_value,
                            double_value);
        case PType.TYPECODE_BYTEARRAY:
        case PType.TYPECODE_CHARARRAY:
        case PType.TYPECODE_OBJBOOLEAN:
        case PType.TYPECODE_OBJBYTE:
        case PType.TYPECODE_OBJCHAR:
        case PType.TYPECODE_OBJSHORT:
        case PType.TYPECODE_OBJINT:
        case PType.TYPECODE_OBJLONG:
        case PType.TYPECODE_OBJFLOAT:
        case PType.TYPECODE_OBJDOUBLE:
        case PType.TYPECODE_STRING:
        case PType.TYPECODE_DATE:
        case PType.TYPECODE_BIGDECIMAL:
        case PType.TYPECODE_SERIALIZED:
            return assertEqualArray("object values error (1)", object_value
                    , ((GcBasicTypeAccessor) other).object_value)
                    && assertEqualArray("object values error (2)",
                            ((GcBasicTypeAccessor) other).object_value,
                            object_value);
        default :
            Assert.fail("Unmanaged Type code: " + typeCode);
            return false;
        }
    }


    private boolean assertEqualArray(String msg, boolean[] a1, boolean[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, byte[] a1, byte[] a2, boolean fail) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                if (fail) {
                    Assert.fail(msg + ": element " + a1[i] + " has not been found");
                }
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, short[] a1, short[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, int[] a1, int[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, long[] a1, long[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, float[] a1, float[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, double[] a1, double[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, char[] a1, char[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            for (int j = 0; j < a2.length && !exist; j++) {
                exist = a1[i] == a2[j];
            }
            if (!exist) {
                Assert.fail(msg + ": element " + a1[i] + " has not been found");
                return false;
            }
        }
        return true;
    }

    private boolean assertEqualArray(String msg, Object[] a1, Object[] a2) {
        Assert.assertEquals(msg + ": bad array size", a1.length, a2.length);
        if (a1.length != a2.length) {
            return false;
        }
        for (int i = 0; i < a1.length; i++) {
            boolean exist = false;
            if (a1[i] == null) {
                for (int j = 0; j < a2.length && !exist; j++) {
                    exist = a2[j] == null;
                }
            } else  if (a1[i] instanceof byte[]) {
                for (int j = 0; j < a2.length && !exist; j++) {
                    exist = assertEqualArray(msg, (byte[]) a1[i], (byte[]) a2[i], false);
                }
            } else if (a1[i] instanceof char[]) {
                for (int j = 0; j < a2.length && !exist; j++) {
                    exist = assertEqualArray(msg, (char[]) a1[i], (char[]) a2[i]);
                }
            } else if (a1[i] instanceof Date) {
                for (int j = 0; j < a2.length && !exist; j++) {
                    exist = (a1[i] == null && a1[i] == a2[j])
                            || (a1[i] != null
                                && a2[i] != null
                                && ((Date) a1[i]).getTime()==((Date) a2[i]).getTime());
                }
            } else {
                for (int j = 0; j < a2.length && !exist; j++) {
                    exist = (a1[i] == null && a1[i] == a2[j])
                            || (a1[i] != null && a1[i].equals(a2[j]));
                }
            }
            if (!exist) {
                Assert.fail(msg + ": element (" + i + "): " + toString(a1[i]) + " has not been found in " + toString(a2));
                return false;
            }
        }
        return true;
    }
    String toString(Object o) {
        if (o == null) {
            return "null";
        } else if (o instanceof byte[]) {
            return toString((byte[]) o);
        } else if (o instanceof char[]) {
            return toString((char[]) o);
        } else if (o instanceof Object[]) {
            return toString((Object[]) o);
        } else {
            return o.toString();
        }
    }
    String toString(Object[] os) {
        StringBuffer sb = new StringBuffer("[");
        String sep ="";
        for(int i=0; i<os.length; i++) {
            sb.append(sep);
            sep = ",";
            sb.append(toString(os[i]));
        }
        sb.append(']');
        return sb.toString();
    }

    String toString(byte[] bs) {
        StringBuffer sb = new StringBuffer("[");
        String sep ="";
        for(int i=0; i<bs.length; i++) {
            sb.append(sep);
            sep = ",";
            sb.append(bs[i]);
        }
        sb.append(']');
        return sb.toString();
    }

    String toString(char[] cs) {
        StringBuffer sb = new StringBuffer("[");
        String sep ="";
        for(int i=0; i<cs.length; i++) {
            sb.append(sep);
            sep = ",";
            sb.append(cs[i]);
        }
        sb.append(']');
        return sb.toString();
    }


    // IMPLEMENTATION OF METHODS FROM THE PGenClassAccessor INTERFACE

    public void paAdd(PIndexedElem elem, Object conn) throws PException {
        if (nextToAdd == -1) {
            throw new PException("Wrong number of elements retrieved!");
        }
        switch (typeCode) {
        case PType.TYPECODE_BOOLEAN:
            boolean_value[nextToAdd] = elem.pieGetBooleanElem();
            break;
        case PType.TYPECODE_BYTE:
            byte_value[nextToAdd] = elem.pieGetByteElem();
            break;
        case PType.TYPECODE_CHAR:
            char_value[nextToAdd] = elem.pieGetCharElem();
            break;
        case PType.TYPECODE_SHORT:
            short_value[nextToAdd] = elem.pieGetShortElem();
            break;
        case PType.TYPECODE_INT:
            int_value[nextToAdd] = elem.pieGetIntElem();
            break;
        case PType.TYPECODE_LONG:
            long_value[nextToAdd] = elem.pieGetLongElem();
            break;
        case PType.TYPECODE_FLOAT:
            float_value[nextToAdd] = elem.pieGetFloatElem();
            break;
        case PType.TYPECODE_DOUBLE:
            double_value[nextToAdd] = elem.pieGetDoubleElem();
            break;
        case PType.TYPECODE_OBJBOOLEAN:
        case PType.TYPECODE_OBJBYTE:
        case PType.TYPECODE_OBJCHAR:
        case PType.TYPECODE_OBJSHORT:
        case PType.TYPECODE_OBJINT:
        case PType.TYPECODE_OBJLONG:
        case PType.TYPECODE_OBJFLOAT:
        case PType.TYPECODE_OBJDOUBLE:
        case PType.TYPECODE_STRING:
        case PType.TYPECODE_DATE:
        case PType.TYPECODE_BYTEARRAY:
        case PType.TYPECODE_CHARARRAY:
        case PType.TYPECODE_BIGDECIMAL:
        case PType.TYPECODE_SERIALIZED:
            object_value[nextToAdd] = ((BasicTypeIndexedElem) elem).getObjectValue();
            break;
        default :
            throw new PExceptionTyping("Wrong field type.");
        }
        nextToAdd ++;
    }

    public boolean paDeltaSupported() {
        return false;
    }

    public int paGetNbElem() {
        return nbElem;
    }

    public Iterator paIterator() {
        return new GcIterator(this);
    }

    public void paSetNbElem(int nbelem) {
        if (nbelem == -1) {
            nextToAdd = 0;
            return;
        }
        if (nbelem == nbElem) {
            nextToAdd = 0;
            return;
        }
        nextToAdd = -1;
        Assert.assertEquals("Wrong number of elements", nbElem, nbelem);
    }

    // IMPLEMENTATION OF METHODS FROM THE PAccessor INTERFACE

    public Object getMemoryInstance() {
        return this;
    }

    // IMPLEMENTATION OF METHODS FROM THE PIndexedElemFactory INTERFACE

    public PIndexedElem createPIndexedElem() throws PException {
        return new BasicTypeIndexedElem(typeCode);
    }

    // THE ITERATOR FOR paIterator
    class GcIterator implements Iterator {
        private int nextToRead = 0;
        private GcBasicTypeAccessor theGC;

        GcIterator(GcBasicTypeAccessor gc) {
            theGC = gc;
        }

        public boolean hasNext() {
            return nextToRead < theGC.nbElem;
        }

        public Object next() {
            if (!hasNext()) {
                return null;
            }
            try {
                BasicTypeIndexedElem pie = (BasicTypeIndexedElem)
                        theGC.createPIndexedElem();
                switch (theGC.typeCode) {
                case PType.TYPECODE_BOOLEAN:
                    pie.pieSetBooleanElem(boolean_value[nextToRead]);
                    break;
                case PType.TYPECODE_BYTE:
                    pie.pieSetByteElem(byte_value[nextToRead]);
                    break;
                case PType.TYPECODE_CHAR:
                    pie.pieSetCharElem(char_value[nextToRead]);
                    break;
                case PType.TYPECODE_SHORT:
                    pie.pieSetShortElem(short_value[nextToRead]);
                    break;
                case PType.TYPECODE_INT:
                    pie.pieSetIntElem(int_value[nextToRead]);
                    break;
                case PType.TYPECODE_LONG:
                    pie.pieSetLongElem(long_value[nextToRead]);
                    break;
                case PType.TYPECODE_FLOAT:
                    pie.pieSetFloatElem(float_value[nextToRead]);
                    break;
                case PType.TYPECODE_DOUBLE:
                    pie.pieSetDoubleElem(double_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJBOOLEAN:
                    pie.pieSetObooleanElem((Boolean) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJBYTE:
                    pie.pieSetObyteElem((Byte) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJCHAR:
                    pie.pieSetOcharElem((Character) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJSHORT:
                    pie.pieSetOshortElem((Short) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJINT:
                    pie.pieSetOintElem((Integer) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJLONG:
                    pie.pieSetOlongElem((Long) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJFLOAT:
                    pie.pieSetOfloatElem((Float) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJDOUBLE:
                    pie.pieSetOdoubleElem((Double) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_STRING:
                    pie.pieSetStringElem((String) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_DATE:
                    pie.pieSetDateElem((Date) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_BYTEARRAY:
                    pie.pieSetByteArrayElem((byte[]) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_CHARARRAY:
                    pie.pieSetCharArrayElem((char[]) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_BIGDECIMAL:
                    pie.pieSetBigDecimalElem((BigDecimal) object_value[nextToRead]);
                    break;
                case PType.TYPECODE_SERIALIZED:
                    pie.setObjectValue(object_value[nextToRead]);
                    break;
                default :
                    throw new PExceptionTyping("Wrong field type.");
                }
                nextToRead++;
                return pie;
            } catch (PException e) {
                e.printStackTrace();
            }
            return null;
        }

        public void remove() {
        }
    }
}

class BasicTypeIndexedElem implements PIndexedElem {
    private boolean boolean_value;
    private byte byte_value;
    private char char_value;
    private short short_value;
    private int int_value;
    private long long_value;
    private float float_value;
    private double double_value;
    private Object object_value;
    private int typeCode;

    public BasicTypeIndexedElem(int typecode) throws PException {
        typeCode = typecode;
    }

    public Object getObjectValue() {
        return object_value;
    }

    public void setObjectValue(Object o) {
        object_value = o;
    }

    public byte getElemStatus() {
        return 0;
    }

    public boolean pieGetBooleanElem() throws PException {
        if (typeCode != PType.TYPECODE_BOOLEAN) {
            throw new PException("Wrong type!!");
        }
        return boolean_value;
    }

    public Boolean pieGetObooleanElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJBOOLEAN) {
            throw new PException("Wrong type!!");
        }
        return (Boolean) object_value;
    }

    public byte pieGetByteElem() throws PException {
        if (typeCode != PType.TYPECODE_BYTE) {
            throw new PException("Wrong type!!");
        }
        return byte_value;
    }

    public Byte pieGetObyteElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJBYTE) {
            throw new PException("Wrong type!!");
        }
        return (Byte) object_value;
    }

    public byte pieGetByteIndexField(String fn) throws PException {
        return 0;
    }

    public Byte pieGetObyteIndexField(String fn) throws PException {
        return null;
    }

    public char pieGetCharElem() throws PException {
        if (typeCode != PType.TYPECODE_CHAR) {
            throw new PException("Wrong type!!");
        }
        return char_value;
    }

    public Character pieGetOcharElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJCHAR) {
            throw new PException("Wrong type!!");
        }
        return (Character) object_value;
    }

    public char pieGetCharIndexField(String fn) throws PException {
        return 0;
    }

    public Character pieGetOcharIndexField(String fn) throws PException {
        return null;
    }

    public short pieGetShortElem() throws PException {
        if (typeCode != PType.TYPECODE_SHORT) {
            throw new PException("Wrong type!!");
        }
        return short_value;
    }

    public Short pieGetOshortElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJSHORT) {
            throw new PException("Wrong type!!");
        }
        return (Short) object_value;
    }

    public short pieGetShortIndexField(String fn) throws PException {
        return 0;
    }

    public Short pieGetOshortIndexField(String fn) throws PException {
        return null;
    }

    public int pieGetIntElem() throws PException {
        if (typeCode != PType.TYPECODE_INT) {
            throw new PException("Wrong type!!");
        }
        return int_value;
    }

    public Integer pieGetOintElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJINT) {
            throw new PException("Wrong type!!");
        }
        return (Integer) object_value;
    }

    public int pieGetIntIndexField(String fn) throws PException {
        return 0;
    }

    public Integer pieGetOintIndexField(String fn) throws PException {
        return null;
    }

    public long pieGetLongElem() throws PException {
        if (typeCode != PType.TYPECODE_LONG) {
            throw new PException("Wrong type!!");
        }
        return long_value;
    }

    public Long pieGetOlongElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJLONG) {
            throw new PException("Wrong type!!");
        }
        return (Long) object_value;
    }

    public long pieGetLongIndexField(String fn) throws PException {
        return 0;
    }

    public Long pieGetOlongIndexField(String fn) throws PException {
        return null;
    }

    public float pieGetFloatElem() throws PException {
        if (typeCode != PType.TYPECODE_FLOAT) {
            throw new PException("Wrong type!!");
        }
        return float_value;
    }

    public Float pieGetOfloatElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJFLOAT) {
            throw new PException("Wrong type!!");
        }
        return (Float) object_value;
    }

    public double pieGetDoubleElem() throws PException {
        if (typeCode != PType.TYPECODE_DOUBLE) {
            throw new PException("Wrong type!!");
        }
        return double_value;
    }

    public Double pieGetOdoubleElem() throws PException {
        if (typeCode != PType.TYPECODE_OBJDOUBLE) {
            throw new PException("Wrong type!!");
        }
        return (Double) object_value;
    }

    public String pieGetStringElem() throws PException {
        if (typeCode != PType.TYPECODE_STRING) {
            throw new PException("Wrong type!!");
        }
        return (String) object_value;
    }

    public String pieGetStringIndexField(String fn) throws PException {
        return null;
    }

    public Date pieGetDateElem() throws PException {
        if (typeCode != PType.TYPECODE_DATE) {
            throw new PException("Wrong type!!");
        }
        return (Date) object_value;
    }

    public Date pieGetDateIndexField(String fn) throws PException {
        return null;
    }

    public char[] pieGetCharArrayElem() throws PException {
        if (typeCode != PType.TYPECODE_CHARARRAY) {
            throw new PException("Wrong type!!");
        }
        return (char[]) object_value;
    }

    public byte[] pieGetByteArrayElem() throws PException {
        if (typeCode != PType.TYPECODE_BYTEARRAY) {
            throw new PException("Wrong type!!");
        }
        return (byte[]) object_value;
    }

    public Serializable pieGetSerializedElem() throws PException {
        if (typeCode != PType.TYPECODE_SERIALIZED) {
            throw new PException("Wrong type!!");
        }
        return (Serializable) object_value;
    }

	public BigDecimal pieGetBigDecimalElem() throws PException {
		if (typeCode != PType.TYPECODE_BIGDECIMAL) {
			throw new PException("Wrong type!!");
		}
		return (BigDecimal) object_value;
	}

	public BigInteger pieGetBigIntegerElem() throws PException {
		if (typeCode != PType.TYPECODE_BIGINTEGER) {
			throw new PException("Wrong type!!");
		}
		return (BigInteger) object_value;
	}

    public PName pieGetRefElem() throws PException {
        throw new PException("Wrong type!!");
    }

    public void pieSetBooleanElem(boolean value) throws PException {
        if (typeCode != PType.TYPECODE_BOOLEAN) {
            throw new PException("Wrong type!!");
        }
        boolean_value = value;
    }

    public void pieSetObooleanElem(Boolean value) throws PException {
        if (typeCode != PType.TYPECODE_OBJBOOLEAN) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetByteElem(byte value) throws PException {
        if (typeCode != PType.TYPECODE_BYTE) {
            throw new PException("Wrong type!!");
        }
        byte_value = value;
    }

    public void pieSetObyteElem(Byte value) throws PException {
        if (typeCode != PType.TYPECODE_OBJBYTE) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetByteIndexField(String fn, byte value) throws PException {
    }

    public void pieSetObyteIndexField(String fn, Byte value) throws PException {
    }

    public void pieSetCharElem(char value) throws PException {
        if (typeCode != PType.TYPECODE_CHAR) {
            throw new PException("Wrong type!!");
        }
        char_value = value;
    }

    public void pieSetOcharElem(Character value) throws PException {
        if (typeCode != PType.TYPECODE_OBJCHAR) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetCharIndexField(String fn, char value) throws PException {
    }

    public void pieSetOcharIndexField(String fn, Character value) throws PException {
    }

    public void pieSetShortElem(short value) throws PException {
        if (typeCode != PType.TYPECODE_SHORT) {
            throw new PException("Wrong type!!");
        }
        short_value = value;
    }

    public void pieSetOshortElem(Short value) throws PException {
        if (typeCode != PType.TYPECODE_OBJSHORT) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetShortIndexField(String fn, short value) throws PException {
    }

    public void pieSetOshortIndexField(String fn, Short value) throws PException {
    }

    public void pieSetIntElem(int value) throws PException {
        if (typeCode != PType.TYPECODE_INT) {
            throw new PException("Wrong type!!");
        }
        int_value = value;
    }

    public void pieSetOintElem(Integer value) throws PException {
        if (typeCode != PType.TYPECODE_OBJINT) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetIntIndexField(String fn, int value) throws PException {
    }

    public void pieSetOintIndexField(String fn, Integer value) throws PException {
    }

    public void pieSetLongElem(long value) throws PException {
        if (typeCode != PType.TYPECODE_LONG) {
            throw new PException("Wrong type!!");
        }
        long_value = value;
    }

    public void pieSetOlongElem(Long value) throws PException {
        if (typeCode != PType.TYPECODE_OBJLONG) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetLongIndexField(String fn, long value) throws PException {
    }

    public void pieSetOlongIndexField(String fn, Long value) throws PException {
    }

    public void pieSetFloatElem(float value) throws PException {
        if (typeCode != PType.TYPECODE_FLOAT) {
            throw new PException("Wrong type!!");
        }
        float_value = value;
    }

    public void pieSetOfloatElem(Float value) throws PException {
        if (typeCode != PType.TYPECODE_OBJFLOAT) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetDoubleElem(double value) throws PException {
        if (typeCode != PType.TYPECODE_DOUBLE) {
            throw new PException("Wrong type!!");
        }
        double_value = value;
    }

    public void pieSetOdoubleElem(Double value) throws PException {
        if (typeCode != PType.TYPECODE_OBJDOUBLE) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetStringElem(String value) throws PException {
        if (typeCode != PType.TYPECODE_STRING) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetStringIndexField(String fn, String value) throws PException {
    }

    public void pieSetDateElem(Date value) throws PException {
        if (typeCode != PType.TYPECODE_DATE) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetDateIndexField(String fn, Date value) throws PException {
    }

    public void pieSetCharArrayElem(char[] value) throws PException {
        if (typeCode != PType.TYPECODE_CHARARRAY) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetByteArrayElem(byte[] value) throws PException {
        if (typeCode != PType.TYPECODE_BYTEARRAY) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

    public void pieSetSerializedElem(Serializable value) throws PException {
        if (typeCode != PType.TYPECODE_SERIALIZED) {
            throw new PException("Wrong type!!");
        }
        object_value = value;
    }

	public void pieSetBigDecimalElem(BigDecimal value) throws PException {
		if (typeCode != PType.TYPECODE_BIGDECIMAL) {
			throw new PException("Wrong type!!");
		}
		object_value = value;
	}

	public void pieSetBigIntegerElem(BigInteger value) throws PException {
		if (typeCode != PType.TYPECODE_BIGINTEGER) {
			throw new PException("Wrong type!!");
		}
		object_value = value;
	}

    public void pieSetRefElem(PName value) throws PException {
        throw new PException("Wrong type!!");
    }
}
