/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.gcbasictypeelem;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Map;
import java.util.HashMap;

/**
 * @author P. Dechamboux
 */
public class TestGcBasicTypeElem extends TestRuntimeHelper {
    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.gcbasictypeelem";
    private Map cn2Binder = new HashMap();

    public TestGcBasicTypeElem(String name) throws Exception {
        super(name);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        try {
            PBinder pb = (PBinder) cn2Binder.get(className);
            if (pb == null) {
                pb = new BasidBinder(mapper, className, PClassMapping.CLEANUP_REMOVEALL);
                cn2Binder.put(className, pb);
            }
            return pb;
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
    }

    // Definition of test units

    public void commonTestSet(GcBasicTypeAccessor gcacc1,
                              String classname,
                              String loggername) throws Exception {
        GcBasicTypeAccessor gcacc2 = new GcBasicTypeAccessor(
                gcacc1.getTypeCode(), gcacc1.paGetNbElem());
        changeLogger(loggername);
        logger.log(BasicLevel.DEBUG, "export the genClassRef");
        PBinding bind = export(classname, null);
        logger.log(BasicLevel.DEBUG, "export the genClass");
        PBinding gcbind = export(classname + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        logger.log(BasicLevel.DEBUG, "write and read the genClassRef");
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        logger.log(BasicLevel.DEBUG, "write and read the genClass");
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the boolean GC value", gcacc1, gcacc2);
        logger.log(BasicLevel.DEBUG, "unexport GenClassRef");
        unexport(bind, acc1);
        logger.log(BasicLevel.DEBUG, "unexport GenClass");
        unexport(gcbind, gcacc1);
    }

    // ============================= boolean =============================== //

    private final static String CNBOOLEAN
            = "org.objectweb.jorm.pobject.gcbasictypeelem.BooleanSet";

    public void commonTestBooleanSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNBOOLEAN, LOGGER_NAME + ".boolean");
    }
    public void testBooleanSetEmpty() throws Exception {
        commonTestBooleanSet(
                new GcBasicTypeAccessor(PType.TYPECODE_BOOLEAN, 0));
    }
    public void testBooleanSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BOOLEAN, 1);
        gcacc1.setElem(0, true);
        commonTestBooleanSet(gcacc1);
    }
    public void testBooleanSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BOOLEAN, 2);
        gcacc1.setElem(0, false);
        gcacc1.setElem(1, true);
        commonTestBooleanSet(gcacc1);
    }

    // ============================= Boolean =============================== //

    private final static String CNOBOOLEAN
            = "org.objectweb.jorm.pobject.gcbasictypeelem.ObooleanSet";

    public void commonTestObooleanSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOBOOLEAN, LOGGER_NAME + ".oboolean");
    }
    public void testObooleanSetEmpty() throws Exception {
        commonTestObooleanSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJBOOLEAN, 0));
    }
    public void testObooleanSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBOOLEAN, 1);
        gcacc1.setElem(0, Boolean.TRUE);
        commonTestObooleanSet(gcacc1);
    }
    public void __testObooleanSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBOOLEAN, 1);
        gcacc1.setElem(0, null);
        commonTestObooleanSet(gcacc1);
    }
    public void testObooleanSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBOOLEAN, 2);
        gcacc1.setElem(0, Boolean.FALSE);
        gcacc1.setElem(1, Boolean.TRUE);
        commonTestObooleanSet(gcacc1);
    }
    // ============================= byte =============================== //

    private final static String CNBYTE
            = "org.objectweb.jorm.pobject.gcbasictypeelem.ByteSet";

    public void commonTestByteSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNBYTE, LOGGER_NAME + ".byte");
    }
    public void testByteSetEmpty() throws Exception {
        commonTestByteSet(
                new GcBasicTypeAccessor(PType.TYPECODE_BYTE, 0));
    }
    public void testByteSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BYTE, 1);
        gcacc1.setElem(0, (byte) 23);
        commonTestByteSet(gcacc1);
    }
    public void testByteSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BYTE, 2);
        gcacc1.setElem(0, (byte) 45);
        gcacc1.setElem(1, (byte) 67);
        commonTestByteSet(gcacc1);
    }

    // ============================= Byte =============================== //

    private final static String CNOBYTE
            = "org.objectweb.jorm.pobject.gcbasictypeelem.ObyteSet";

    public void commonTestObyteSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOBYTE, LOGGER_NAME + ".obyte");
    }
    public void testObyteSetEmpty() throws Exception {
        commonTestObyteSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJBYTE, 0));
    }
    public void testObyteSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBYTE, 1);
        gcacc1.setElem(0, new Byte((byte)23));
        commonTestObyteSet(gcacc1);
    }
    public void __testObyteSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBYTE, 1);
        gcacc1.setElem(0, null);
        commonTestObyteSet(gcacc1);
    }
    public void testObyteSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJBYTE, 2);
        gcacc1.setElem(0, new Byte((byte) 45));
        gcacc1.setElem(1, new Byte((byte) 67));
        commonTestObyteSet(gcacc1);
    }
    // ============================= short =============================== //

    private final static String CNSHORT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.ShortSet";

    public void commonTestShortSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNSHORT, LOGGER_NAME + ".short");
    }
    public void testShortSetEmpty() throws Exception {
        commonTestShortSet(
                new GcBasicTypeAccessor(PType.TYPECODE_SHORT, 0));
    }
    public void testShortSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_SHORT, 1);
        gcacc1.setElem(0, (short) 23);
        commonTestShortSet(gcacc1);
    }
    public void testShortSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_SHORT, 2);
        gcacc1.setElem(0, (short) 45);
        gcacc1.setElem(1, (short) 67);
        commonTestShortSet(gcacc1);
    }

    // ============================= Short =============================== //

    private final static String CNOSHORT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OshortSet";

    public void commonTestOshortSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOSHORT, LOGGER_NAME + ".oshort");
    }
    public void testOshortSetEmpty() throws Exception {
        commonTestOshortSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJSHORT, 0));
    }
    public void testOshortSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJSHORT, 1);
        gcacc1.setElem(0, new Short((short)23));
        commonTestOshortSet(gcacc1);
    }
    public void __testOshortSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJSHORT, 1);
        gcacc1.setElem(0, null);
        commonTestOshortSet(gcacc1);
    }
    public void testOshortSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJSHORT, 2);
        gcacc1.setElem(0, new Short((short) 45));
        gcacc1.setElem(1, new Short((short) 67));
        commonTestOshortSet(gcacc1);
    }
    // ============================= int =============================== //

    private final static String CNINT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.IntSet";

    public void commonTestIntSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNINT, LOGGER_NAME + ".int");
    }
    public void testIntSetEmpty() throws Exception {
        commonTestIntSet(
                new GcBasicTypeAccessor(PType.TYPECODE_INT, 0));
    }
    public void testIntSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_INT, 1);
        gcacc1.setElem(0, 23);
        commonTestIntSet(gcacc1);
    }
    public void testIntSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_INT, 2);
        gcacc1.setElem(0, 45);
        gcacc1.setElem(1, 67);
        commonTestIntSet(gcacc1);
    }

    // ============================= Int =============================== //

    private final static String CNOINT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OintSet";

    public void commonTestOintSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOINT, LOGGER_NAME + ".oint");
    }
    public void testOintSetEmpty() throws Exception {
        commonTestOintSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJINT, 0));
    }
    public void testOintSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJINT, 1);
        gcacc1.setElem(0, new Integer(23));
        commonTestOintSet(gcacc1);
    }
    public void __testOintSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJINT, 1);
        gcacc1.setElem(0, null);
        commonTestOintSet(gcacc1);
    }
    public void testOintSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJINT, 2);
        gcacc1.setElem(0, new Integer(45));
        gcacc1.setElem(1, new Integer(67));
        commonTestOintSet(gcacc1);
    }
    // ============================= long =============================== //

    private final static String CNLONG
            = "org.objectweb.jorm.pobject.gcbasictypeelem.LongSet";

    public void commonTestLongSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNLONG, LOGGER_NAME + ".long");
    }
    public void testLongSetEmpty() throws Exception {
        commonTestLongSet(
                new GcBasicTypeAccessor(PType.TYPECODE_LONG, 0));
    }
    public void testLongSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_LONG, 1);
        gcacc1.setElem(0, (long) 23);
        commonTestLongSet(gcacc1);
    }
    public void testLongSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_LONG, 2);
        gcacc1.setElem(0, (long) 45);
        gcacc1.setElem(1, (long) 67);
        commonTestLongSet(gcacc1);
    }

    // ============================= Long =============================== //

    private final static String CNOLONG
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OlongSet";

    public void commonTestOlongSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOLONG, LOGGER_NAME + ".olong");
    }
    public void testOlongSetEmpty() throws Exception {
        commonTestOlongSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJLONG, 0));
    }
    public void testOlongSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJLONG, 1);
        gcacc1.setElem(0, new Long((long)23));
        commonTestOlongSet(gcacc1);
    }
    public void __testOlongSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJLONG, 1);
        gcacc1.setElem(0, null);
        commonTestOlongSet(gcacc1);
    }
    public void testOlongSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJLONG, 2);
        gcacc1.setElem(0, new Long((long) 45));
        gcacc1.setElem(1, new Long((long) 67));
        commonTestOlongSet(gcacc1);
    }
    // ============================= float =============================== //

    private final static String CNFLOAT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.FloatSet";

    public void commonTestFloatSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNFLOAT, LOGGER_NAME + ".float");
    }
    public void testFloatSetEmpty() throws Exception {
        commonTestFloatSet(
                new GcBasicTypeAccessor(PType.TYPECODE_FLOAT, 0));
    }
    public void testFloatSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_FLOAT, 1);
        gcacc1.setElem(0, (float) 23);
        commonTestFloatSet(gcacc1);
    }
    public void testFloatSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_FLOAT, 2);
        gcacc1.setElem(0, (float) 45);
        gcacc1.setElem(1, (float) 67);
        commonTestFloatSet(gcacc1);
    }

    // ============================= Float =============================== //

    private final static String CNOFLOAT
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OfloatSet";

    public void commonTestOfloatSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOFLOAT, LOGGER_NAME + ".ofloat");
    }
    public void testOfloatSetEmpty() throws Exception {
        commonTestOfloatSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJFLOAT, 0));
    }
    public void testOfloatSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJFLOAT, 1);
        gcacc1.setElem(0, new Float((float)23));
        commonTestOfloatSet(gcacc1);
    }
    public void __testOfloatSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJFLOAT, 1);
        gcacc1.setElem(0, null);
        commonTestOfloatSet(gcacc1);
    }
    public void testOfloatSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJFLOAT, 2);
        gcacc1.setElem(0, new Float((float) 45));
        gcacc1.setElem(1, new Float((float) 67));
        commonTestOfloatSet(gcacc1);
    }
    // ============================= double =============================== //

    private final static String CNDOUBLE
            = "org.objectweb.jorm.pobject.gcbasictypeelem.DoubleSet";

    public void commonTestDoubleSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNDOUBLE, LOGGER_NAME + ".double");
    }
    public void testDoubleSetEmpty() throws Exception {
        commonTestDoubleSet(
                new GcBasicTypeAccessor(PType.TYPECODE_DOUBLE, 0));
    }
    public void testDoubleSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_DOUBLE, 1);
        gcacc1.setElem(0, (double) 23);
        commonTestDoubleSet(gcacc1);
    }
    public void testDoubleSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_DOUBLE, 2);
        gcacc1.setElem(0, (double) 45);
        gcacc1.setElem(1, (double) 67);
        commonTestDoubleSet(gcacc1);
    }

    // ============================= Double =============================== //

    private final static String CNODOUBLE
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OdoubleSet";

    public void commonTestOdoubleSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNODOUBLE, LOGGER_NAME + ".odouble");
    }
    public void testOdoubleSetEmpty() throws Exception {
        commonTestOdoubleSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJDOUBLE, 0));
    }
    public void testOdoubleSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJDOUBLE, 1);
        gcacc1.setElem(0, new Double((double)23));
        commonTestOdoubleSet(gcacc1);
    }
    public void __testOdoubleSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJDOUBLE, 1);
        gcacc1.setElem(0, null);
        commonTestOdoubleSet(gcacc1);
    }
    public void testOdoubleSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJDOUBLE, 2);
        gcacc1.setElem(0, new Double((double) 45));
        gcacc1.setElem(1, new Double((double) 67));
        commonTestOdoubleSet(gcacc1);
    }
    // ============================= char =============================== //

    private final static String CNCHAR
            = "org.objectweb.jorm.pobject.gcbasictypeelem.CharSet";

    public void commonTestCharSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNCHAR, LOGGER_NAME + ".char");
    }
    public void testCharSetEmpty() throws Exception {
        commonTestCharSet(
                new GcBasicTypeAccessor(PType.TYPECODE_CHAR, 0));
    }
    public void testCharSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_CHAR, 1);
        gcacc1.setElem(0, 'e');
        commonTestCharSet(gcacc1);
    }
    public void testCharSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_CHAR, 2);
        gcacc1.setElem(0, 'a');
        gcacc1.setElem(1, 'b');
        commonTestCharSet(gcacc1);
    }

    // ============================= Char =============================== //

    private final static String CNOCHAR
            = "org.objectweb.jorm.pobject.gcbasictypeelem.OcharSet";

    public void commonTestOcharSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNOCHAR, LOGGER_NAME + ".ochar");
    }
    public void testOcharSetEmpty() throws Exception {
        commonTestOcharSet(
                new GcBasicTypeAccessor(PType.TYPECODE_OBJCHAR, 0));
    }
    public void testOcharSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJCHAR, 1);
        gcacc1.setElem(0, new Character('e'));
        commonTestOcharSet(gcacc1);
    }
    public void __testOcharSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJCHAR, 1);
        gcacc1.setElem(0, null);
        commonTestOcharSet(gcacc1);
    }
    public void testOcharSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_OBJCHAR, 2);
        gcacc1.setElem(0, new Character('a'));
        gcacc1.setElem(1, new Character('b'));
        commonTestOcharSet(gcacc1);
    }
    // ============================= String =============================== //

    private final static String CNSTRING
            = "org.objectweb.jorm.pobject.gcbasictypeelem.StringSet";

    public void commonTestStringSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNSTRING, LOGGER_NAME + ".string");
    }
    public void testStringSetEmpty() throws Exception {
        commonTestStringSet(
                new GcBasicTypeAccessor(PType.TYPECODE_STRING, 0));
    }
    public void testStringSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_STRING, 1);
        gcacc1.setElem(0, new String("azerty"));
        commonTestStringSet(gcacc1);
    }
    public void __testStringSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_STRING, 1);
        gcacc1.setElem(0, null);
        commonTestStringSet(gcacc1);
    }
    public void testStringSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_STRING, 2);
        gcacc1.setElem(0, new String("qsdfgh"));
        gcacc1.setElem(1, new String("wxcvbn"));
        commonTestStringSet(gcacc1);
    }

    // ============================= Date =============================== //

    private final static String CNDATE
            = "org.objectweb.jorm.pobject.gcbasictypeelem.DateSet";

    public void commonTestDateSet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNDATE, LOGGER_NAME + ".date");
    }
    public void testDateSetEmpty() throws Exception {
        commonTestDateSet(
                new GcBasicTypeAccessor(PType.TYPECODE_DATE, 0));
    }
    public void testDateSetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_DATE, 1);
        gcacc1.setElem(0, getTime());
        commonTestDateSet(gcacc1);
    }
    public void __testDateSetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_DATE, 1);
        gcacc1.setElem(0, null);
        commonTestDateSet(gcacc1);
    }
    public void testDateSetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_DATE, 2);
        gcacc1.setElem(0, getTime());
        Thread.sleep(1000);
        gcacc1.setElem(1, getTime());
        commonTestDateSet(gcacc1);
    }

    // ============================= byte[] =============================== //

    private final static String CNBYTEARRAY
            = "org.objectweb.jorm.pobject.gcbasictypeelem.BytearraySet";

    public void commonTestByteArraySet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNBYTEARRAY, LOGGER_NAME + ".bytearray");
    }
    public void testByteArraySetEmpty() throws Exception {
        commonTestByteArraySet(
                new GcBasicTypeAccessor(PType.TYPECODE_BYTEARRAY, 0));
    }
    public void testByteArraySetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BYTEARRAY, 1);
        gcacc1.setElem(0, new byte[]{1,2,3});
        commonTestByteArraySet(gcacc1);
    }
    public void __testByteArraySetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BYTEARRAY, 1);
        gcacc1.setElem(0, null);
        commonTestByteArraySet(gcacc1);
    }
    public void testByteArraySetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_BYTEARRAY, 2);
        gcacc1.setElem(0, new byte[]{4,5,6});
        gcacc1.setElem(1, new byte[]{7,8,9});
        commonTestByteArraySet(gcacc1);
    }

    // ============================= char[] =============================== //

    private final static String CNCHARARRAY
            = "org.objectweb.jorm.pobject.gcbasictypeelem.ChararraySet";

    public void commonTestCharArraySet(GcBasicTypeAccessor gcacc1)
            throws Exception {
        commonTestSet(gcacc1, CNCHARARRAY, LOGGER_NAME + ".chararray");
    }
    public void testCharArraySetEmpty() throws Exception {
        commonTestCharArraySet(
                new GcBasicTypeAccessor(PType.TYPECODE_CHARARRAY, 0));
    }
    public void testCharArraySetOne() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_CHARARRAY, 1);
        gcacc1.setElem(0, new char[]{'w','x','c','v','b','n'});
        commonTestCharArraySet(gcacc1);
    }
    public void __testCharArraySetNull() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_CHARARRAY, 1);
        gcacc1.setElem(0, null);
        commonTestCharArraySet(gcacc1);
    }
    public void testCharArraySetSeveral() throws Exception {
        GcBasicTypeAccessor gcacc1 = new GcBasicTypeAccessor(PType.TYPECODE_CHARARRAY, 2);
        gcacc1.setElem(0, new char[]{'a','z','e','r','t','y'});
        gcacc1.setElem(1, new char[]{'q','s','d','f','g','h'});
        commonTestCharArraySet(gcacc1);
    }
}
