/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.gcindex;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.type.api.PType;

import java.util.Map;
import java.util.HashMap;
import java.util.Date;

/**
 * @author P. Dechamboux
 */
public class TestGcIndexStoreLoad extends TestRuntimeHelper {
    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.gcindex";
    private Map cn2Binder = new HashMap();

    public TestGcIndexStoreLoad(String name) throws Exception {
        super(name);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        try {
            PBinder pb = (PBinder) cn2Binder.get(className);
            if (pb == null) {
                pb = new BasidBinder(mapper, className, PClassMapping.CLEANUP_REMOVEALL);
                cn2Binder.put(className, pb);
            }
            return pb;
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
    }

    // Definition of test units

    // test units for byte index
    private final static String CNIBYTE = "org.objectweb.jorm.pobject.gcindex.ByteIndexCol";

    public void testIByteColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, 0);
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIByteColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIByteColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIByteColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIByteColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIByteColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".ibyte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_BYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIBYTE, null);
        PBinding gcbind = export(CNIBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the byte indexed GC value", gcacc1, gcacc2);
    }

    // test units for char index
    private final static String CNICHAR = "org.objectweb.jorm.pobject.gcindex.CharIndexCol";

    public void testICharColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, 0);
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    public void testICharColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    public void testICharColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    public void testICharColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    public void testICharColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    public void testICharCol94Elem() throws Exception {
        int NBELEM = 94;
        changeLogger(LOGGER_NAME + ".ichar");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_CHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNICHAR, null);
        PBinding gcbind = export(CNICHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the char indexed GC value", gcacc1, gcacc2);
    }

    // test units for short index
    private final static String CNISHORT = "org.objectweb.jorm.pobject.gcindex.ShortIndexCol";

    public void testIShortColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, 0);
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    public void testIShortColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    public void testIShortColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    public void testIShortColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    public void testIShortColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    public void testIShortColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".ishort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_SHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNISHORT, null);
        PBinding gcbind = export(CNISHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the short indexed GC value", gcacc1, gcacc2);
    }

    // test units for short index
    private final static String CNIINT = "org.objectweb.jorm.pobject.gcindex.IntIndexCol";

    public void testIIntColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, 0);
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    public void testIIntColThousandElem() throws Exception {
        int NBELEM = 1000;
        changeLogger(LOGGER_NAME + ".iint");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_INT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIINT, null);
        PBinding gcbind = export(CNIINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the int indexed GC value", gcacc1, gcacc2);
    }

    // test units for short index
    private final static String CNILONG = "org.objectweb.jorm.pobject.gcindex.LongIndexCol";

    public void testILongColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, 0);
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    public void testILongColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    public void testILongColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    public void testILongColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    public void testILongColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    public void testILongColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".ilong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_LONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNILONG, null);
        PBinding gcbind = export(CNILONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the long indexed GC value", gcacc1, gcacc2);
    }

    // test units for Byte index
    private final static String CNIOBJBYTE = "org.objectweb.jorm.pobject.gcindex.ObyteIndexCol";

    public void testIObyteColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, 0);
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIObyteColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIObyteColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIObyteColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIObyteColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    public void testIObyteColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iByte");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJBYTE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJBYTE, null);
        PBinding gcbind = export(CNIOBJBYTE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Byte indexed GC value", gcacc1, gcacc2);
    }

    // test units for Character index
    private final static String CNIOBJCHAR = "org.objectweb.jorm.pobject.gcindex.OcharIndexCol";

    public void testIOCharacterColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, 0);
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    public void testIOCharacterColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    public void testIOCharacterColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    public void testIOCharacterColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    public void testIOCharacterColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    public void testIOCharacterCol94Elem() throws Exception {
        int NBELEM = 94;
        changeLogger(LOGGER_NAME + ".iCharacter");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJCHAR, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJCHAR, null);
        PBinding gcbind = export(CNIOBJCHAR + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Character indexed GC value", gcacc1, gcacc2);
    }

    // test units for Short index
    private final static String CNIOBJSHORT = "org.objectweb.jorm.pobject.gcindex.OshortIndexCol";

    public void testIOshortColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, 0);
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    public void testIOShortColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    public void testIOShortColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    public void testIOShortColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    public void testIOShortColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    public void testIOShortColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iShort");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJSHORT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJSHORT, null);
        PBinding gcbind = export(CNIOBJSHORT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Short indexed GC value", gcacc1, gcacc2);
    }

    // test units for Integer index
    private final static String CNIOBJINT = "org.objectweb.jorm.pobject.gcindex.OintIndexCol";

    public void testIOintColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, 0);
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    public void testIOintColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    public void testIOintColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    public void testIOintColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    public void testIOintColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    public void testIOintColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iInteger");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJINT, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJINT, null);
        PBinding gcbind = export(CNIOBJINT + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Integer indexed GC value", gcacc1, gcacc2);
    }

    // test units for Long index
    private final static String CNIOBJLONG = "org.objectweb.jorm.pobject.gcindex.OlongIndexCol";

    public void testIOlongColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, 0);
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    public void testIOlongColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, 1);
        gcacc1.setElem(0, null);
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    public void testIOlongColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    public void testIOlongColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    public void testIOlongColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    public void testIOlongColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iLong");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_OBJLONG, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(i, "The string " + i);
        }
        PBinding bind = export(CNIOBJLONG, null);
        PBinding gcbind = export(CNIOBJLONG + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Long indexed GC value", gcacc1, gcacc2);
    }

    // test units for String index
    private final static String CNISTRING = "org.objectweb.jorm.pobject.gcindex.StringIndexCol";

    public void testIStringColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, 0);
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    public void testIStringColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, 1);
        gcacc1.setElem("index_" + 0, null);
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    public void testIStringColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem("index_" + i, "The string " + i);
        }
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    public void testIStringColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem("index_" + i, "The string " + i);
        }
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    public void testIStringColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem("index_" + i, "The string " + i);
        }
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    public void testIStringColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iString");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_STRING, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem("index_" + i, "The string " + i);
        }
        PBinding bind = export(CNISTRING, null);
        PBinding gcbind = export(CNISTRING + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the String indexed GC value", gcacc1, gcacc2);
    }

    // test units for Date index
    private final static String CNIDATE = "org.objectweb.jorm.pobject.gcindex.DateIndexCol";

    public void testIDateColEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, 0);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, 0);
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }

    public void testIDateColNullElem() throws Exception {
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, 1);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, 1);
        gcacc1.setElem(getTime(), null);
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }

    public void testIDateColOneElem() throws Exception {
        int NBELEM = 1;
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(new Date((long) (3456000 + i * 10000000)), "The string " + i);
        }
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }

    public void testIDateColTwoElem() throws Exception {
        int NBELEM = 2;
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(new Date((long) (3456000 + i * 10000000)), "The string " + i);
        }
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }

    public void testIDateColTenElem() throws Exception {
        int NBELEM = 10;
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(new Date((long) (3456000 + i * 10000000)), "The string " + i);
        }
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }

    public void testIDateColHundredElem() throws Exception {
        int NBELEM = 100;
        changeLogger(LOGGER_NAME + ".iDate");
        GcIndexAccessor gcacc1 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        GcIndexAccessor gcacc2 = new GcIndexAccessor(PType.TYPECODE_DATE, NBELEM);
        for (int i = 0; i < NBELEM; i++) {
            gcacc1.setElem(new Date((long) (3456000 + i * 10000000)), "The string " + i);
        }
        PBinding bind = export(CNIDATE, null);
        PBinding gcbind = export(CNIDATE + "/mygc", null);
        GcRefAccessor acc1 = new GcRefAccessor(gcbind.getPName());
        GcRefAccessor acc2 = new GcRefAccessor(null);
        writeRead(bind, acc1, acc2);
        assertEquals("Cannot read the GC reference", acc1, acc2);
        writeRead(gcbind, gcacc1, gcacc2);
        assertEquals("Cannot read the Date indexed GC value", gcacc1, gcacc2);
    }
}
