/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.multitable;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdPNG;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * @author S.Chassande-Barrioz
 */
public class TestPrimitiveMultitable extends TestRuntimeHelper {

    private final static String LOGGER_NAME
        = "test.org.objectweb.jorm.multitable.primitive";

    private final static String Primitive_SN_MT
        = "org.objectweb.jorm.pobject.multitable.Primitive_SN_MT";
    private final static String Primitive_CN_MT
        = "org.objectweb.jorm.pobject.multitable.Primitive_CN_MT";

    public TestPrimitiveMultitable(String s) throws Exception {
        super(s);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        try {
            if (Primitive_SN_MT.equals(className)) {
                PBinder binder = new BasidBinder();
                binder.setNullPName(new Long(-1));
                ((BasidBinder) binder).setCodingType(PNameCoder.CTLONG);
                return binder;
            } else if (Primitive_CN_MT.equals(className)) {
                PBinder binder = new PolymorphIdBinder();
                binder.setNullPName(new PolyIdPNG());
                return binder;
            }
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
        throw new PException("Unmanaged class " + className);
    }

    public class PolyIdPNG implements PolymorphIdPNG {
        public long cid = -1;
        public long oid = -1;

        public PolyIdPNG() {
        }

        public PolyIdPNG(long cid, long oid) {
            this.cid = cid;
            this.oid = oid;
        }

        public long pnGetObjectId(Object o) throws PException {
            return oid;
        }

        public long pnGetClassId(Object o) throws PException {
            return cid;
        }
    }

    private void primitiveTest(PBinding pb) {
        Primitive_MT acc1 = new Primitive_MT();
        acc1.setFx(1, 2, 3, 4, 5, 6, 7, 8);
        Primitive_MT acc2 = new Primitive_MT();
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad values", acc1, acc2);
        Primitive_MT acc3 = new Primitive_MT();
        acc3.setFx(-1, -2, 3, 4, -5, -6, 7, 8);
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad values", acc3, acc2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
    }

    public void testPrimitiveSingle() {
        changeLogger(LOGGER_NAME + ".single");
        logger.log(BasicLevel.DEBUG, "testPrimitiveSingle begin");
        primitiveTest(export(Primitive_SN_MT, new Long(45)));
        logger.log(BasicLevel.DEBUG, "testPrimitiveSingle end");
    }

    public void testPrimitiveComposite() {
        changeLogger(LOGGER_NAME + ".composite");
        logger.log(BasicLevel.DEBUG, "testPrimitiveComposite begin");
        primitiveTest(export(Primitive_CN_MT, new PolyIdPNG(56, 78)));
        logger.log(BasicLevel.DEBUG, "testPrimitiveComposite end");
    }
}
