/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.namedef;

import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.pobject.compositename.CompositeByteIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeObyteIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeShortIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeOshortIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeIntIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeOintIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeLongIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeOlongIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeStringIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeDateIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeBigDecimalIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeBigIntegerIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeCharIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeOcharIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeBytearrayIdPNG;
import org.objectweb.jorm.pobject.compositename.CompositeChararrayIdPNG;
import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * @author S.Chassande-Barrioz
 */
public class TestNameDefComposite
    extends TestRuntimeHelper {

    private final static String LOGGER_NAME
        = "test.org.objectweb.jorm.compositename";

    public TestNameDefComposite(String s) throws Exception {
        super(s);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        int idx = className.lastIndexOf('.');
        String cn = className.substring(idx + 1);
        if (!cn.startsWith("AComposite")) {
            throw new Exception(
                "Unable to find a binder for the class: " + className);
        }
        String bcn = className.substring(0, idx) + "." + cn.substring(1) + "Binder";
        logger.log(BasicLevel.DEBUG, "Allocating the Binder " + bcn
            + " to manage the class " + className);
        PBinder binder = (PBinder) Class.forName(bcn).newInstance();
        if (CNBYTE.equals(className)) {
            binder.setNullPName(new ByteIdPNG(new Byte((byte) -1)));
        } else if (CNOBYTE.equals(className)) {
            binder.setNullPName(new ObyteIdPNG(null));
        } else if (CNSHORT.equals(className)) {
            binder.setNullPName(new ShortIdPNG(new Short((short) -1)));
        } else if (CNOSHORT.equals(className)) {
            binder.setNullPName(new OshortIdPNG(null));
        } else if (CNINT.equals(className)) {
            binder.setNullPName(new IntIdPNG(new Integer(-1)));
        } else if (CNOINT.equals(className)) {
            binder.setNullPName(new OintIdPNG(null));
        } else if (CNLONG.equals(className)) {
            binder.setNullPName(new LongIdPNG(new Long(-1)));
        } else if (CNOLONG.equals(className)) {
            binder.setNullPName(new OlongIdPNG(null));
        } else if (CNCHAR.equals(className)) {
            binder.setNullPName(new CharIdPNG(new Character('0')));
        } else if (CNOCHAR.equals(className)) {
            binder.setNullPName(new OcharIdPNG(null));
        } else if (CNSTRING.equals(className)) {
            binder.setNullPName(new StringIdPNG(null));
		} else if (CNDATE.equals(className)) {
			binder.setNullPName(new DateIdPNG(null));
		} else if (CNBIGDECIMAL.equals(className)) {
			binder.setNullPName(new BigDecimalIdPNG(null));
		} else if (CNBIGINTEGER.equals(className)) {
			binder.setNullPName(new BigIntegerIdPNG(null));
        } else if (CNCHARARRAY.equals(className)) {
            binder.setNullPName(new ChararrayIdPNG(null));
        } else if (CNBYTEARRAY.equals(className)) {
            binder.setNullPName(new BytearrayIdPNG(null));
        }
        return binder;
    }

    private static int f1 = 0;

    private static synchronized int getF1() {
        return f1++;
    }

    private final static String CNBYTE = "org.objectweb.jorm.pobject.compositename.ACompositeByteId";
    class ByteIdPNG implements CompositeByteIdPNG {
        public Byte val;

        public ByteIdPNG(Byte val) {
            this.val = val;
        }

        public byte pnGetCnf1(Object ctx) throws PException {
            return val.byteValue();
        }
    }
    public void testByte() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        logger.log(BasicLevel.DEBUG, "testByte begin");
        int f = getF1();
        Byte val = new Byte((byte) 56);
        NameDefAccessor acc2 = new NameDefAccessor();
        ByteIdPNG png = new ByteIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNBYTE, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val.byteValue(), pb.getPName().encodeByte());
        assertEquals("Bad pname encoded value",
            png.val.byteValue(), ((CompositeByteIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testByte end");
    }

    private final static String CNOBYTE = "org.objectweb.jorm.pobject.compositename.ACompositeObyteId";
    class ObyteIdPNG implements CompositeObyteIdPNG {
        public Byte val;

        public ObyteIdPNG(Byte val) {
            this.val = val;
        }

        public Byte pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testObyte() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        logger.log(BasicLevel.DEBUG, "testObyte begin");
        int f = getF1();
        Byte val = new Byte((byte) 56);
        NameDefAccessor acc2 = new NameDefAccessor();
        ObyteIdPNG png = new ObyteIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNOBYTE, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeObyte());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeObyteIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testObyte end");
    }

    private final static String CNCHAR = "org.objectweb.jorm.pobject.compositename.ACompositeCharId";
    class CharIdPNG implements CompositeCharIdPNG {
        public Character val;

        public CharIdPNG(Character val) {
            this.val = val;
        }

        public char pnGetCnf1(Object ctx) throws PException {
            return val.charValue();
        }
    }
    public void testChar() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        logger.log(BasicLevel.DEBUG, "testChar begin");
        int f = getF1();
        Character val = new Character('e');
        NameDefAccessor acc2 = new NameDefAccessor();
        CharIdPNG png = new CharIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNCHAR, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val.charValue(), pb.getPName().encodeChar());
        assertEquals("Bad pname encoded value",
            png.val.charValue(), ((CompositeCharIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testChar end");
    }

    private final static String CNOCHAR = "org.objectweb.jorm.pobject.compositename.ACompositeOcharId";
    class OcharIdPNG implements CompositeOcharIdPNG {
        public Character val;

        public OcharIdPNG(Character val) {
            this.val = val;
        }

        public Character pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testOchar() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        logger.log(BasicLevel.DEBUG, "testOchar begin");
        int f = getF1();
        Character val = new Character('e');
        NameDefAccessor acc2 = new NameDefAccessor();
        OcharIdPNG png = new OcharIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNOCHAR, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeOchar());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeOcharIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOchar end");
    }

    private final static String CNSHORT = "org.objectweb.jorm.pobject.compositename.ACompositeShortId";
    class ShortIdPNG implements CompositeShortIdPNG {
        public Short val;

        public ShortIdPNG(Short val) {
            this.val = val;
        }

        public short pnGetCnf1(Object ctx) throws PException {
            return val.shortValue();
        }
    }
    public void testShort() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        logger.log(BasicLevel.DEBUG, "testShort begin");
        int f = getF1();
        Short val = new Short((short) 5600);
        NameDefAccessor acc2 = new NameDefAccessor();
        ShortIdPNG png = new ShortIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNSHORT, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val.shortValue(), pb.getPName().encodeShort());
        assertEquals("Bad pname encoded value",
            png.val.shortValue(), ((CompositeShortIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testShort end");
    }

    private final static String CNOSHORT = "org.objectweb.jorm.pobject.compositename.ACompositeOshortId";
    class OshortIdPNG implements CompositeOshortIdPNG {
        public Short val;

        public OshortIdPNG(Short val) {
            this.val = val;
        }

        public Short pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testOshort() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        logger.log(BasicLevel.DEBUG, "testOshort begin");
        int f = getF1();
        Short val = new Short((short) 5600);
        NameDefAccessor acc2 = new NameDefAccessor();
        OshortIdPNG png = new OshortIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNOSHORT, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeOshort());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeOshortIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOshort end");
    }

    private final static String CNINT = "org.objectweb.jorm.pobject.compositename.ACompositeIntId";
    class IntIdPNG implements CompositeIntIdPNG {
        public Integer val;

        public IntIdPNG(Integer val) {
            this.val = val;
        }

        public int pnGetCnf1(Object ctx) throws PException {
            return val.intValue();
        }
    }
    public void testInt() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        logger.log(BasicLevel.DEBUG, "testInt begin");
        int f = getF1();
        Integer val = new Integer(560000);
        NameDefAccessor acc2 = new NameDefAccessor();
        IntIdPNG png = new IntIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNINT, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val.intValue(), pb.getPName().encodeInt());
        assertEquals("Bad pname encoded value",
            png.val.intValue(), ((CompositeIntIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testInt end");
    }

    private final static String CNOINT = "org.objectweb.jorm.pobject.compositename.ACompositeOintId";
    class OintIdPNG implements CompositeOintIdPNG {
        public Integer val;

        public OintIdPNG(Integer val) {
            this.val = val;
        }

        public Integer pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testOint() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        logger.log(BasicLevel.DEBUG, "testOint begin");
        int f = getF1();
        Integer val = new Integer(560000);
        NameDefAccessor acc2 = new NameDefAccessor();
        OintIdPNG png = new OintIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNOINT, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeOint());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeOintIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOint end");
    }

    private final static String CNLONG = "org.objectweb.jorm.pobject.compositename.ACompositeLongId";
    class LongIdPNG implements CompositeLongIdPNG {
        public Long val;

        public LongIdPNG(Long val) {
            this.val = val;
        }

        public long pnGetCnf1(Object ctx) throws PException {
            return val.longValue();
        }
    }
    public void testLong() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        logger.log(BasicLevel.DEBUG, "testLong begin");
        int f = getF1();
        Long val = new Long(56000000);
        NameDefAccessor acc2 = new NameDefAccessor();
        LongIdPNG png = new LongIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNLONG, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val.longValue(), pb.getPName().encodeLong());
        assertEquals("Bad pname encoded value",
            png.val.longValue(), ((CompositeLongIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testLong end");
    }

    private final static String CNOLONG = "org.objectweb.jorm.pobject.compositename.ACompositeOlongId";
    class OlongIdPNG implements CompositeOlongIdPNG {
        public Long val;

        public OlongIdPNG(Long val) {
            this.val = val;
        }

        public Long pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testOlong() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        logger.log(BasicLevel.DEBUG, "testOlong begin");
        int f = getF1();
        Long val = new Long(56000000);
        NameDefAccessor acc2 = new NameDefAccessor();
        OlongIdPNG png = new OlongIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNOLONG, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeOlong());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeOlongIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOlong end");
    }

    private final static String CNSTRING = "org.objectweb.jorm.pobject.compositename.ACompositeStringId";
    class StringIdPNG implements CompositeStringIdPNG {
        public String val;

        public StringIdPNG(String val) {
            this.val = val;
        }

        public String pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testString() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        logger.log(BasicLevel.DEBUG, "testString begin");
        int f = getF1();
        String val = "azerty";
        NameDefAccessor acc2 = new NameDefAccessor();
        StringIdPNG png = new StringIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNSTRING, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeStringIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testString end");
    }

	private final static String CNDATE = "org.objectweb.jorm.pobject.compositename.ACompositeDateId";
	class DateIdPNG implements CompositeDateIdPNG {
		public Date val;

		public DateIdPNG(Date val) {
			this.val = val;
		}

		public Date pnGetCnf1(Object ctx) throws PException {
			return val;
		}
	}
	public void testDate() throws Exception {
		changeLogger(LOGGER_NAME + ".date");
		logger.log(BasicLevel.DEBUG, "testDate begin");
		int f = getF1();
		Date val = getTime();
		NameDefAccessor acc2 = new NameDefAccessor();
		DateIdPNG png = new DateIdPNG(val);
		logger.log(BasicLevel.DEBUG, "export");
		PBinding pb = export(CNDATE, png);
		NameDefAccessor acc1 = new NameDefAccessor(f, val,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value",
			png.val, pb.getPName().encodeDate());
		assertEquals("Bad pname encoded value",
			png.val, ((CompositeDateIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f, val,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testDate end");
	}

	private final static String CNBIGDECIMAL = "org.objectweb.jorm.pobject.compositename.ACompositeBigDecimalId";
	class BigDecimalIdPNG implements CompositeBigDecimalIdPNG {
		public BigDecimal val;

		public BigDecimalIdPNG(BigDecimal val) {
			this.val = val;
		}

		public BigDecimal pnGetCnf1(Object ctx) throws PException {
			return val;
		}
	}
	public void testBigDecimal() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		logger.log(BasicLevel.DEBUG, "testBigDecimal begin");
		int f = getF1();
		BigDecimal val = new BigDecimal("123456789.45");
		NameDefAccessor acc2 = new NameDefAccessor();
		BigDecimalIdPNG png = new BigDecimalIdPNG(val);
		logger.log(BasicLevel.DEBUG, "export");
		PBinding pb = export(CNBIGDECIMAL, png);
		NameDefAccessor acc1 = new NameDefAccessor(f, val,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value",
			png.val, pb.getPName().encodeBigDecimal());
		assertEquals("Bad pname encoded value",
			png.val, ((CompositeBigDecimalIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f, val,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testBigDecimal end");
	}

	private final static String CNBIGINTEGER = "org.objectweb.jorm.pobject.compositename.ACompositeBigIntegerId";
	class BigIntegerIdPNG implements CompositeBigIntegerIdPNG {
		public BigInteger val;

		public BigIntegerIdPNG(BigInteger val) {
			this.val = val;
		}

		public BigInteger pnGetCnf1(Object ctx) throws PException {
			return val;
		}
	}
	public void testBigInteger() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		logger.log(BasicLevel.DEBUG, "testBigInteger begin");
		int f = getF1();
		BigInteger val = new BigInteger("123456789");
		NameDefAccessor acc2 = new NameDefAccessor();
		BigIntegerIdPNG png = new BigIntegerIdPNG(val);
		logger.log(BasicLevel.DEBUG, "export");
		PBinding pb = export(CNBIGINTEGER, png);
		NameDefAccessor acc1 = new NameDefAccessor(f, val,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value",
			png.val, pb.getPName().encodeBigInteger());
		assertEquals("Bad pname encoded value",
			png.val, ((CompositeBigIntegerIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f, val,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testBigInteger end");
	}

    private final static String CNBYTEARRAY = "org.objectweb.jorm.pobject.compositename.ACompositeBytearrayId";
    class BytearrayIdPNG implements CompositeBytearrayIdPNG {
        public byte[] val;

        public BytearrayIdPNG(byte[] val) {
            this.val = val;
        }

        public byte[] pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testBytearray() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        logger.log(BasicLevel.DEBUG, "testBytearray begin");
        int f = getF1();
        byte[] val = "azerty".getBytes();
        NameDefAccessor acc2 = new NameDefAccessor();
        BytearrayIdPNG png = new BytearrayIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNBYTEARRAY, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
//TODO: support direct encoding for the composite names which have only one byte[] field
        //assertEquals("Bad pname encoded value", png.val, pb.getPName().encode());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeBytearrayIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testBytearray end");
    }

    private final static String CNCHARARRAY = "org.objectweb.jorm.pobject.compositename.ACompositeChararrayId";
    class ChararrayIdPNG implements CompositeChararrayIdPNG {
        public char[] val;

        public ChararrayIdPNG(char[] val) {
            this.val = val;
        }

        public char[] pnGetCnf1(Object ctx) throws PException {
            return val;
        }
    }
    public void testChararray() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        logger.log(BasicLevel.DEBUG, "testChararray begin");
        int f = getF1();
        char[] val = new char[]{'a', 'z', 'e', 'r', 't', 'y'};
        NameDefAccessor acc2 = new NameDefAccessor();
        ChararrayIdPNG png = new ChararrayIdPNG(val);
        logger.log(BasicLevel.DEBUG, "export");
        PBinding pb = export(CNCHARARRAY, png);
        NameDefAccessor acc1 = new NameDefAccessor(f, val,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value",
            png.val, pb.getPName().encodeCharArray());
        assertEquals("Bad pname encoded value",
            png.val, ((CompositeChararrayIdPNG) pb.getPName().encodeAbstract()).pnGetCnf1(null));
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f, val,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testChararray end");
    }
}