/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.verifier;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.lib.JormManager;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.util.lib.BasicClassLoader;
import org.objectweb.jorm.util.lib.CompilerLogger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.jorm.compiler.lib.Compiler;
import org.objectweb.jorm.cmdline.api.CmdLineParser;

import java.util.Vector;

/**
 * The VerifierCentral testcase verifies that the parser module works without
 * Value or Class Mapping, but the Verifier module must verify all of these:<ul>
 * <li>RefGenClass (defined by a RefClass) without ClassMapping
 * <li>Class without ClassMapping
 * <li>PrimitiveType without ValueMapping
 * <li>RefGenClass (defined by a PrimitiveType) without ClassMapping
 * </ul>
 * @author	X. Spengler
 */
public class VerifierCentral extends TestCase {

    /**
     * prefix defines the path to the persistence description files
     */
    private static String prefix =
            "test/deviance/org/objectweb/jorm/verifier/";

    /**
     * properties defines the name of the jorm properties file
     */
    private static String properties =
            prefix + "jorm.properties";

    /**
     * mapperOption defines the name of the class which manages the mapper
     * option
     */
    private static String mapperOption =
            "org.objectweb.jorm.cmdline.rdb.RdbMapperOption";

    /**
     * dbprop defines the name of the property which manages the name of the
     * database
     */
    private static String dbprop = "DBName";

    /**
     * dbname defines the name of the database
     */
    private static String dbname = "postgres";

	/**
	 * Builds a new VerifierCentral testcase
	 */
	public VerifierCentral(String name) {
		super(name);
	}

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * Sets up the current test
	 */
	protected void setUp() {
	}

	/**
	 * Creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(VerifierCentral.class);
	}

    /**
     * Tests when a RefGenClass does not define a ClassMapping.
     * The parser does not return an error, but the Verifer module must verify
     * this section.
     */
	public void testRefGenClass_RefClass_WithoutClassMapping() {
        Compiler comp = new Compiler(properties);
        comp.addInputFileName(prefix+"RefGenClass_RefClass.pd");
        comp.addInputFileName(prefix+"aRefClass.pd");
        try {
            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
        } catch (Exception e) {

        }

        try {
        assertTrue("a RefGenClass without a ClassMapping (for rdb) does not work",
                comp.execute());
        } catch (Exception e) {
        }
    }

    /**
     * Tests when a Class does not define a ClassMapping.
     * The parser does not return an error, but the Verifer module must verify
     * this section.
     */
    public void testClassWithoutClassMapping() {
        Compiler comp = new Compiler(properties);
        comp.addInputFileName(prefix+"WithoutClassMapping.pd");
        try {
            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
        } catch (Exception e) {

        }

        assertTrue("Without ClassMapping does not work", comp.execute());
    }

    /**
     * Tests when a PrimitiveType does not define a ValueMapping.
     * The parser does not return an error, but the Verifer module must verify
     * this section.
     */
    public void testPrimitiveWithoutMapping() {
        Compiler comp = new Compiler(properties);
        comp.addInputFileName(prefix+"PrimitiveWithoutMapping.pd");
        try {
            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
        } catch (Exception e) {

        }

        assertTrue("A PrimitiveType without ValueMapping does not work",
                comp.execute());
    }

    /**
     * Tests when a RefGenClass does not define a ClassMapping.
     * The parser does not return an error, but the Verifer module must verify
     * this section.
     * A Field is defined by a RefGenClass which is defined by a PrimitiveType,
     * the RefGenClass does not declare the corresponding ClassMapping.
     */
    public void testRefGenClass_Primitive_WithoutClassMapping() {
        Compiler comp = new Compiler(properties);
        comp.addInputFileName(prefix+"RefGenClass_Primitive.pd");
        try {
            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
        } catch (Exception e) {

        }

        assertTrue("A RefGenClass without a ClassMapping (for rdb) does not work",
                comp.execute());
    }
}
