/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.xml2mi.lib;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.lib.JormManager;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.util.lib.BasicClassLoader;
import org.objectweb.jorm.util.lib.CompilerLogger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.jorm.compiler.lib.Compiler;
import org.objectweb.jorm.cmdline.api.CmdLineParser;

import java.util.Vector;

/**
 * The EmptyObject testcase tests cases when object are empty but works for
 * the dtd parsing. In a first time, all these tests fail, because the verifier
 * is not yet implemented. When this part will be done all these tests must
 * work well.
 * Tested objects are :<ul>
 * <li>Class
 * <li>GenClass
 * <li>PrimitiveField
 * <li>a Mapping without Rdb part
 * <li>an Rdb Mapping without Column object
 * <li>NameDef
 * <li>NameDef and ClassMapping
 * <li>Package
 * <li>ScalarField without Mapping
 * </ul>
 * @author	X. Spengler
 */
public class EmptyObject extends TestCase {

    /**
     * prefix defines the path to the persistence description files
     */
    private static String prefix =
            "test/deviance/org/objectweb/jorm/parser/lib/";

    /**
     * properties defines the name of the jorm properties file
     */
    private static String properties =
            prefix + "jorm.properties";

    /**
     * mapperOption defines the name of the class which manages the mapper
     * option
     */
    private static String mapperOption =
            "org.objectweb.jorm.cmdline.rdb.RdbMapperOption";

    /**
     * dbprop defines the name of the property which manages the name of the
     * database
     */
    private static String dbprop = "DBName";

    /**
     * dbname defines the name of the database
     */
    private static String dbname = "postgres";

    /**
     * Builds a new EmptyObject testcase object
     */
    public EmptyObject(String name) {
        super(name);
    }

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * Sets up the current test.
     */
    protected void setUp() {
    }

    /**
     * Creates a TestSuite object with the current tests
     */
    public static Test suite() {
        return new TestSuite(EmptyObject.class);
    }

    /**
     * Tests when the Class is empty, just the declaration.
     */
    public void testEmptyClass() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"EmptyClass.pd");
            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("an empty Class does not work", comp.execute());
        } catch (Exception e) {

        }


    }

    /**
     * Tests when the GenClass is empty, just the declaration.
     */
    public void testEmptyGenClass() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"EmptyGenClass.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("an empty GenClass does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when the PrimitiveType is empty, just the definition of the class,
     * and the declaration of the field.
     */
    public void testEmptyPrimitiveType() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"EmptyPrimitiveType.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("an empty PrimitiveType does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a Mapping is created without reference to the name of the
     * mapping (rdb).
     * The parser does not return an error, but the verifier must verify this
     * section. A Class is created, a Field is declared, and a PrimitiveType is
     * created with only the declaration of the ValueMapping.
     */
    public void testMappingWithoutRdb() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"MappingWithoutRdb.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("A ValueMapping without RdbValueMapping does not work",
                    comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a rdbMapping is created without the corresponding Column tag.
     * The parser does not return an error, but the Verifier module must verify
     * this section.
     * A Class is created, a Field is declared with a PrimitiveType, a
     * ValueMapping is associated with a RdbValueMapping but no ColumnSpec.
     */
    public void testRdbMappingWithoutColumn() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"RdbMappingWithoutColumn.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("An RdbValueMapping without RdbColumnSpec does not work",
                    comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when no NameDef object are declared for a Class.
     * The parser does not return an error, but the Verifier must verify this
     * section.
     */
    public void testWithoutNameDef() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"WithoutNameDef.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Without NameDef does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when no NameDef and ClassMapping are declared for a given Class.
     * The parser does not return an error, but the Verifier must verify this
     * section.
     */
    public void testWithoutNameDefClassMapping() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"WithoutNameDefClassMapping.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Without NameDef and ClassMapping does not work",
                    comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when the Package is not specified in the persistence description
     * file.
     */
    public void testWithoutSchema() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"WithoutSchema.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Without Package does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a GenClass is defined by a ScalarField and the corresponding
     * mapping is not specified.
     * The parser module does not return an error, but the verifier has to
     * verify it.
     */
    public void testScalarWithoutMapping() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"ScalarWithoutMapping.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("ScalarField Without ValueMapping does not work", comp.execute());
        } catch (Exception e) {

        }
    }
}
