/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.xml2mi.lib;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.lib.JormManager;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.util.lib.BasicClassLoader;
import org.objectweb.jorm.util.lib.CompilerLogger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.jorm.compiler.lib.Compiler;
import org.objectweb.jorm.cmdline.api.CmdLineParser;

import java.util.Vector;

/**
 * MultipleObject testcase verifies the behavior when many objects from the
 * same type are defined more than one time: <ul>
 * <li>more than one Class
 * <li>more than one GenClass
 * <li>more than one Extension on Class object
 * <li>more than one PrimitiveType on Field object
 * </ul>
 * @author	X. Spengler
 */
public class MultipleObject extends TestCase {

    /**
     * prefix defines the path to the persistence description files
     */
    private static String prefix =
            "test/deviance/org/objectweb/jorm/parser/lib/";

    /**
     * properties defines the name of the jorm properties file
     */
    private static String properties =
            prefix + "jorm.properties";

    /**
     * mapperOption defines the name of the class which manages the mapper
     * option
     */
    private static String mapperOption =
            "org.objectweb.jorm.cmdline.rdb.RdbMapperOption";

    /**
     * dbprop defines the name of the property which manages the name of the
     * database
     */
    private static String dbprop = "DBName";

    /**
     * dbname defines the name of the database
     */
    private static String dbname = "postgres";

    /**
     * Builds a new MultipleObject testcase
     */
    public MultipleObject(String name) {
        super(name);
    }

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * Sets up the current test
     */
    protected void setUp() {
    }

    /**
     * Creates a TestSuite object with the current tests
     */
    public static Test suite() {
        return new TestSuite(MultipleObject.class);
    }

    /**
     * Tests when a second Class is defined after a parsed Class
     */
    public void testMultipleClass() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"MultipleClass.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Multiple Classes does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a second GenClass object is defined after a parsed GenClass
     */
    public void testMultipleGenClass() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"MultipleGenClass.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Multiple GenClasses does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a lot of extensions are defined for a given Class object
     */
    public void testMultipleExtension() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"MultipleExtension.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Multiple Extension does not work", comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a second PrimitiveType object is defined after a parsed
     * PrimitiveType object
     */
    public void testMultiplePrimitive() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"MultiplePrimitive.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("Multiple Primitive does not work", comp.execute());
        } catch (Exception e) {

        }
    }
}
