/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.xml2mi.lib;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.lib.JormManager;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.util.lib.BasicClassLoader;
import org.objectweb.jorm.util.lib.CompilerLogger;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.jorm.compiler.lib.Compiler;
import org.objectweb.jorm.cmdline.api.CmdLineParser;

import java.util.Vector;
import java.util.Properties;
import java.io.FileInputStream;

/**
 * SameObject testcase test cases where objects are defined two times.
 * This testcase concerns:<ul>
 * <li>two PrimitiveType objects
 * <li>two PrimitiveType objects (just identical by their Names)
 * <li>two NameDef objects in the same Class
 * <li>two ScalarField in the same GenClass
 * </ul>
 * @author	X. Spengler
 */
public class SameObject extends TestCase {

    /**
     * prefix defines the path to the persistence description files
     */
    private static String prefix =
            "test/deviance/org/objectweb/jorm/parser/lib/";

    /**
     * properties defines the name of the jorm properties file
     */
    private static String properties =
            prefix + "jorm.properties";

    /**
     * mapperOption defines the name of the class which manages the mapper
     * option
     */
    private static String mapperOption =
            "org.objectweb.jorm.cmdline.rdb.RdbMapperOption";

    /**
     * dbprop defines the name of the property which manages the name of the
     * database
     */
    private static String dbprop = "DBName";

    /**
     * dbname defines the name of the database
     */
    private static String dbname = "postgres";

    /**
     * Builds a new SameObject testcase
     */
    public SameObject(String name) {
        super(name);
    }

    /**
     * main method to launch the tests manually
     */
    public static void main(String[] args) {
        TestRunner.run(suite());
    }

    /**
     * Sets up the current test
     */
    protected void setUp() {
    }

    /**
     * Creates a TestSuite object with the current tests
     */
    public static Test suite() {
        return new TestSuite(SameObject.class);
    }

    /**
     * Tests When the same PrimitiveType is defined more than one time.
     */
    public void test2Primitive() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"2Primitive.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);
            Logger logger = null;
            try {
                Properties p = new Properties();
                p.load(new FileInputStream("config/logSystem.properties"));
                CompilerLogger.configure(p);
                logger =
                        CompilerLogger.getLoggerFactory().
                        getLogger("org.objectweb.jorm.compiler");

            } catch (Exception e) {}

            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("2 identical primitive type must be catched",
                    comp.execute());
        } catch (Exception e) {

        }

    }

    /**
     * Tests when the same PrimitiveType is defined more than one time but the
     * two objects just have the name in common.
     */
    public void testNot2Primitive() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"Not2Primitive.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);

            Logger logger = null;
            try {
                Properties p = new Properties();
                p.load(new FileInputStream("config/logSystem.properties"));
                CompilerLogger.configure(p);
                logger =
                        CompilerLogger.getLoggerFactory().
                        getLogger("org.objectweb.jorm.compiler");

            } catch (Exception e) {}

            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);

            assertTrue("2 identical primitive name type must be catched",
                    comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when the same NameDef object is defined more than one time for a
     * given Class object.
     */
    public void testSameNameDef() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"SameNameDef.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);

            Logger logger = null;
            try {
                Properties p = new Properties();
                p.load(new FileInputStream("config/logSystem.properties"));
                CompilerLogger.configure(p);
                logger =
                        CompilerLogger.getLoggerFactory().
                        getLogger("org.objectweb.jorm.compiler");

            } catch (Exception e) {}

            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("2 identical NameDef must be catched",
                    comp.execute());
        } catch (Exception e) {

        }
    }

    /**
     * Tests when a GenClass object defines more than one time a ScalarField
     * objects.
     */
    public void test2ScalarField() {
        try {
            Compiler comp = new Compiler(properties,"config/logSystem.properties");
            comp.addInputFileName(prefix+"2ScalarField.pd");

            CmdLineParser clp = (CmdLineParser)
                    comp.getClassLoader().load(mapperOption);

            Logger logger = null;
            try {
                Properties p = new Properties();
                p.load(new FileInputStream("config/logSystem.properties"));
                CompilerLogger.configure(p);
                logger =
                        CompilerLogger.getLoggerFactory().
                        getLogger("org.objectweb.jorm.compiler");

            } catch (Exception e) {}

            comp.getCompilerParameter().addCPExtension(
                    mapperOption,clp);

            clp.parseProperty(dbprop, dbname);
            assertTrue("2 identical ScalarField must be catched",
                    comp.execute());

        } catch (Exception e) {

        }
    }
}
